#!BPY
"""Registration info for Blender menus:
Name: 'Render with Indigo...'
Blender: 248
Group: 'Render'
Tooltip: 'material editor and exporter for Indigo v2.2.2'
"""
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# --------------------------------------------------------------------------
# BLENDIGO
# --------------------------------------------------------------------------
#
# Authors:
# * Indigo exporter - Nick Chapman, U3dreal, Zuegs, Ewout Fernhout, Leope, psor, Wedge, SmartDen, BIGFAN(BF), DaveC
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

blendigoVersion = "2.2.2"

######################################################
# Importing modules
######################################################



import math
import os
import sys
import copy
import array
import Blender
from Blender import Mesh, Scene, Object, Material, Texture, Window, Draw, BGL, Mathutils, Text
from Blender.sys import *

# Import the blendigo modules

modulesPath = os.path.join(Blender.Get(u"scriptsdir"), "blendigopkg")
assetsPath = os.path.join(Blender.Get(u"scriptsdir"), "blendigopkg", "assets")

sys.path.append(os.path.join(Blender.Get("uscriptsdir")))

#try:
from blendigopkg import *
#except ImportError:
#	Draw.PupMenu("Error%nBroken installation of Blendigo, please reinstall%t|Abort")

# Load the indigo logo..
indigoLogo = Blender.Image.Load(os.path.join(assetsPath, "indigo_logo.png"))

def drawImage(img,x,y,dx,dy):
	"""
	img =  objetc loaded with Blender.Image.Load() function
	x,y =   location in the script window from the left bottom corner
	dx,dy = scaling the image size dx for width and dy for height
	"""
	
	BGL.glEnable(BGL.GL_TEXTURE_2D)
	BGL.glColor4f(1,1,1,0)
	
	BGL.glBegin(BGL.GL_QUADS)
	BGL.glBindTexture(BGL.GL_TEXTURE_2D, img.getBindCode())
	BGL.glTexCoord2f(0.0,0.0)
	BGL.glVertex3f(float(x),float(y),0.0)
	BGL.glTexCoord2f(1.0,0.0)
	BGL.glVertex3f(float(x+img.getSize()[0]*dx),float(y),0.0)
	BGL.glTexCoord2f(1.0,1.0)
	BGL.glVertex3f(float(x+img.getSize()[0]*dx),float(y+img.getSize()[1]*dy),0.0)
	BGL.glTexCoord2f(0.0,1.0)
	BGL.glVertex3f(float(x),float(y+img.getSize()[1]*dy),0.0 )
	BGL.glEnd()
	
	BGL.glDisable(BGL.GL_TEXTURE_2D)


currentMaterial = None
class IndigoMaterial:
	def __init__(self, blenderMaterial):
		# if blenderMaterial:
		# try:
		# 	self.properties = blenderMaterial.properties['indigo']
		# except KeyError:
		# 	print 'WARNING: material "%s" has no Indigo material definitions - converting blender material'%(mat.getName())
		# 
		# 	self.properties = {}
		# 	convertMaterial(blenderMaterial)
		# 	self.properties = mat.properties['indigo'].convert_to_pyobject()
		# 
		self.blenderMaterial = blenderMaterial

		self.properties = {}
		
		if "indigo" not in self.blenderMaterial.properties:
			print 'WARNING: material "%s" has no Indigo material definitions - converting blender material'%(self.blenderMaterial.getName())
			convertMaterial(self.blenderMaterial)
			
		self.properties = self.blenderMaterial.properties['indigo'].convert_to_pyobject()

		self.albedo = Channel()
		self.albedo.enabled.val = True
		
		self.displacement = Channel()
		self.displacement.type.val = 1
		self.displacement.enabled.val = (self.read('displacement.path') != '')

		self.bump = Channel()
		self.bump.type.val = 1
		self.bump.enabled.val = (self.read('bump.path') != '')

		self.exponent = Channel()
		self.exponent.type.val = 1
		self.exponent.enabled.val = (self.read('exponent.path') != '')

	def read(self, name):
		if name in self.properties:
			return self.properties[name]
		else:
			return ""


class Channel:
	def __init__(self):
		self.enabled = Draw.Create(False)
		self.type = Draw.Create(False)
	def typeShader(self):
		return self.type==2
	def typeTexture(self):
		return self.type==1
	def typeColor(self):
		return self.type==0

def setCurrentMaterial(material):
	global currentMaterial
	currentMaterial = IndigoMaterial(material)

# Adapted from luxblend.
#
#
class guiLayout:
    def __init__(self, y=-1):
        global ScrollY
        self.x = 120 # left start position after captions
        self.xmax = 110+2*(140+4)
        
        if(y==-1):
            self.y = Window.GetAreaSize()[1] - 70 + ScrollY
        else:
            self.y = y
        
        self.w = 140 # default element width in pixels
        self.h = 18  # default element height in pixels
        self.hmax = 0
        self.xgap = 4
        self.ygap = 4
        self.resethmax = False
        self.left = 15
        self.control = 110
        self.compact = False
		
    def units(self, unit, controlWidth=0):
    	self.x += controlWidth + 5
    	BGL.glColor3f(0.5,0.5,0.5);
    	self.text(unit)
    	BGL.glColor3f(0.9,0.9,0.9) ;

    def text(self, text):
    	BGL.glRasterPos2i(self.x,self.y+5);
    	Draw.Text(text, 'small')
    	self.x += Draw.GetStringWidth(text) + 5

    def title(self, title):
    	BGL.glColor3f(0.6,0.6,0.6) ;
    	BGL.glRectf(10,self.y,Draw.GetStringWidth(title)+20,self.y-30)
    	
    	BGL.glColor3f(0.1,0.1,0.1) ;
    	BGL.glRectf(12,self.y,Draw.GetStringWidth(title)+18,self.y-28)

    	BGL.glColor3f(0.2,0.2,0.2)
    	BGL.glRasterPos2i(self.left+1,self.y-21);
    	Draw.Text(title)
    	
    	BGL.glColor3f(1,1,1)
    	BGL.glRasterPos2i(self.left,self.y-20);
    	Draw.Text(title)
    	
    	self.newline()
    	self.y -= 10

    def subtitle(self, title):
       	self.y -= 15
       	
       	BGL.glColor3f(0.4,0.4,0.4) ;
       	BGL.glRectf(10,self.y,Draw.GetStringWidth(title)+20,self.y-25)
       	BGL.glRectf(0,self.y-23,460,self.y-25)
       	
       	BGL.glColor3f(0.2,0.2,0.2)
       	BGL.glRasterPos2i(self.left+1,self.y-15);
       	Draw.Text(title)
       	
       	BGL.glColor3f(1,1,1)
       	BGL.glRasterPos2i(self.left,self.y-14);
       	Draw.Text(title)
       	
       	self.newline()

       	if self.compact:
       		self.y -= 8
       	else:
       		self.y -= 5

    def reset(self):
    	BGL.glColor3f(0.9,0.9,0.9) ;
    
    def newline(self, title="", distance=0, level=0, icon=None, color=None):
       	if self.compact:
       	    self.y -= 18
       	else:
       	    self.y -= 21
       	    
       	self.x = 120
       	
       	self.reset()

       	if title!="":
       		BGL.glRasterPos2i(self.left,self.y + 5)
       		Draw.Text(title, 'small')
		
		self.reset()

		# return
		#         # self.x = 110
		#
		#         if not(self.resethmax): self.y -= int(self.hmax + distance)
		#         if color!=None:
		# 	BGL.glColor3f(color[0],color[1],color[2]);
		# 	BGL.glRectf(0,self.y-self.hmax,self.xmax,self.y+distance);
		# 	BGL.glColor3f(0.9, 0.9, 0.9)
		#         if icon!=None: drawIcon(icon, 2+level*10, self.y-16)
		#         self.resethmax = True

########################################################################
########################################################################
### INDIGO material editor submodule - (by zuegs)
### ----------------------------------------------------------------

### Setup constants
evtMatBase = 100 # first event-number for material editor module

### Materialtype numbers
mLight = -1
mDiffuse = 0
mPhong = 1
mSpecular = 2
mBlend = 3
mExitPortal = 4
mInclude = 9

selectedSection = "camera"

### Event-numbers
evtNoEvt = evtMatBase + 0
evtMatsMenu = evtMatBase + 1
evtMatEmittIES = evtMatBase + 9
evtMatEmittTypeRGB = evtMatBase + 36
evtMatEmittTypeBB = evtMatBase + 37
evtMatType = evtMatBase + 10
evtMatEmittScale = evtMatBase + 11
evtMatEmittIESPath = evtMatBase + 12
evtMatCopy = evtMatBase + 14
evtMatPaste = evtMatBase + 15
evtMatRgbPicker = evtMatBase + 20
evtMatEmittRgbPicker = evtMatBase + 17
evtMatRgbPicker_s = evtMatBase + 16
evtMatBlendMatA = evtMatBase + 21
evtMatBlendMatB = evtMatBase + 22
evtMatAlbedoPath = evtMatBase + 23
evtMatAlbedoISL = evtMatBase + 18
evtMatBumpPath = evtMatBase + 24
evtMatExponentPath = evtMatBase + 25
evtMatBlendPath = evtMatBase + 26
evtMatSssRgbPicker = evtMatBase + 27
evtMatPreview = evtMatBase + 28
evtMatInclude = evtMatBase + 29
evtMatAbsPicker = evtMatBase + 30
evtPreviewScene = evtMatBase + 31
evtMatConvert = evtMatBase + 33
evtMatSave = evtMatBase + 34
evtDisplaceImage = evtMatBase + 35

evtNetwork=200


### Globals
activeObject = None
activeMaterial = None
activeObjectMat0 = None
matsMenu = Draw.Create(0)
matClipboard = None
matPreviewFile  = "IMmatdb.xml"
ShaderList = []

ScrollY=0
ScrollAreaY = 485

### GUI Globals
rendererAdvanced = Draw.Create(0)
matGui = Draw.Create(0)
matType = 1   # -1:light 0:diffuse 1:phong 2:specular 3:Blend 9:XML
matTypeMenu = Draw.Create(1) #workaround for buggy GUI
matDiffTrans = Draw.Create(0)
matEmitt = Draw.Create(0)
matEmittScale = Draw.Create(1)
matEmittIES=Draw.Create(0)
matEmittIESPath = Draw.Create('')
matEmittScaleValue = Draw.Create(1500)
matEmittTypeRGB=Draw.Create(1)
matEmittTypeBB=Draw.Create(0)
matEmittScaleMeasure = Draw.Create(0)
matEmittGain = Draw.Create(1)
matEmittMult = Draw.Create(0)
matEmittRgb = Draw.Create(1.0,1.0,1.0)
matEmittRgbR = Draw.Create(1.0)
matEmittRgbG = Draw.Create(1.0)
matEmittRgbB = Draw.Create(1.0)
matEmittLayer = Draw.Create(1)
matEmittTemp = Draw.Create(5000.0)
matTogglePhong=Draw.Create(0)
matRgb = Draw.Create(0.8, 0.8, 0.8)
matRgbR = Draw.Create(0.8)
matRgbG = Draw.Create(0.8)
matRgbB = Draw.Create(0.8)
matRgb_s = Draw.Create(0.8, 0.8, 0.8)
matRgbR_s = Draw.Create(0.8)
matRgbG_s = Draw.Create(0.8)
matRgbB_s = Draw.Create(0.8)

matRgbGain = Draw.Create(1.0)
matIOR = Draw.Create(1.0)
matExponent = Draw.Create(100)
matNKData = Draw.Create('')
matSigma = Draw.Create(0.2)
matTransparent = Draw.Create(0)
matSpecularType = Draw.Create(1) # 1:specular 2:glossy_transparent
matPrecedence = Draw.Create(10)
matMediumType = Draw.Create(1) # 1:basic
matCauchyBCoeff = Draw.Create(0.0)
matAbsorptionType = Draw.Create(1) # 1:RGB
matAbs = Draw.Create(0.5, 0.5, 0.5)
matAbsR = Draw.Create(0.0)
matAbsG = Draw.Create(0.0)
matAbsB = Draw.Create(0.0)
matAbsGain = Draw.Create(1.0)
matSss = Draw.Create(0)
matSssType = Draw.Create(2) # 1:RGB
matSssR = Draw.Create(0.8)
matSssG = Draw.Create(0.8)
matSssB = Draw.Create(0.8)
matSssRgb = Draw.Create(0.8, 0.8, 0.8)
matSssGain = Draw.Create(1.0)
matSssUniform = Draw.Create(1.0)
matSssUniformExp = Draw.Create(0.0)
matSssPhaseType = Draw.Create(1)
matSssPhaseSpec=Draw.Create(1)
matSssPhaseSpecUni = Draw.Create(0.5)
matSssPhaseSpecR=Draw.Create(0.5)
matSssPhaseSpecG=Draw.Create(0.5)
matSssPhaseSpecB=Draw.Create(0.5)
matAlbedoUVset = Draw.Create('UVTex')
matAlbedoPath = Draw.Create('')
matAlbedoISL = Draw.Create(0)
matBumpISL=Draw.Create(0)
matExponentISL=Draw.Create(0)
matDisplaceISL=Draw.Create(0)
matAlbedoShader = Draw.Create(0)
matBumpShader=Draw.Create(0)
matExponentShader=Draw.Create(0)
matDisplaceShader=Draw.Create(0)
matAlbedoExponent = Draw.Create(2.2)
matAlbedoA = Draw.Create(0.0)
matAlbedoB = Draw.Create(1.0)
matAlbedoC = Draw.Create(0.0)
matBumpUVset = Draw.Create('UVTex')
matBumpPath = Draw.Create('')
matBumpExponent = Draw.Create(2.2)
matBumpA = Draw.Create(0.0)
matBumpB = Draw.Create(0.01)
matBumpC = Draw.Create(0.0)
matExponentUVset = Draw.Create('UVTex')
matExponentPath = Draw.Create('')
matExponentExponent = Draw.Create(2.2)
matExponentA = Draw.Create(0.0)
matExponentB = Draw.Create(10000)
matExponentC = Draw.Create(1.0)
matBlendMatA = Draw.Create('')
matBlendMatB = Draw.Create('')
matBlendFactor = Draw.Create(0.5)
matBlendUVset = Draw.Create('UVTex')
matBlendPath = Draw.Create('')
matBlendExponent = Draw.Create(1.0)
matBlendA = Draw.Create(0.0)
matBlendB = Draw.Create(1.0)
matBlendC = Draw.Create(1.0)
matInclude = Draw.Create('')
PreviewScene = Draw.Create(3)
matHemoglobin = Draw.Create(0.001)
matMelanin = Draw.Create(0.15)
matEumelanin = Draw.Create(0.001)
matTurbidity=Draw.Create(2.2)
matMediumType = Draw.Create(1)
matDisplaceType=Draw.Create(0)
matDisplacePath=Draw.Create('')
matDisplaceExp=Draw.Create(2.2)
matDisplaceUVSet=Draw.Create('UVTex')
matDisplaceA=Draw.Create(0.0)
matDisplaceB=Draw.Create(0.01)
matDisplaceC=Draw.Create(0)
matDisplaceFBMUScale=Draw.Create(1.00)
matDisplaceFBMVScale=Draw.Create(1.00)

meshGui = Draw.Create(0)

meshSubdivide=Draw.Create(0)
MaxNumSubdiv = Draw.Create(2)
SubdivPixelTreshold=Draw.Create(0.0)
SubdivCurvatureTreshold=Draw.Create(0.1)
SubdivSmooth = Draw.Create(1)

subdivMeshes={}

### ================================================================
### relpath(base, target) -> relative path
###		returns the relative path between base and target path
### todo: this propabely don't work for linux yet ;(
### ----------------------------------------------------------------
def relpath(base,target):
	return utils.getRelativePath(base, target)

### ===Changed code region 2/2===04-Nov-2008 Hans.P.G.==============
def normcase2(text): # if win(?), don't call normcase, but only replace / with \
    if os.path.normcase("/") == "\\":
        return text.replace("/", "\\")
    else:
        return os.path.normcase(text)

### ================================================================
### convertMaterial(material)
###		converts a blender-material-to an indigo-material
### ----------------------------------------------------------------
def convertMaterial(mat):
	d = {}
	transparent = (mat.mode & Material.Modes.RAYTRANSP) or (mat.getAlpha() < 1.0)
	if transparent or mat.enableSSS:
		# SPECULAR
		d['type'] = 2
		d['precedence'] = 10
		d['cauchy_b_coeff'] = 0.0
		d['specular.type'] = 1
		if transparent:
			d['transparent'] = 1
			d['ior'] = mat.getIOR()
		else:
			d['transparent'] = 0
			d['ior'] = 1.0/(1.001-mat.getRayMirr())
		if d['ior'] < 1.01:  # as values equal or smaller than 1.0 gives unexpected results
			d['ior'] = 1.01
		d['abs.r'] = 1.0-mat.R  # absorption is the inverted color!
		d['abs.g'] = 1.0-mat.G
		d['abs.b'] = 1.0-mat.B
		d['abs.gain'] = mat.getFilter()*100
		if mat.getFilter() == 0:
			d['abs.gain'] = 1
		if mat.enableSSS: #SmartDen
			d['sss'] = 1
			d['sss.r'] = mat.sssR
			d['sss.g'] = mat.sssG
			d['sss.b'] = mat.sssB
		if mat.getAlpha() == 0:
			d['type'] = 4
	#elif transparent and (mat.getAlpha() == 0):
	#	d['type'] = 4
	elif (not transparent and mat.getIOR() > 1) or mat.getSpec() > 0:
		specfac = mat.getSpec()
		if specfac > 0.0001:
			# PHONG
			d['type'] = 1
			d['rgb.r'] = mat.R
			d['rgb.g'] = mat.G
			d['rgb.b'] = mat.B
			d['rgb.gain'] = mat.getRef()
			d['ior'] = 1.0+math.exp(mat.getSpec())/2.71
			d['exponent'] = mat.hard*10
	elif mat.getIOR() == 1:
		# DIFFUSE
		if mat.getDiffuseShader() == Material.Shaders.DIFFUSE_ORENNAYAR:
			d['type'] = 5
			d['sigma'] = mat.getRoughness()
		else:
			d['type'] = 0
			if transparent:
				d['diffuse.transmitter'] = 1
		d['rgb.r'] = mat.R
		d['rgb.g'] = mat.G
		d['rgb.b'] = mat.B
		d['rgb.gain'] = mat.getRef()
	if mat.getEmit() > 0:
		d['emitter']=1
	enabledTextures=mat.enabledTextures
	textures=mat.getTextures()
	for mtex_index in enabledTextures:
		mtex=textures[mtex_index]
		if (mtex) and (mtex.texco & Blender.Texture.TexCo['UV']) and (mtex.tex) and (mtex.tex.type == Blender.Texture.Types.IMAGE) and (mtex.tex.image):
			if mtex.mapto & Blender.Texture.MapTo['COL']:
				uvlayer=mtex.uvlayer
				obj = Blender.Scene.GetCurrent().objects.active
				if uvlayer == '':
					try:
						uvlayer=obj.getData(mesh=1).activeUVLayer
					except:
						uvlayer='uv'
				
				d['albedo.path'] = mtex.tex.image.filename
				d['albedo.uv_set'] = uvlayer
				d['albedo.exponent'] = 2.2
				d['albedo.a'] = 0.0
				d['albedo.b'] = mtex.colfac
				d['albedo.c'] = 0.0
			if mtex.mapto & Blender.Texture.MapTo['NOR']:
				d['bump.path'] = mtex.tex.image.filename
				uvlayer=mtex.uvlayer
				if uvlayer == '':
					obj = Blender.Scene.GetCurrent().objects.active
					uvlayer=obj.getData(mesh=1).activeUVLayer
				d['bump.uv_set'] = uvlayer
				d['bump.exponent'] = 2.2
				d['bump.a'] = 0.0
				d['bump.b'] = mtex.norfac * 0.01
				d['bump.c'] = 0.0
			if mtex.mapto & Blender.Texture.MapTo['DISP']:
				d['displace.path'] = mtex.tex.image.filename
				uvlayer=mtex.uvlayer
				if uvlayer == '':
					obj = Blender.Scene.GetCurrent().objects.active
					uvlayer=obj.getData(mesh=1).activeUVLayer
				d['displace.uv_set'] = uvlayer
				d['displace.exponent'] = 2.2
				d['displace.a'] = 0.0
				d['displace.b'] = mtex.dispfac * 0.01
				d['displace.c'] = 0.0
			if mtex.mapto & Blender.Texture.MapTo['SPEC']:
				d['exponent.path'] = mtex.tex.image.filename
				uvlayer=mtex.uvlayer
				if uvlayer == '':
					obj = Blender.Scene.GetCurrent().objects.active
					uvlayer=obj.getData(mesh=1).activeUVLayer
				d['exponent.uv_set'] = uvlayer
				d['exponent.exponent'] = 2.2
				d['exponent.a'] = 0.0
				d['exponent.b'] = 1000
				d['exponent.c'] = 0.0
	mat.properties['indigo'] = d
	drawGUI()


### ================================================================
### readWriteMaterial(material, write)
###		reads or writes material-data from or to blender material property
### ----------------------------------------------------------------
def readWriteValue(dict, name, draw, write, default):
	if write == 0:
		try:
			try:
				draw.val = dict[name]
			except KeyError:
				draw.val = default
		except ValueError:
			print "ValueError: on property \"%s\""%name
	else:
		dict[name] = draw.val


def readWriteMaterial(m, write=0):
	global matsMenu, activeMaterial, matType
	global matDiffTrans, matEmitt, matEmittScale, matEmittIES, matEmittIESPath, matEmittTypeRGB, matEmittTypeBB, matEmittScaleValue, matEmittScaleMasure, matEmittGain, matEmittMult, matEmittRgb, matEmittRgbR, matEmittRgbG, matEmittRgbB, matEmittTemp, matEmittLayer
	global matTogglePhong, matRgb, matRgbR, matRgbG, matRgbB,matRgb_s, matRgbR_s, matRgbG_s, matRgbB_s, matRgbGain
	global matSigma, matIOR, matTransparent, matExponent, matNKData, matCauchyBCoeff, matPrecedence
	global matSpecularType, matMediumType, matAbsorptionType, matSss, matSssType, matSssR, matSssG, matSssB, matSssGain, matSssUniform, matSssUniformExp, matSssPhaseType, matSssFuncSpec
	global matAlbedoUVset, matAlbedoISL, matAlbedoShader, matAlbedoPath, matAlbedoExponent, matAlbedoA, matAlbedoB, matAlbedoC
	global matBumpUVset, matBumpISL, matBumpShader, matBumpPath, matBumpExponent, matBumpA, matBumpB, matBumpC
	global matExponentUVset, matExponentISL, matExponentShader, matExponentPath, matExponentExponent, matExponentA, matExponentB, matExponentC, matDosplaceISL, matDisplaceShader
	global matBlendMatA, matBlendMatB, matBlendFactor, matBlendUVset, matBlendPath, matBlendExponent, matBlendA, matBlendB, matBlendC, PreviewScene, matPreviewFile
	global matHemoglobin, matTurbidity, matMelanin, matEumelanin, matAbsR, matAbsG, matAbsB, matAbsGain #BF
	global currentMaterial
	
	if m != None:
		
		try:
			d = m.properties['indigo'].convert_to_pyobject()
		except KeyError:
			d = {}
			if write == 0:
				convertMaterial(m)
				d = m.properties['indigo'].convert_to_pyobject()
		if write == 0:
			try:
		 		matType = d['type']
			except KeyError:
				matType = 0
		else:
			d['type'] = matType
		readWriteValue(d, 'emitter', matEmitt, write, 0)
		readWriteValue(d, 'emitter.scale', matEmittScale, write, 1)
		readWriteValue(d, 'emitter.ies', matEmittIES, write, 0)
		readWriteValue(d, 'emitter.ies_path', matEmittIESPath, write, '')
		readWriteValue(d, 'emitter.type_rgb', matEmittTypeRGB, write, 1)
		readWriteValue(d, 'emitter.type_bb', matEmittTypeBB, write, 0)
		readWriteValue(d, 'emitter.temperature', matEmittTemp, write, 5000)
		readWriteValue(d, 'emitter.scale_value', matEmittScaleValue, write, 1500)
		readWriteValue(d, 'emitter.scale_measure', matEmittScaleMeasure, write, 0)
		readWriteValue(d, 'emitter.gain', matEmittGain, write, 1)
		readWriteValue(d, 'emitter.mult', matEmittMult, write, 0)
		readWriteValue(d, 'rgb.r_e', matEmittRgbR, write, 1)
		readWriteValue(d, 'rgb.g_e', matEmittRgbG, write, 1)
		readWriteValue(d, 'rgb.b_e', matEmittRgbB, write, 1)
		readWriteValue(d, 'emitter.layer', matEmittLayer, write, 1)
		readWriteValue(d, 'diffuse.transmitter', matDiffTrans, write, 0)
		readWriteValue(d, 'toggle_phong', matTogglePhong, write, 0)
		readWriteValue(d, 'rgb.r', matRgbR, write, 0.8)
		readWriteValue(d, 'rgb.g', matRgbG, write, 0.8)
		readWriteValue(d, 'rgb.b', matRgbB, write, 0.8)
		readWriteValue(d, 'rgb.r_s', matRgbR_s, write, 0.8)
		readWriteValue(d, 'rgb.g_s', matRgbG_s, write, 0.8)
		readWriteValue(d, 'rgb.b_s', matRgbB_s, write, 0.8)
		readWriteValue(d, 'rgb.gain', matRgbGain, write, 1.0)
		readWriteValue(d, 'ior', matIOR, write, 1.0)
		readWriteValue(d, 'exponent', matExponent, write, 100)
		readWriteValue(d, 'nk_data', matNKData, write, '')
		readWriteValue(d, 'sigma', matSigma, write, 0.2)
		readWriteValue(d, 'transparent', matTransparent, write, 0)
		readWriteValue(d, 'specular.type', matSpecularType, write, 1)
		readWriteValue(d, 'precedence', matPrecedence, write, 10)
		readWriteValue(d, 'medium.type', matMediumType, write, 1)
		readWriteValue(d, 'hemoglobin', matHemoglobin, write, 0.001)
		readWriteValue(d, 'turbidity', matTurbidity, write, 2.2)
		readWriteValue(d, 'melanin', matMelanin, write, 0.15)
		readWriteValue(d, 'eumelanin', matEumelanin, write, 0.001)
		readWriteValue(d, 'cauchy_b_coeff', matCauchyBCoeff, write, 0.0)
		readWriteValue(d, 'absorption.type', matAbsorptionType, write, 1)
		readWriteValue(d, 'abs.r', matAbsR, write, 0.0)
		readWriteValue(d, 'abs.g', matAbsG, write, 0.0)
		readWriteValue(d, 'abs.b', matAbsB, write, 0.0)
		readWriteValue(d, 'abs.gain', matAbsGain, write, 1)
		readWriteValue(d, 'sss', matSss, write, 0)
		readWriteValue(d, 'sss.type', matSssType, write, 2)
		readWriteValue(d, 'sss.r', matSssR, write, 1)
		readWriteValue(d, 'sss.g', matSssG, write, 1)
		readWriteValue(d, 'sss.b', matSssB, write, 1)
		readWriteValue(d, 'sss.gain', matSssGain, write, 1.0)
		readWriteValue(d, 'sss.uniform', matSssUniform, write, 1.0)
		readWriteValue(d, 'sss.uniformExp', matSssUniformExp, write, 0.0)
		readWriteValue(d,'sss.phasespec',matSssPhaseSpec,write,1)
		readWriteValue(d,'sss.phasespec_uni',matSssPhaseSpecUni,write,0.5)
		readWriteValue(d,'sss.phasespec_r',matSssPhaseSpecR,write,0.5)
		readWriteValue(d,'sss.phasespec_g',matSssPhaseSpecG,write,0.5)
		readWriteValue(d,'sss.phasespec_b',matSssPhaseSpecB,write,0.5)
		readWriteValue(d, 'sss.phasetype' , matSssPhaseType, write, 1)
		readWriteValue(d, 'albedo.uv_set', matAlbedoUVset, write, 'UVTex')
		readWriteValue(d, 'albedo.path', matAlbedoPath, write, '')
		readWriteValue(d, 'albedo.isl',matAlbedoISL, write, 0)
		readWriteValue(d, 'bump.isl', matBumpISL,write, 0)
		readWriteValue(d, 'exponent.isl', matExponentISL,write, 0)
		readWriteValue(d, 'displace.isl', matDisplaceISL,write, 0)
		readWriteValue(d, 'bump.shader', matBumpShader,write, -1)
		readWriteValue(d, 'exponent.shader', matExponentShader,write, -1)
		readWriteValue(d, 'displace.shader', matDisplaceShader,write, -1)
		readWriteValue(d, 'albedo.shader', matAlbedoShader, write, -1)
		readWriteValue(d, 'albedo.exponent', matAlbedoExponent, write, 2.2)
		readWriteValue(d, 'albedo.a', matAlbedoA, write, 0.0)
		readWriteValue(d, 'albedo.b', matAlbedoB, write, 1.0)
		readWriteValue(d, 'albedo.c', matAlbedoC, write, 0.0)
		readWriteValue(d, 'bump.uv_set', matBumpUVset, write, 'UVTex')
		readWriteValue(d, 'bump.path', matBumpPath, write, '')
		readWriteValue(d, 'bump.exponent', matBumpExponent, write, 2.2)
		readWriteValue(d, 'bump.a', matBumpA, write, 0.0)
		readWriteValue(d, 'bump.b', matBumpB, write, 0.01)
		readWriteValue(d, 'bump.c', matBumpC, write, 0.0)
		readWriteValue(d, 'exponent.uv_set', matExponentUVset, write, 'UVTex')
		readWriteValue(d, 'exponent.path', matExponentPath, write, '')
		readWriteValue(d, 'exponent.exponent', matExponentExponent, write, 2.2)
		readWriteValue(d, 'exponent.a', matExponentA, write, 0.0)
		readWriteValue(d, 'exponent.b', matExponentB, write, 10000)
		readWriteValue(d, 'exponent.c', matExponentC, write, 1.0)
		readWriteValue(d, 'blend.a_name', matBlendMatA, write, '')
		readWriteValue(d, 'blend.b_name', matBlendMatB, write, '')
		readWriteValue(d, 'blend.factor', matBlendFactor, write, 0.5)
		readWriteValue(d, 'blend.uv_set', matBlendUVset, write, 'UVTex')
		readWriteValue(d, 'blend.path', matBlendPath, write, '')
		readWriteValue(d, 'blend.exponent', matBlendExponent, write, 1.0)
		readWriteValue(d, 'blend.a', matBlendA, write, 0.0)
		readWriteValue(d, 'blend.b', matBlendB, write, 1.0)
		readWriteValue(d, 'blend.c', matBlendC, write, 0.0)
		readWriteValue(d, 'include', matInclude, write, '')
		readWriteValue(d, 'displace.path',matDisplacePath,write,'')
		readWriteValue(d, 'displace.exp',matDisplaceExp,write,2.2)
		readWriteValue(d, 'displace.uv_set',matDisplaceUVSet,write,'UVTex')
		readWriteValue(d, 'displace.a',matDisplaceA,write,0.01)
		readWriteValue(d, 'displace.b',matDisplaceB,write,0.01)
		readWriteValue(d, 'displace.c',matDisplaceC,write,0)
		if write != 0:
			m.properties['indigo'] = d


### ================================================================
### exportMaterial(material) -> string
###		returns indigo XML code for material
### ----------------------------------------------------------------
def exportMaterial(mat, file, exportedmatlist, includelist=[], newmatname='', uvset=''):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	#End of readValue
	
	texture_index=0
	if mat != None:
		try:
			d = mat.properties['indigo'].convert_to_pyobject()
		except KeyError:
			d = {}
			convertMaterial(mat)
			d = mat.properties['indigo'].convert_to_pyobject()
			print 'WARNING: material "%s" has no Indigo material definitions - converting blender material'%(mat.getName())

		print("Material dictionary:")
		print(d)

		matname = mat.getName()
		if newmatname != '':
			matname = newmatname
		exportedmatlist.append(matname)
		m = []
		if not ClayRender.val:
			matType = readValue(d, 'type', 0)
		else:
			matType = 0
		matMediumType = readValue(d, 'medium.type', 1)
		if (matType >= 0) and (matType != mInclude) and (matType != mExitPortal):
			if (matType == 2) and (matMediumType == 1):
				file.write("\t<medium>\n")
				file.write("\t\t<name>%s</name>\n"%(matname))
				file.write("\t\t<precedence>%g</precedence>\n"%(readValue(d, 'precedence', 1.0)))
				file.write("\t\t<basic>\n")
				file.write("\t\t\t<ior>%g</ior>\n"%(readValue(d, 'ior', 1.0)))
				file.write("\t\t\t<cauchy_b_coeff>%g</cauchy_b_coeff>\n"%(readValue(d, 'cauchy_b_coeff', 1.0)))
				file.write("\t\t\t<absorption_coefficient_spectrum>\n")
				if readValue(d, 'absorption.type', 1) == 1:
					gain = readValue(d, 'abs.gain', 1.0)
					file.write("\t\t\t\t<rgb>\n")
					file.write("\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n"%((readValue(d, 'abs.r', 0.0))*gain, (readValue(d, 'abs.g', 0.0))*gain, (readValue(d, 'abs.b', 0.0))*gain))
					file.write("\t\t\t\t</rgb>\n")
				else:
					file.write("\t\t\t\t<uniform>\n")
					file.write("\t\t\t\t\t<value>0</value>\n")
					file.write("\t\t\t\t</uniform>\n")
				file.write("\t\t\t</absorption_coefficient_spectrum>\n")
				if (readValue(d, 'sss', 0) != 0):
					file.write("\t\t\t<subsurface_scattering>\n")
					file.write("\t\t\t\t<scattering_coefficient_spectrum>\n")
					if (readValue(d, 'sss.type', 1) == 1):
						gain = readValue(d, 'sss.gain', 1.0)
						file.write("\t\t\t\t\t<rgb>\n")
						file.write("\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d, 'sss.r', 0.0)*gain, readValue(d, 'sss.g', 0.0)*gain, readValue(d, 'sss.b', 0.0)*gain))
						file.write("\t\t\t\t\t\t<gamma>2.2</gamma>\n")
						file.write("\t\t\t\t\t</rgb>\n")
					if (readValue(d,'sss.type',1) == 2):
						file.write("\t\t\t\t\t<uniform>\n")
						file.write("\t\t\t\t\t\t<value>%g</value>\n"%(readValue(d,'sss.uniform',0.0)*(10**readValue(d, 'sss.uniformExp',0.0))))
						file.write("\t\t\t\t\t</uniform>\n")
					file.write("\t\t\t\t</scattering_coefficient_spectrum>\n")
					if (readValue(d, 'sss.phasetype', 1) == 1):
						file.write("\t\t\t\t<phase_function>\n")
						file.write("\t\t\t\t\t<uniform/>\n")
						file.write("\t\t\t\t</phase_function>\n")
					if (readValue(d, 'sss.phasetype', 1) == 2):
						file.write("\t\t\t\t<phase_function>\n")
						file.write("\t\t\t\t\t<henyey_greenstein>\n")
						file.write("\t\t\t\t\t\t<g_spectrum>\n")
						if (readValue(d,'sss.phasespec',1) == 1):
							file.write("\t\t\t\t\t\t\t<uniform>\n")
							file.write("\t\t\t\t\t\t\t\t<value>%g</value>\n"%(readValue(d, 'sss.phasespec_uni', 0.5)))
							file.write("\t\t\t\t\t\t\t</uniform>\n")
						else:
							file.write("\t\t\t\t\t\t\t<rgb>\n")
							file.write("\t\t\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d,'sss.phasespec_r',0.5),readValue(d,'sss.phasespec_g',0.5),readValue(d,'sss.phasespec_b',0.5)))
							file.write("\t\t\t\t\t\t\t\t<gamma>2.2</gamma>\n")
							file.write("\t\t\t\t\t\t\t</rgb>\n")
						file.write("\t\t\t\t\t\t</g_spectrum>\n")
						file.write("\t\t\t\t\t</henyey_greenstein>\n")
						file.write("\t\t\t\t</phase_function>\n")
					file.write("\t\t\t</subsurface_scattering>\n")
				file.write("\t\t</basic>\n")
				file.write("\t</medium>\n")
			if (matType == 2) and (matMediumType == 2): #specular and dermis
				file.write("\t<medium\n>")
				file.write("\t\t<name>%s</name>\n"%(matname))
				file.write("\t\t<precedence>%g</precedence>\n"%(readValue(d, 'precedence', 1.0)))
				file.write("\t\t<dermis>\n")
				file.write("\t\t\t<hemoglobin_fraction>%g</hemoglobin_fraction>\n"%(readValue(d, 'hemoglobin', 0.15)))
				file.write("\t\t</dermis>\n")
				file.write("\t</medium>\n")
			if (matType == 2) and (matMediumType == 3): #specular and epidermis
				file.write("\t<medium>\n")
				file.write("\t\t<name>%s</name>\n"%(matname))
				file.write("\t\t<precedence>%g</precedence>"%(readValue(d, 'precedence', 1.0)))
				file.write("\t\t<epidermis>")
				file.write("\t\t\t<melanin_fraction>%g</melanin_fraction>"%(readValue(d, 'melanin', 0.001)))
				file.write("\t\t\t<melanin_type_blend>%g</melanin_type_blend>"%(readValue(d, 'eumelanin', 0.001)))
				file.write("\t\t</epidermis>")
				file.write("\t</medium>")
			if (matType == 2) and (matMediumType == 4):
				file.write("\t<medium>")
				file.write("\t\t<name>%s</name>"%(matname))
				file.write("\t\t<precedence>%g</precedence>"%(readValue(d, 'precedence', 1.0)))
				file.write("\t\t\t<atmosphere>")
				file.write("\t\t\t\t<turbidity>%s</turbidity>"%(readValue(d,'turbidity',2.2)))
				file.write("\t\t\t</atmosphere>")
				file.write("\t</medium>")
			#check if blended material
			if (matType == 3):
				matAName = readValue(d, 'blend.a_name', '')
				matBName = readValue(d, 'blend.b_name', '')
				# check if linked materials have already been exported -
				# if not do it now and add it infront of this material.
				try:
					exportedmatlist.index(matAName);
				except:
					matA = Blender.Material.Get(matAName)
					exportedmatlist.append(matAName)
					exportMaterial(matA, file, exportedmatlist,[],'',uvset)
				try:
					exportedmatlist.index(matBName);
				except:
					matB = Blender.Material.Get(matBName)
					exportedmatlist.append(matBName)
					exportMaterial(matB, file, exportedmatlist,[],'',uvset)
			file.write("\t<material>\n")
			file.write("\t\t<name>%s</name>\n"%(matname))
			if matType == 0:
				transparent=readValue(d,'diffuse.transmitter',0)
				if not transparent or ClayRender.val:
					file.write("\t\t<diffuse>\n")
				else:
					file.write("\t\t<diffuse_transmitter>\n")
			elif(matType== 1):
				file.write("\t\t<phong>\n")
				#export exponent only if no exponent map
				if readValue(d,'exponent.path','')  == '':
					file.write("\t\t\t<exponent>\n\t\t\t\t<constant>%g</constant>\n\t\t\t</exponent>\n"%(readValue(d, 'exponent', 1000.0)))
				file.write("\t\t\t<ior>%g</ior>\n"%(readValue(d, 'ior', 1.0)))
				if not readValue(d,'toggle_phong',0) and (readValue(d, 'nk_data', '').strip() != ''):
						file.write("\t\t\t<nk_data>nkdata%s%s</nk_data>\n"%(os.sep,readValue(d, 'nk_data', '')))
			elif(matType== 2):
				speculartype = readValue(d, 'specular.type', 1)
				if (speculartype == 1):
					file.write("\t\t<specular>\n")
				if (speculartype == 2):
					file.write("\t\t<glossy_transparent>\n")
					file.write("\t\t\t<exponent>%g</exponent>\n"%(readValue(d, 'exponent', 1000.0)))
				else:
					if (readValue(d, 'transparent', 0) != 0):
						file.write("\t\t\t<transparent>true</transparent>\n")
					else:
						file.write("\t\t\t<transparent>false</transparent>\n")
				file.write("\t\t\t<internal_medium_name>%s</internal_medium_name>\n"%(matname))
			elif matType == 3:
				matAName = readValue(d, 'blend.a_name', '')
				matBName = readValue(d, 'blend.b_name', '')
				file.write("\t\t<blend>\n")
				file.write("\t\t\t<a_name>%s</a_name>\n"%(matAName))
				file.write("\t\t\t<b_name>%s</b_name>\n"%(matBName))
				file.write("\t\t\t<step_blend>false</step_blend>\n")
			elif matType == 5:
				file.write("\t\t<oren_nayar>\n")
				file.write("\t\t\t<sigma><constant>%f</constant></sigma>\n"%readValue(d,'sigma',0.2))
			
			#---TEXUTES EXPORT---
			albedo=0
			exponent=0
			bump=0
			displace=0
			blend=0
			if not ClayRender.val and matType != 3 and readValue(d,'albedo.path','') != '' :
				albedo=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'albedo.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				#file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d, 'albedo.path', ''))))
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, target)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%uvset)
				else:
					#Check if any uv sets exists
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'albedo.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'albedo.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'albedo.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'albedo.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'albedo.c', 0.0)))
				file.write("\t\t\t</texture>\n")
			
			if not ClayRender.val and matType==1 and readValue(d,'exponent.path','')  != '':
				exponent=1
				file.write("\t\t\t<texture>\n");
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'exponent.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, target)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%uvset)
				else:
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'exponent.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'exponent.exponent', 2.2)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'exponent.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'exponent.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'exponent.c', 0.0)))
				file.write("\t\t\t</texture>\n");
			if not ClayRender.val and matType != 3 and readValue(d,'bump.path','') != '':
				bump=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'bump.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, target)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%uvset)
				else:
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'bump.uv_set', '')))
				print("BUMP UV SET: %s"%(readValue(d, 'bump.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'bump.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'bump.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'bump.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'bump.c', 0.0)))
				file.write("\t\t\t</texture>\n")
			if not ClayRender.val and matType != 3 and readValue(d, 'displace.path', '') != '':
				displace=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'displace.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, target)))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'displace.exp', 2.2)))
				file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'displace.uv_set', 'uv')))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'displace.b', 1.0)))
				file.write("\t\t\t</texture>\n")
			if not ClayRender.val and matType==3 and readValue(d, 'blend.path','') != '':
				blend=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'blend.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(relpath(file.name, target)))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'blend.exp', 2.2)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%uvset)
				else:
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'blend.uv_set', '')))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'blend.a', 1.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'blend.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'blend.c', 1.0)))
				file.write("\t\t\t</texture>\n")
			
			#---ALBEDO TEXTURE OR SHADER---
			if matType == 0 or matType == 1 or matType == 5:
				if albedo and not readValue(d,'albedo.isl',0):
					if readValue(d,'emitter',0):
						file.write("\t\t\t<emission>\n")
					elif matType == 1:
						file.write("\t\t\t<diffuse_albedo>\n")
					else:
						file.write("\t\t\t<albedo>\n")
					file.write("\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n"%texture_index)
					if readValue(d,'emitter',0):
						file.write("\t\t\t</emission>\n")
					elif matType == 1:
						file.write("\t\t\t</diffuse_albedo>\n")
					else:
						file.write("\t\t\t</albedo>\n")
					texture_index+=1
				elif readValue(d,'albedo.isl',0):
					if readValue(d,'emitter',0):
						file.write("\t\t\t<emission>\n")
					elif matType == 1:
						file.write("\t\t\t<diffuse_albedo>\n")
					else:
						file.write("\t\t\t<albedo>\n")
					file.write("\t\t\t\t<shader>\n")
					file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
					shadername=ShaderList[readValue(d,'albedo.shader',-1)]
					text=Text.Get(shadername)
					for l in text.asLines():
						file.write("\t\t\t\t\t%s\n"%l);
					file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n")
					if readValue(d,'emitter',0):
						file.write("\t\t\t<emission>\n")
					elif matType == 1:
						file.write("\t\t\t</diffuse_albedo>\n")
					else:
						file.write("\t\t\t</albedo>\n")
				else:
					if matType == 1 and readValue(d,'toggle_phong',0):
						file.write("\t\t\t<specular_reflectivity>\n")
						file.write("\t\t\t\t<constant>\n\t\t\t\t\t<rgb>\n\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t\t<gamma>1</gamma>\n\t\t\t\t\t</rgb>\n\t\t\t\t</constant>\n"%(readValue(d, 'rgb.r_s', 0.0), readValue(d, 'rgb.g_s', 0.0), readValue(d, 'rgb.b_s', 0.0)))
					else:
						if matType == 1:
							file.write("\t\t\t<diffuse_albedo>\n")
						else:
							file.write("\t\t\t<albedo>\n")
						if not ClayRender.val:
							file.write("\t\t\t\t<constant><rgb>\n\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb></constant>\n"%(readValue(d, 'rgb.r', 0.0), readValue(d, 'rgb.g', 0.0), readValue(d, 'rgb.b', 0.0)))
						else:
							file.write("\t\t\t\t<constant><rgb>\n\t\t\t\t\t<rgb>0.85 0.85 0.85</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb></constant>\n")
					if matType == 1 and readValue(d,'toggle_phong',0):
						file.write("\t\t\t</specular_reflectivity>\n")
					else:
						if matType == 1:
							file.write("\t\t\t</diffuse_albedo>\n")
						else:
							file.write("\t\t\t</albedo>\n")
			
			#---EXPONENT TEXTURE OR SHADER
			if exponent and not readValue(d,'exponent.isl',0):
				file.write("\t\t\t<exponent>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</exponent>\n"%texture_index)
				texture_index+=1
			elif readValue(d,'exponent.isl',0):
				file.write("\t\t\t<exponent>\n")
				file.write("\t\t\t\t<shader>\n")
				file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
				shadername=ShaderList[readValue(d,'exponent.shader',-1)]
				text=Text.Get(shadername)
				for l in text.asLines():
					file.write("\t\t\t\t\t%s\n"%l);
				file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</exponent>\n")
			#---BUMP TEXTURE OR SHADER---
			if bump and not readValue(d,'bump.isl',0):
				file.write("\t\t\t<bump>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</bump>\n"%texture_index)
				texture_index+=1
			elif readValue(d,'bump.isl',0):
				file.write("\t\t\t<bump>\n")
				file.write("\t\t\t\t<shader>\n")
				file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
				shadername=ShaderList[readValue(d,'bump.shader',-1)]
				text=Text.Get(shadername)
				for l in text.asLines():
					file.write("\t\t\t\t\t%s\n"%l);
				file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</bump>\n")
			#---DISPLACEMNET TEXTURE OR SHADER
			if displace and not readValue(d,'displace.isl',0):
				file.write("\t\t\t<displacement>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</displacement>"%texture_index)
				texture_index+=1
			elif readValue(d,'displace.isl',0):
				file.write("\t\t\t<displacement>\n")
				file.write("\t\t\t\t<shader>\n")
				file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
				shadername=ShaderList[readValue(d,'displace.shader',-1)]
				text=Text.Get(shadername)
				for l in text.asLines():
					file.write("\t\t\t\t\t%s\n"%l);
				file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</displacement>\n")
			#---BLEND TEXTURE OR SHADER
			if matType == 3:
				if blend and not readValue(d,'blend.isl',0):
					file.write("\t\t\t<blend>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</blend>\n"%texture_index)
					texture_index+=1
				elif readValue(d,'blend.isl',0):
					file.write("\t\t\t<blend>\n")
					file.write("\t\t\t\t<shader>\n")
					file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
					shadername=ShaderList[readValue(d,'blend.shader',-1)]
					text=Text.Get(shadername)
					for l in text.asLines():
						file.write("\t\t\t\t\t%s\n"%l);
					file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</blend>\n")
				else:
					file.write("\t\t\t<blend>\n")
					file.write("\t\t\t\t<constant>%g</constant>\n"%(readValue(d, 'blend.factor', 0.0)))
					file.write("\t\t\t</blend>\n")
			#^^^END TEXTURES^^^
			
			
			
			if (matType == 0 or matType == 5 or matType == 1 or matType == 2):#can Emitt
				emitter=readValue(d, 'emitter',0)
				if(emitter):
					gain=readValue(d, 'emitter.gain',0)
					mult=readValue(d, 'emitter.mult',0)
					file.write("\t\t\t<base_emission>\n")
					file.write("\t\t\t\t<constant>\n")
					if readValue(d,'emitter.type_rgb',1):
						file.write("\t\t\t\t\t<rgb>\n")
						file.write("\t\t\t\t\t\t<rgb>%f %f %f</rgb>\n"%(readValue(d, 'rgb.r_e',0)*gain*10**mult,readValue(d, 'rgb.g_e',0)*gain*10**mult,readValue(d, 'rgb.b_e',0)*gain*10**mult))
						file.write("\t\t\t\t\t\t<gamma>1</gamma>\n")
						file.write("\t\t\t\t\t</rgb>\n")
					else:
						file.write("\t\t\t\t\t<blackbody>\n")
						file.write("\t\t\t\t\t\t<temperature>%d</temperature>\n"%readValue(d,'emitter.temperature',5000))
						file.write("\t\t\t\t\t\t<gain>%f</gain>\n"%float(gain*10**mult))
						file.write("\t\t\t\t\t</blackbody>\n")
					file.write("\t\t\t\t</constant>\n")
					file.write("\t\t\t</base_emission>\n")
					if readValue(d,'emitter',0):
						file.write("\t\t\t<layer>%d</layer>\n"%readValue(d,'emitter.layer',0))
			if (matType == 0):
				transparent=readValue(d,'diffuse.transmitter',0)
				if not transparent or ClayRender.val:
					file.write("\t\t</diffuse>\n")
				else:
					file.write("\t\t</diffuse_transmitter>\n")
			if (matType == 5):
				file.write("\t\t</oren_nayar>\n")
			if (matType == 1):
				file.write("\t\t</phong>\n")
			if (matType == 2):
				speculartype = readValue(d, 'specular.type', 1)
				if (speculartype == 1):
					file.write("\t\t</specular>\n")
				if (speculartype == 2):
					file.write("\t\t</glossy_transparent>\n")
			if (matType == 3):
				file.write("\t\t</blend>\n")
			file.write("\t</material>\n")
			file.write("\n")
		if (matType == mInclude):
			includefile = readValue(d, 'include', '').strip()
			if includefile != '':
				includefile = relpath(file.name, includefile)
				print("Include file: %s"%includefile)
				print "including external file: \""+includefile+"\""
				try:
					includelist.index(includefile)
				except:
					includelist.append(includefile)
					#file.write("\t<include>\n")
					#file.write("\t\t<pathname>%s</pathname>\n"%includefile)
					#file.write("\t</include>\n")
		if (matType == mLight) or (matType == mExitPortal):
			file.write("\t<material> <!-- light or exit portal dummy material -->\n")
			file.write("\t\t<name>%s</name>\n"%(matname))
			file.write("\t\t<null_material/>\n")
			file.write("\t</material>\n\n")



### ================================================================
### exportLight(material, pathname gain) -> string
###		returns indigo XML code for light material
### ----------------------------------------------------------------
def exportLight(mat, extgain, file):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	#End readValue
	if mat != None:
		try:
			d = mat.properties['indigo']
		except KeyError:
			d = {}
			print 'WARNING: material "%s" has no Indigo material definitions'%(mat.getName())
		m = []
		matType = readValue(d, 'type', 0)
		if matType == -1:
			lighttype = readValue(d, 'light.type', 1)
			gain = readValue(d, 'light.gain', 1.0) * extgain
			file.write("\t\t<spectrum>\n")
			if (lighttype == 1):
				file.write("\t\t\t<rgb>\n")
				file.write("\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d, 'rgb.r', 0.0)*gain, readValue(d, 'rgb.g', 0.0)*gain, readValue(d, 'rgb.b', 0.0)*gain))
				file.write("\t\t\t\t<gamma>2.2</gamma>\n");
				file.write("\t\t\t</rgb>\n")
			if (lighttype == 2):
				file.write("\t\t\t<peak>\n")
				file.write("\t\t\t\t<base_value>%g</base_value>\n"%(readValue(d, 'light.base', 0.0)))
				file.write("\t\t\t\t<peak_min>%g</peak_min>\n"%(readValue(d, 'light.peak_min', 0.0)))
				file.write("\t\t\t\t<peak_width>%g</peak_width>\n"%(readValue(d, 'light.peak_width', 0.0)))
				file.write("\t\t\t\t<peak_value>%g</peak_value>\n"%(gain))
				file.write("\t\t\t</peak>\n")
			if (lighttype == 3):
				file.write("\t\t\t<blackbody>\n")
				temp = readValue(d, 'light.temperature', 1.0)
				file.write("\t\t\t\t<temperature>%g</temperature>\n"%(temp))
				# blackbody correction removed as indigo v0.7t3 has "efficacy scale" feature now
				# blackbody gain correction depending on temperature (see Stephan Boltzmann Law)
				file.write("\t\t\t\t<gain>%g</gain>\n"%(gain)) # *100/(5.6705e-8*pow(temp, 4))))
				file.write("\t\t\t</blackbody>\n")
			file.write("\t\t</spectrum>\n")
			if (readValue(d, 'light.efficacy_scale', 0) != 0):
				file.write("\t\t<efficacy_scale>\n")
				file.write("\t\t\t<power_drawn>%g</power_drawn>\n"%(readValue(d, 'light.power_drawn', 0.0)))
				file.write("\t\t\t<overall_luminous_efficacy>%g</overall_luminous_efficacy>\n"%(readValue(d, 'light.overall_luminous_efficacy', 0.0)))
				file.write("\t\t</efficacy_scale>\n")
			elif (readValue(d,'light.ies',0) != 0):
				file.write("\t\t<ies_profile>\n")
				file.write("\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d,'light.ies_path', ''))))
				file.write("\t\t</ies_profile>\n")
			if (readValue(d, 'albedo.path', '').strip() != ''):
				file.write("\t\t<texture>\n")
				file.write("\t\t\t<path>%s</path>\n"%(relpath(file.name, readValue(d, 'albedo.path', ''))))
				file.write("\t\t\t<uv_set>%s</uv_set>\n"%(readValue(d, 'albedo.uv_set', 'UVTex')))
				file.write("\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'albedo.exponent', 1.0)))
				file.write("\t\t</texture>\n")



### ================================================================
### matIsLight(material) -> boolean
###		returns true if material is a light
### ----------------------------------------------------------------
def matIsEmitter(mat):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	if mat != None:
		try:
			d = mat.properties['indigo']
		except KeyError:
			d = {}
			print 'WARNING: material "%s" has no Indigo material definitions'%(mat.getName())
		return readValue(d, 'emitter', 0) == 1

### ================================================================
### matIsExitPortal(material) -> boolean
###		returns true if material is an exit portal
### ----------------------------------------------------------------
def matIsExitPortal(mat):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	if mat != None:
		try:
			d = mat.properties['indigo']
		except KeyError:
			d = {}
			print 'WARNING: material "%s" has no Indigo material definitions'%(mat.getName())
		return readValue(d, 'type', 0) == 4

### ================================================================
### matXXXXPathCB(filename)
###		GUI file-selector callbacks
### ----------------------------------------------------------------
def matEmittIESPathCB(filename):
	global matEmittIESPath, activeMaterial
	matEmittIESPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matAlbedoPathCB(filename):
	global matAlbedoPath, activeMaterial
	matAlbedoPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matBumpPathCB(filename):
	global matBumpPath, activeMaterial
	matBumpPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matExponentPathCB(filename):
	global matExponentPath, activeMaterial
	matExponentPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matBlendPathCB(filename):
	global matBlendPath, activeMaterial
	matBlendPath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matIncludeCB(filename):
	global matInclude, activeMaterial
	matInclude.val = filename
	readWriteMaterial(activeMaterial, 1)

def matDisplaceCB(filename):
	global matDisplacePath, activeMaterial
	matDisplacePath.val = filename
	readWriteMaterial(activeMaterial, 1)

def matSaveCB(filename):
	global activeMaterial
	igmfile=open(filename,'w')
	igmfile.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
	igmfile.write('<scenedata>\n')
	exportedmatlist=[]
	includelist=[]
	exportMaterial(activeMaterial, igmfile, exportedmatlist, includelist)
	igmfile.write('</scenedata>\n')
	igmfile.close()

### ================================================================
### materialGUI()
###		draws the material GUI
### ----------------------------------------------------------------
materialGUIwidth = 440
def materialGUI():
	global ScrollY, ScrollAreaY, matClipboard
	global matEmitt, matEmittScale, matEmittIES, matEmittIESPath, matEmittTypeRGB, matEmittTypeBB, matEmittScaleValue, matEmittScaleMeasure, matEmittGain, matEmittMult, matEmittRgb, matEmittRgbR, matEmittRgbG, matEmittRgbB, matEmittTemp, matEmittLayer
	global matDiffTrans
	global matsMenu, activeMaterial, matType, matTypeMenu
	global matLightType, matLightGain, matLightBaseValue, matLightPeakMin, matLightPeakWidth, matLightTemp
	global matTogglePhong,matRgb, matRgbR, matRgbG, matRgbB, matRgb, matRgb_s, matRgbR_s, matRgbG_s, matRgbB_s, matRgbGain
	global matIOR, matExponent, matNKData
	global matSpecularType, matSigma, matTransparent, matMediumType, matPrecedence
	global matCauchyBCoeff, matAbsorptionType, matAbsR, matAbsG, matAbsB, matAbsGain, matAbs
	global matSss, matSssType, matSssR, matSssG, matSssB, matSssRgb, matSssGain, matSssUniform, matSssUniformExp
	global matAlbedoUVset, matAlbedoISL, matAlbedoShader, matAlbedoPath, matAlbedoExponent, matAlbedoA, matAlbedoB, matAlbedoC
	global matBumpUVset, matBumpISL, matBumpShader, matBumpPath, matBumpExponent, matBumpA, matBumpB, matBumpC
	global matExponentUVset, matExponentISL, matExponentShader, matExponentPath, matExponentExponent, matExponentA, matExponentB, matExponentC, matDisplaceISL, matDisplaceShader
	global matBlendMatA, matBlendMatB
	global matBlendFactor, matBlendUVset, matBlendPath, matBlendExponent, matBlendA, matBlendB, matBlendC
	global matInclude, PreviewScene, matPreviewFile, PreviewName
	global matSssPhaseType, matSssPhaseSpec , matSssPhaseSpecUni, matSssPhaseSpecR, matSssPhaseSpecG, matSssPhaseSpecB
	global matHemoglobin, matTurbidity, matMelanin, matEumelanin
	global matDisplaceType, matDisplacePath, matDisplaceExp, matDisplaceUVSet, matDisplaceA, matDisplaceB, matDisplaceC, matDisplaceFBMUScale, matDisplaceFBMVScale
	global currentMaterial
	
	drawScreenButtons()
	
	def xd(pos, div):
		return (materialGUIwidth-10)/div*pos+10
	def wd(div):
		return (materialGUIwidth-10)/div-10
	
	gui = guiLayout()
	
	gui.title("Material editor")
	
	gui.newline()
	
	AreaDims = Window.GetAreaSize()
	
	mats = Blender.Material.Get()
	matsmenustr = "Materials:%t|"
	
	for m in mats:
		matsmenustr += m.getName()+"|"
	matsmenustr = matsmenustr[:-1]
	gui.newline("Blender material")
	matsMenu = Draw.Menu(matsmenustr, evtMatsMenu, gui.x, gui.y, wd(2)-10, 18, matsMenu.val, "select Blender material")
	
	gui.newline()
	
	Draw.Button("Reset", evtMatConvert, gui.x, gui.y, 60, 18, "Reset to the original blender material")
	gui.x += 65
	gui.text("or")
	
	# Draw.Button("Import", evtMatSave, gui.x, gui.y, 60, 18, "Load an indigo material from an igm file")
	# gui.x += 65
	
	Draw.Button("Export", evtMatSave, gui.x, gui.y, 60, 18, "Save current indigo material into separate igm file")
	gui.x += 65
	
	if activeMaterial != None:
		gui.text("or")
		Draw.Button("Copy", evtMatCopy, gui.x, gui.y, 40, 18, "the present editor material")
		gui.x += 45
		if matClipboard != None:
			Draw.Button("Paste", evtMatPaste, gui.x, gui.y, 45, 18, "the  previously copied editor material")
		
		gui.subtitle("Indigo Material")
		
		gui.newline("Material type")
		matTypeMenu = Draw.Menu("Material type:%t|DIFFUSE = Flat matte paper-like material%x0|OREN-NAYAR = Rough clay material%x5|PHONG = Shiny, reflective, metallic materials%x1|SPECULAR = Transparent, semi-transparent  material%x2|BLENDED = A blend between two other materials%x3|EXIT PORTAL%x4|EXTERNAL%x9", evtMatType, gui.x, gui.y, 320, 18, matType, "select the desired material type")
		
		gui.newline()
		Draw.Button("Preview", evtMatPreview, gui.x, gui.y, 70, 18, "render a preview of the material in simple scene")
		
		y = gui.y + 40
		
		if (matType == 0) or (matType == 5):
			#ScrollY=0
			ScrollAreaY = 560
		if (matType == 1):
			#ScrollY=0
			ScrollAreaY = 560
		if (matType == 2):
			#ScrollY=0
			ScrollAreaY = 470
		if (matType == 3):
			#ScrollY=0
			ScrollAreaY = 350
		
		if (matType == 0):
			gui.subtitle("Diffuse material")
			gui.newline("Transparent")
			matDiffTrans = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 30, 18, matDiffTrans.val, "enable transparency")
			
		if (matType == 5):
			gui.subtitle("Oren-nayar material")
			gui.newline("Sigma")
			matSigma = Draw.Number("",evtNoEvt, gui.x, gui.y,wd(4)+5,18,matSigma.val,0.0,1.0,"material roughness 0.0. - 1.0")
		
		# if (matType == 0) or (matType == 5) :#diffuse, oren-nayar
		# 	matRgb = Draw.ColorPicker(evtMatRgbPicker, xd(0,2), y-180, 30, 58, (matRgbR.val, matRgbG.val, matRgbB.val), "colour and picker (LMB)")
		# 	matRgbR = Draw.Slider("R ", evtNoEvt, xd(0,2)+30, y-140, wd(2)-30, 18, matRgbR.val, 0.0, 1.0, 1, "red component of colour")
		# 	matRgbG = Draw.Slider("G ", evtNoEvt, xd(0,2)+30, y-160, wd(2)-30, 18, matRgbG.val, 0.0, 1.0, 1, "green component of colour")
		# 	matRgbB = Draw.Slider("B ", evtNoEvt, xd(0,2)+30, y-180, wd(2)-30, 18, matRgbB.val, 0.0, 1.0, 1, "blue component of colour")
		
		if (matType == 1):#phong
			gui.subtitle("Phong material")
			
			gui.newline("NK Data")
			matNKData = Draw.String("", evtNoEvt, gui.x, gui.y, 100, 18, matNKData.val, 20, "lab measured mat. parameters called as e.g. Au.nk = gold")
			
			gui.newline("Exponent")
			matExponent = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matExponent.val, 1.0, 1e+9, "surface shininess - rubber 50 , mirror 1 million")
			gui.units("dimensionless", 100)
			
			gui.newline("Index of Refract")
			matIOR = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matIOR.val, 1e-4, 1000.0, "index of refraction,water 1.33 glass 1.5")
			gui.units("dimensionless", 100)
			
			gui.newline("Option")
			matTogglePhong=Draw.Toggle("Specular Reflectivity",evtNoEvt, gui.x, gui.y,wd(2),18,matTogglePhong.val,"enable specular reflectivity color")
			
			y = gui.y
			
			if matTogglePhong.val:
				gui.subtitle("Specular reflectivity")
				
				gui.newline("Material type")
				matTypeMenu = Draw.Menu("Material type:%t|DIFFUSE = Flat matte paper-like material%x0|OREN-NAYAR = Rough clay material%x5|PHONG = Shiny, reflective, metallic materials%x1|SPECULAR = Transparent, semi-transparent  material%x2|BLENDED = A blend between two other materials%x3|EXIT PORTAL%x4|EXTERNAL%x9", evtMatType, gui.x, gui.y, 320, 18, matType, "select the desired material type")
				
				gui.newline("Color")
				matRgb_s = Draw.ColorPicker(evtMatRgbPicker, gui.x, gui.y - 40, 30, 58, (matRgbR_s.val, matRgbG_s.val, matRgbB_s.val), "colour and picker (LMB)")
				gui.x += 32
				matRgbR_s = Draw.Slider("R ", evtNoEvt, gui.x, gui.y, 120, 18, matRgbR_s.val, 0.0, 1.0, 1, "red component of colour")
				gui.y -= 20
				matRgbG_s = Draw.Slider("G ", evtNoEvt, gui.x, gui.y, 120, 18,  matRgbG_s.val, 0.0, 1.0, 1, "green component of colour")
				gui.y -= 20
				matRgbB_s = Draw.Slider("B ", evtNoEvt, gui.x, gui.y, 120, 18, matRgbB_s.val, 0.0, 1.0, 1, "blue component of colour")
				gui.y -= 20
		
		if (matType == 2):
			gui.subtitle("Specular material")
			gui.newline("Spec type:")

			matSpecularType = Draw.Menu("Specular type:%t|specular|glossy transparent", evtNoEvt, gui.x, gui.y, wd(3), 18, matSpecularType.val, "select specular type")

			if matSpecularType.val == 1: # specular
				gui.newline("Transparency")
				matTransparent = Draw.Toggle(" x ", evtNoEvt, gui.x, gui.y, 30, 18, matTransparent.val, "enable transparency")
			if matSpecularType.val == 2: # glossy
				gui.newline("Exponent")
				matExponent = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matExponent.val, 1.0, 1e+9, "surface shininess - rubber 50 , mirror 1 million")
				gui.units("dimensionless", 100)

			gui.newline("Precedence")
			matPrecedence = Draw.Number("", evtNoEvt, gui.x, gui.y, 50, 18, matPrecedence.val, 1, 100, "set precendence of mediums when sharing same space")
			gui.units("integer", 50)

			gui.newline("Medium type:")
			matMediumType = Draw.Menu("Medium type:%t|basic%x1|dermis%x2|epidermis%x3|atmosphere%x4", evtNoEvt, gui.x, gui.y, wd(4), 18, matMediumType.val, "select medium type")

			if matMediumType.val == 2: #dermis
				gui.newline("  Hemoglobin")
				matHemoglobin = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matHemoglobin.val, 0.0, 1.0, "amount of hemoglobin, typ range .001-.01")
				gui.units("fraction", 100)
				
			if matMediumType.val == 3: #epidermis
				gui.newline("  Melanin")
				matMelanin = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matMelanin.val, 0.0, 1.0, "amount of melanin, typ range 0-.05")
				gui.units("fraction", 100)

				gui.newline("  Eumelanin")
				matEumelanin = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matEumelanin.val, 0.0, 1.0, "amount of eumelanin relative to pheomelanin")
				gui.units("fraction", 100)
				
			if matMediumType.val == 4:
				gui.newline("  Turbidity")
				matTurbidity = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matTurbidity.val, 2, 10, "Atmosphere turbidity. Values 2-10")
				gui.units("real", 100)
				
			if matMediumType.val == 1: # basic
				gui.newline("  Index of refrac.")
				matIOR = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matIOR.val, 1e-4, 1000.0, "index of refraction,water 1.33 glass 1.5")
				
				gui.newline("  Cauchy B coeff")
				matCauchyBCoeff = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matCauchyBCoeff.val, 0.0, 1.0, "dispersive refr, range .003-.01, 0 disables ")
				gui.units("coefficient", 100)
				
				gui.newline("  Gain")
				matAbsGain = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matAbsGain.val, 1e-3, 1e3, "RGB gain")
				gui.units("dimensionless", 100)

				gui.newline("  Color")
				matAbs = Draw.ColorPicker(evtMatAbsPicker, gui.x, gui.y - 40, 30, 58, (1.0-matAbsR.val, 1.0-matAbsG.val, 1.0-matAbsB.val), "absorption color and picker (LMB)")
				gui.x += 32
				matAbsR = Draw.Slider("R ", evtNoEvt, gui.x, gui.y, wd(2)-30, 18, matAbsR.val, 0.0, 1.0, 1, "red component of absorption colour")
				gui.y -= 20
				matAbsG = Draw.Slider("G ", evtNoEvt, gui.x, gui.y, wd(2)-30, 18, matAbsG.val, 0.0, 1.0, 1, "green component of absorption colour")
				gui.y -= 20
				matAbsB = Draw.Slider("B ", evtNoEvt, gui.x, gui.y, wd(2)-30, 18, matAbsB.val, 0.0, 1.0, 1, "blue component of absorption colour")
				
				gui.subtitle("       Sub surface scattering")
				matSss = Draw.Toggle(" x ", evtNoEvt, 16, gui.y+5, 20, 18, matSss.val, "enable subsurface scattering")
				
				if matSss.val != 0:#RGB
					gui.newline("Scatter type:")
					matSssType = Draw.Menu("Scattering type:%t|RGB%x1|Uniform%x2", evtNoEvt, gui.x, gui.y, wd(3), 18, matSssType.val, "select scattering type")

					if matSssType.val == 1:
						gui.newline("  Gain")
						matSssGain = Draw.Number("Gain ", evtNoEvt, gui.x, gui.y, 100, 18, matSssGain.val, 1e-3, 1e3, "RGB gain")
						gui.units("dimensionless", 100)

						gui.newline("  Color")
						matSssRgb = Draw.ColorPicker(evtMatSssRgbPicker, gui.x, gui.y - 40, 30, 58, (matSssR.val, matSssG.val, matSssB.val), "SSS color and picker (LMB)")
						gui.x += 32
						matSssR = Draw.Slider("R ", evtNoEvt, gui.x, gui.y, wd(2)-30, 18, matSssR.val, 0.0, 1.0, 1, "red component of SSS color")
						gui.y -= 20
						matSssG = Draw.Slider("G ", evtNoEvt, gui.x, gui.y, wd(2)-30, 18, matSssG.val, 0.0, 1.0, 1, "green component of SSS color")
						gui.y -= 20
						matSssB = Draw.Slider("B ", evtNoEvt, gui.x, gui.y, wd(2)-30, 18, matSssB.val, 0.0, 1.0, 1, "blue component of SSS color")
						
					if matSssType.val == 2:
						gui.newline("   Value")
						matSssUniform = Draw.Number("",evtNoEvt, gui.x, gui.y, 80, 18, matSssUniform.val,0,10,"scattering value mantissa")
						gui.units("x10^", 80)
						matSssUniformExp = Draw.Number("",evtNoEvt, gui.x, gui.y, wd(4), 18, matSssUniformExp.val,-5,5,"scattering value exponent 10^+-5")

					gui.newline("Phase function")
					matSssPhaseType = Draw.Menu("Phase type:%t|Uniform%x1|Henyey Greenstein%x2", evtNoEvt, gui.x, gui.y, wd(3), 18, matSssPhaseType.val, "select phase type")

					if matSssPhaseType.val == 2:
						gui.newline("   Spectrum")
						matSssPhaseSpec = Draw.Menu("Phase Type Spectrum:%t|Uniform%x1|RGB%x2",evtNoEvt, gui.x, gui.y,wd(4),18,matSssPhaseSpec.val,"Select phase spectrum")

						if matSssPhaseSpec.val == 1:
							gui.newline("   Value")
							matSssPhaseSpecUni=Draw.Number("",evtNoEvt,gui.x, gui.y,wd(4),18,matSssPhaseSpecUni.val,-1,1,"Phase Function Uniform Value")

						if matSssPhaseSpec.val == 2:
							gui.newline("     Red")
							matSssPhaseSpecR=Draw.Number("R",evtNoEvt, gui.x, gui.y,wd(4),18,matSssPhaseSpecR.val,0,1,"Phase Function Red Value")
							gui.newline("     Green")
							matSssPhaseSpecG=Draw.Number("G",evtNoEvt, gui.x, gui.y,wd(4),18,matSssPhaseSpecG.val,0,1,"Phase Function Green Value")
							gui.newline("     Blue")
							matSssPhaseSpecB=Draw.Number("B",evtNoEvt, gui.x, gui.y,wd(4),18,matSssPhaseSpecB.val,0,1,"Phase Function Blue Value")
		
		if matType == 2:
			y-=90
			ScrollAreaY+=90

		if (matType == 3):#Blended Materials
			BGL.glRasterPos2i(xd(0,2), y-135); Draw.Text("Material A: "+matBlendMatA.val)
			Draw.Button("...", evtMatBlendMatA, xd(0,2)+wd(2)-40, y-140, 40, 18, "select a material from list")
			BGL.glRasterPos2i(xd(1,2), y-135); Draw.Text("Material B: "+matBlendMatB.val)
			Draw.Button("...", evtMatBlendMatB, xd(1,2)+wd(2)-40, y-140, 40, 18, "select a material from list")
			matBlendFactor = Draw.Slider("Blend-factor ", evtNoEvt, xd(0,1), y-160, wd(1), 18, matBlendFactor.val, 0.0, 1.0, 0, "blend ratio of the materials")
			BGL.glColor3f(1.0,1.0,1.0); BGL.glRasterPos2i(xd(0,1),y-180); Draw.Text("Blend map:")
			matBlendPath = Draw.String("filename: ", evtNoEvt, xd(0,1)+70, y-185, wd(1)-90, 18, matBlendPath.val, 250, "blend texture filename")
			kk = Draw.Button("...", evtMatBlendPath, xd(0,1)+wd(1)-20, y-185, 20, 18, "open file selector")
			matBlendExponent = Draw.Number("gamma ", evtNoEvt, xd(0,2), y-205, wd(4), 18, matBlendExponent.val, 0.0, 10.0, "texture gammma")
			matBlendUVset = Draw.String("UVset: ", evtNoEvt, 107, y-205, 95, 18, matBlendUVset.val, 20, "blend texture UV-set")
			#obj = Blender.Scene.GetCurrent().objects.active
			#mesh=obj.getData(mesh=True)
			#uvLayerNames=mesh.getUVLayerNames()
			#uvsets="UVsets:%t"
			#index=0
			#for l in uvLayerNames:
			#	uvsets+="|%s%%x%d"%(l,index)
			#	index+=1
			#matBlendUVset = Draw.Menu(uvsets,evtNoEvt, 107,y-205,95,18,matBlendUVset.val,"UV sets");
			matBlendA = Draw.Number("A ", evtNoEvt, xd(2,5)+20, y-205, wd(5), 18, matBlendA.val, 0.0, 10.0, "")
			matBlendB = Draw.Number("B ", evtNoEvt, xd(3,5)+10, y-205, wd(5), 18, matBlendB.val, 0.0, 10.0, "")
			matBlendC = Draw.Number("C ", evtNoEvt, xd(4,5), y-205, wd(5), 18, matBlendC.val, 0.0, 10.0, "")
		
		##
		## Get textures and shaders...
		##
		
		texts=Text.Get()
		shader="NONE%x-1"
		ShaderList[:]=[]
		for s in texts:
			shadername=s.getName()
			if(shadername[:4] == "isl_"):
				ShaderList.append(s.getName())
		index=0
		for l in ShaderList:
			shader+="|%s%%x%d"%(ShaderList[index],index)
			index+=1


		##
		##	Albedo channel
		##
		
		if (matType == 0) or (matType == 1 and not matTogglePhong.val) or (matType == 5) or (matType == -1):
			gui.subtitle("       Color")
			currentMaterial.albedo.enabled = Draw.Toggle(" X ", evtNoEvt, 16, gui.y + 5, 20, 18, currentMaterial.albedo.enabled.val, "enable color channel")
			
			if currentMaterial.albedo.enabled.val:
				gui.newline("Type")
				currentMaterial.albedo.type = Draw.Menu("Color%x0|Texture%x1|Shader%x2", evtNoEvt, gui.x, gui.y, 100, 18, currentMaterial.albedo.type.val, "select the desired material type")

				if currentMaterial.albedo.typeColor():
					gui.newline("Color")
					matRgb = Draw.ColorPicker(evtMatRgbPicker, gui.x, gui.y - 40, 30, 58, (matRgbR.val, matRgbG.val, matRgbB.val), "colour and picker (LMB)")
					gui.x += 32
					matRgbR = Draw.Slider("R ", evtNoEvt, gui.x, gui.y, 120, 18, matRgbR.val, 0.0, 1.0, 1, "red component of colour")
					gui.y -= 20
					matRgbG = Draw.Slider("G ", evtNoEvt, gui.x, gui.y, 120, 18,  matRgbG.val, 0.0, 1.0, 1, "green component of colour")
					gui.y -= 20
					matRgbB = Draw.Slider("B ", evtNoEvt, gui.x, gui.y, 120, 18, matRgbB.val, 0.0, 1.0, 1, "blue component of colour")

				if (currentMaterial.albedo.typeShader()) or (currentMaterial.albedo.typeTexture()):
					gui.newline("UV Texture")
					matAlbedoUVset = Draw.String("", evtNoEvt, gui.x, gui.y, 95, 18, matAlbedoUVset.val, 20, "albedo texture UV-set")

				if currentMaterial.albedo.typeTexture():
					gui.newline("Filename")
					matAlbedoPath = Draw.String("", evtNoEvt, gui.x, gui.y, 250, 18, matAlbedoPath.val, 250, "albedo texture filename (referencing Blender UV co-ords)")
					Draw.Button("...", evtMatAlbedoPath, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					matAlbedoExponent = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matAlbedoExponent.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)

					gui.newline("Brightness")
					matAlbedoA = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matAlbedoA.val, 0.0, 10.0, "")
					gui.units("dimensionless", 100)
					
					gui.newline("Contrast")
					matAlbedoB = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matAlbedoB.val, 0.0, 10.0, "")
					gui.units("dimensionless", 100)

					gui.newline("Saturation")
					matAlbedoC = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matAlbedoC.val, 0.0, 10.0, "")
					gui.units("dimensionless", 100)

				if currentMaterial.albedo.typeShader():
					# matAlbedoISL = Draw.Toggle("Shader", evtNoEvt, 10, y-280, 70, 18, matAlbedoISL.val, "Use Indigo Shader")
					gui.newline("Indigo Shader")
					matAlbedoShader=Draw.Menu(shader, evtNoEvt, gui.x, gui.y, 250, 18, matAlbedoShader.val, "Select Albedo Shader")
	
		##
		##	Emission channel
		##

		if (matType != 3 and matType != 4 and matType != 9 and matType != -1):
			gui.subtitle("       Light emission")
			matEmitt = Draw.Toggle(" X ", evtNoEvt, 16, gui.y + 6, 20, 18, matEmitt.val, "enable emission")

			if(matEmitt.val):
				gui.newline("Light layer #")
				matEmittLayer=Draw.Number("",evtNoEvt, gui.x, gui.y, 20, 18, matEmittLayer.val, 0, 7, "layer that light emitted from this material will be drawn to")

				gui.newline("Power")
				matEmittScaleValue = Draw.Number("",evtNoEvt , gui.x, gui.y, 80, 18, matEmittScaleValue.val, 0.0, 1000000, "")
				gui.units("lumens (bulb = 1500, sun = 600000)", 80)

				gui.newline("Color")
				matEmittRgb = Draw.ColorPicker(evtMatRgbPicker, gui.x, gui.y - 40, 30, 58, (matEmittRgbR.val, matEmittRgbG.val, matEmittRgbB.val), "colour and picker (LMB)")
				gui.x += 32
				matEmittRgbR = Draw.Slider("R ", evtNoEvt, gui.x, gui.y, 120, 18, matEmittRgbR.val, 0.0, 1.0, 1, "red component of colour")
				gui.y -= 20
				matEmittRgbG = Draw.Slider("G ", evtNoEvt, gui.x, gui.y, 120, 18,  matEmittRgbG.val, 0.0, 1.0, 1, "green component of colour")
				gui.y -= 20
				matEmittRgbB = Draw.Slider("B ", evtNoEvt, gui.x, gui.y, 120, 18, matEmittRgbB.val, 0.0, 1.0, 1, "blue component of colour")
				gui.y -= 20

				gui.y += 19
				gui.newline("IES Profile")
				matEmittIES = Draw.Toggle("Enable", evtMatEmittIES, gui.x, gui.y, 60, 18, matEmittIES.val, "Enable IES Profile")

				if matEmittIES.val:
					gui.newline("Filename")
					matEmittIESPath = Draw.String("filename: ", evtNoEvt, gui.x, gui.y, 300, 18, matEmittIESPath.val, 250, "Path to file with IES Profile ")
					Draw.Button("...", evtMatEmittIESPath, gui.x + 310, gui.y, 20, 18, "open file selector")

		##
		##	Bump channel
		##

		if (matType == 0 and not matDiffTrans.val) or (matType == 1) or (matType == 2) or (matType == 5):
			gui.subtitle("       Bump")
			currentMaterial.bump.enabled = Draw.Toggle(" X ", evtNoEvt, 16, gui.y + 6, 20, 18, currentMaterial.bump.enabled.val, "enable bump channel")
		
			if currentMaterial.bump.enabled.val:
				gui.newline("Type")
				currentMaterial.bump.type = Draw.Menu("Texture%x1|Shader%x2", evtNoEvt, gui.x, gui.y, 100, 18, currentMaterial.bump.type.val, "select the desired channel type")

				if (currentMaterial.bump.typeShader()) or (currentMaterial.bump.typeTexture()):
					gui.newline("UV Texture")
					matBumpUVset = Draw.String("", evtNoEvt, gui.x, gui.y, 95, 18, matBumpUVset.val, 20, "bump texture UV-set")

				if currentMaterial.bump.typeTexture():
					gui.newline("Filename")
					matBumpPath = Draw.String("", evtNoEvt, gui.x, gui.y, 250, 18, matBumpPath.val, 250, "texture filename (referencing Blender UV co-ords)")
					Draw.Button("...", evtMatBumpPath, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					matBumpExponent = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matBumpExponent.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)

					gui.newline("Height")
					matBumpA = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matBumpA.val, -1.0, 1.0, "")
					gui.units("mm", 100)
			
				if currentMaterial.bump.typeShader():
					gui.newline("Indigo Shader")
					matBumpShader=Draw.Menu(shader, evtNoEvt, gui.x, gui.y, 250, 18, matBumpShader.val, "Select Bump Shader")
		
		if (matType == 1):
			gui.subtitle("       Exponent")
			currentMaterial.exponent.enabled = Draw.Toggle(" X ", evtNoEvt, 16, gui.y + 6, 20, 18, currentMaterial.exponent.enabled.val, "enable exponent channel")
		
			if currentMaterial.exponent.enabled.val:
				gui.newline("Type")
				currentMaterial.exponent.type = Draw.Menu("Texture%x1|Shader%x2", evtNoEvt, gui.x, gui.y, 100, 18, currentMaterial.exponent.type.val, "select the desired channel type")

				if (currentMaterial.exponent.typeShader()) or (currentMaterial.exponent.typeTexture()):
					gui.newline("UV Texture")
					matExponentUVset = Draw.String("", evtNoEvt, gui.x, gui.y, 95, 18, matExponentUVset.val, 20, "exponent texture UV-set")

				if currentMaterial.exponent.typeTexture():
					gui.newline("Filename")
					matExponentPath = Draw.String("", evtNoEvt, gui.x, gui.y, 250, 18, matExponentPath.val, 250, "texture filename (referencing Blender UV co-ords)")
					Draw.Button("...", evtMatExponentPath, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					matExponentExponent = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matExponentExponent.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)

					gui.newline("Intensity")
					matExponentA = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matExponentA.val, -1.0, 1.0, "")
					gui.units("dimensionless", 100)
			
				if currentMaterial.exponent.typeShader():
					gui.newline("Indigo Shader")
					matExponentShader=Draw.Menu(shader, evtNoEvt, gui.x, gui.y, 250, 18, matExponentShader.val, "Select Exponent Shader")
			
		if (matType == 0 and not matDiffTrans.val) or (matType == 1) or (matType == 2) or (matType == 5):
			gui.subtitle("       Displacement")
			currentMaterial.displacement.enabled = Draw.Toggle(" X ", evtNoEvt, 16, gui.y + 6, 20, 18, currentMaterial.displacement.enabled.val, "enable displacement channel")

			if currentMaterial.displacement.enabled.val:
				gui.newline("Type")
				currentMaterial.displacement.type = Draw.Menu("Texture%x1|Shader%x2", evtNoEvt, gui.x, gui.y, 100, 18, currentMaterial.displacement.type.val, "select the desired channel type")

				if (currentMaterial.displacement.typeShader()) or (currentMaterial.displacement.typeTexture()):
					gui.newline("UV Texture")
					matDisplaceUVSet = Draw.String("", evtNoEvt, gui.x, gui.y, 95, 18, matDisplaceUVSet.val, 20, "displacement texture UV-set")

				if currentMaterial.displacement.typeTexture():
					gui.newline("Filename")
					matDisplacePath = Draw.String("", evtNoEvt, gui.x, gui.y, 250, 18, matDisplacePath.val, 250, "texture filename (referencing Blender UV co-ords)")
					Draw.Button("...", evtDisplaceImage, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					matDisplaceExp = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matDisplaceExp.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)

					gui.newline("Height")
					matDisplaceB = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, matDisplaceB.val, -1.0, 1.0, "")
					gui.units("metres", 100)

				if currentMaterial.displacement.typeShader():
					gui.newline("Indigo Shader")
					matDisplaceShader=Draw.Menu(shader, evtNoEvt, gui.x, gui.y, 250, 18, matDisplaceShader.val, "Select Displace Shader")
	
		if (matType == mInclude):
			gui.subtitle("External material")
			gui.newline("Filename")
			matInclude = Draw.String("", evtNoEvt, gui.x, gui.y, 250, 18, matInclude.val, 250, "include filename")
			Draw.Button("...", evtMatInclude, gui.x+255, gui.y, 20, 18, "open file selector")

			# if matType == 4:
			# 	ScrollAreaY = 300
			# 
			# if matType != 1:
			# 	y+=70
			# 	ScrollAreaY-=70

	drawExportButtons(gui)
	#Draw.Redraw(1)


### ================================================================
### materialEvent(evt, val)
###		material GUI event routine
### ----------------------------------------------------------------
def materialEvent(evt, val):
	global activeObject, activeObjectMat0, matsMenu, activeMaterial
	
	obj = Blender.Scene.GetCurrent().objects.active
	if(obj != None):
		try:
			try:
				if obj.getType() == "Text":
					cu=Blender.Curve.Get(obj.getData().getName())
					mat0=cu.materials[0]
				else:
					mat0 = (obj.getData(0,1).materials)[obj.activeMaterial - 1]
			except IndexError:
				mat0 = None
		except AttributeError:
			mat0 = None
	else: mat0 = None
	
	if obj != None and ((activeObject != obj) or (activeObjectMat0 != mat0)):
		activeObject = obj
		activeObjectMat0 = mat0
		if (mat0 != None):
			mats = Blender.Material.Get()
			matsMenu.val = 0
			matindex = matsMenu.val
			for m in mats:
				if m.getName() == mat0.getName():
					matindex = mats.index(m)+1
					activeMaterial = m
					setCurrentMaterial(activeMaterial)
			matsMenu.val = matindex
			readWriteMaterial(activeMaterial, 0)
			#drawGUI()
			Draw.Redraw(1)


### ================================================================
### materialButtonEvt(evt)
###		material GUI button-event routine
### ----------------------------------------------------------------
def materialButtonEvt(evt):
	global matsMenu, matType, activeMaterial, matBlendMatA, matBlendMatB, matSssRgb, matSssUniform, matClipboard
	global matAbsR, matAbsG, matAbsB, matRgbR, matRgbG, matRgbB, matSssR, matSssG, matSssB, matAlbedoShader
	if evt == evtMatsMenu:
		mats = Blender.Material.Get()
		activeMaterial = mats[matsMenu.val-1]
		setCurrentMaterial(activeMaterial)
		readWriteMaterial(activeMaterial, 0)
	if evt == evtMatType:
		matType = matTypeMenu.val
	if evt == evtMatAbsPicker:
		matAbsR.val = 1.0-matAbs.val[0]
		matAbsG.val = 1.0-matAbs.val[1]
		matAbsB.val = 1.0-matAbs.val[2]
	if evt == evtMatRgbPicker:
		matRgbR.val = matRgb.val[0]
		matRgbG.val = matRgb.val[1]
		matRgbB.val = matRgb.val[2]
	if evt == evtMatEmittRgbPicker:
		matEmittRgbR.val=matEmittRgb.val[0]
		matEmittRgbG.val=matEmittRgb.val[1]
		matEmittRgbB.val=matEmittRgb.val[2]
	if evt == evtMatRgbPicker_s:
		matRgbR_s.val = matRgb_s.val[0]
		matRgbG_s.val = matRgb_s.val[1]
		matRgbB_s.val = matRgb_s.val[2]
	if evt == evtMatSssRgbPicker:
		matSssR.val = matSssRgb.val[0]
		matSssG.val = matSssRgb.val[1]
		matSssB.val = matSssRgb.val[2]
	if (evt == evtMatBlendMatA) or (evt == evtMatBlendMatB):
		mats = Blender.Material.Get()
		matsmenustr = "materials:%t|"
		for m in mats:
			matsmenustr += m.getName()+"|"
		matsmenustr += "Default|null"
		i = Draw.PupMenu(matsmenustr,20)
		if i > 0:
			if (i <= len(mats)):
				matname = mats[i-1].getName()
			else:
				if (i == len(mats)+1):
					matname = "Default"
				else:
					matname = "null"
			if (evt == evtMatBlendMatA):
				matBlendMatA.val = matname
			if (evt == evtMatBlendMatB):
				matBlendMatB.val = matname
	if evt == evtMatEmittIESPath:
		Blender.Window.FileSelector(matEmittIESPathCB,'select IES file')
	if (evt == evtMatAlbedoPath):
		Blender.Window.FileSelector(matAlbedoPathCB, 'select texture')
	if (evt == evtMatBumpPath):
		Blender.Window.FileSelector(matBumpPathCB, 'select texture')
	if (evt == evtMatExponentPath):
		Blender.Window.FileSelector(matExponentPathCB, 'select texture')
	if (evt == evtMatBlendPath):
		Blender.Window.FileSelector(matBlendPathCB, 'select texture')
	if (evt == evtMatInclude):
		Blender.Window.FileSelector(matIncludeCB, 'select include-file')
	if (evt == evtDisplaceImage):
		Blender.Window.FileSelector(matDisplaceCB, 'select image')
	if (evt == evtMatPreview):
#		if not os.path.isfile(IndigoPath.val):
#			Draw.PupMenu("Error%t|Indigo not found. Please set the correct path in System section.")
#			return
		if (matType >= 0) and (matType != mInclude):
			exportMatPreview(activeMaterial)
		else:
			Draw.PupMenu("can't preview emitter or external materials, sorry%t|OK%x1")
	
	if (evt == evtMatCopy):
		if (activeMaterial != None):
			try:
				readWriteMaterial(activeMaterial, 1)
				matClipboard = activeMaterial.properties['indigo'].convert_to_pyobject()
			except KeyError:
				print 'WARNING: material "%s" has no Indigo material definitions'%(activeMaterial)
	if (evt == evtMatPaste) and (matClipboard != None):
		if (activeMaterial != None):
			try:
				activeMaterial.properties['indigo'] = matClipboard
				readWriteMaterial(activeMaterial, 0)
			except KeyError:
				print 'WARNING: material "%s" has no Indigo material definitions'%(activeMaterial)
	if (evt == evtMatConvert):
		if (activeMaterial != None):
			convertMaterial(activeMaterial);
			readWriteMaterial(activeMaterial, 0);
	if (evt == evtMatSave):
		if (activeMaterial != None):
			Blender.Window.FileSelector(matSaveCB, 'select igm file')
	
	readWriteMaterial(activeMaterial, 1)
	Draw.Redraw(1)

### ----------------------------------------------------------------
### end of INDIGO material editor module
########################################################################
########################################################################

######################################################
# Functions
######################################################


# New name based on old with a different extension
def newFName(ext):
	return Blender.Get('filename')[: -len(Blender.Get('filename').split('.', -1)[-1]) ] + ext

def emptyFName():
	return Blender.sys.dirname(Blender.Get('filename'))+"\\"


def IndigoBoolean(val):
	if val != 0:
		val = "true"
	else:
		val = "false"
	return val

def splitn(t):
	nt = t.split('\n')
	return nt[0]

def colGamma(value):
	global ColExponent
	return value**ColExponent.val


######################################################
# EXPORT
######################################################

def launchAnimIndigo(filename):
	if (os.path.isfile(IndigoPath.val)):
		indigopath=Blender.sys.dirname(IndigoPath.val)
		indigo_app=Blender.sys.basename(IndigoPath.val)
	else:
		Draw.PupMenu("Error%t|Indigo not found. Please set the correct path in System section.")
		return
	f = open(os.path.join(indigopath, 'AnimFrames.cmd'), 'w')
	f.write("@echo off\n")
	f.write("cd /d \"" + os.path.join(indigopath, "") + "\"\n")
	startF = Blender.Get('staframe')
	endF = Blender.Get('endframe')
	f.write('echo Start rendering Animation\n')
	f.write('echo Start Frame: '+str(startF)+'\n')
	f.write('echo End Frame: '+str(endF)+'\n')
	f.write('echo Total Frames: '+str(endF-startF+1)+'\n')
	if indigo_app == "indigo.exe":
		for i in range (startF, endF+1):
			f.write('echo Rendering Frame '+str(i)+'\n')
			string="%03d"%i
			f.write("start /min /belownormal /wait indigo.exe \""+relpath(indigopath, makename(filename, '-'+string+'.igs'))+"\" -t "+str(Threads.val)+" -o \""+relpath(indigopath, makename(filename,'-'+string+'.png'))+"\"\n")
	else:
		for i in range (startF, endF+1):
			string="%03d"%i
			f.write("start /min /belownormal /wait cmd.exe /c indigo_console.exe "+relpath(indigopath, makename(filename, '-'+string+'.igs'))+" -o "+relpath(indigopath, makename(filename,'-'+string+'.png'))+"\n")
	f.close()
	
	f=open(os.path.join(indigopath, "startAnim.cmd"), 'w')
	f.write("cd /d \"" + os.path.join(indigopath, "") + "\"\n")
	f.write("start AnimFrames.cmd"+"\n")
	f.close
	# call external shell script to start Indigo || win
	cmd = "start cmd /c \"" + os.path.join(indigopath, "startAnim.cmd") + "\""
	print cmd
	os.system(cmd)




### END OF PSOR ##########################################################


def getBoundingBox(mesh):
	bb = [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]
	if len(mesh.verts) > 0:
		v = mesh.verts[0]
		for i in range(3):
			bb[i][0] = v[i]
			bb[i][1] = v[i]
		for v in mesh.verts:
			for i in range(3):
				if bb[i][0] < v[i]:
					bb[i][0] = v[i]
				if bb[i][1] > v[i]:
					bb[i][1] = v[i]
	return bb

def exportMesh2(mesh, meshname, objmatrix, file):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	
	smooth = mesh.faces[0].smooth
	
	file.write("\t<mesh>\n")
	file.write("\t\t<name>%s</name>\n" %(meshname))
	
	# if the mesh is supposed to be smooth, then merge vertices to avoid creases.
	file.write("\t\t<merge_vertices_with_same_pos_and_normal>%s</merge_vertices_with_same_pos_and_normal>\n" %(IndigoBoolean(smooth)))
	
	# if the mesh is supposed to be smooth, then we want to use vertex shading normals, but we'll get Indigo to calc them.
	file.write("\t\t<normal_smoothing>%s</normal_smoothing>\n" %(IndigoBoolean(smooth)))
	
	try:
		d=mesh.properties['subdivision'].convert_to_pyobject()
	except:
		d = {}
	
	if meshname in d:
		subdiv_props=d[meshname]
		file.write("\t\t<max_num_subdivisions>%d</max_num_subdivisions>\n"%subdiv_props[0])
		file.write("\t\t<subdivide_pixel_threshold>%f</subdivide_pixel_threshold>\n"%subdiv_props[1])
		file.write("\t\t<subdivide_curvature_threshold>%f</subdivide_curvature_threshold>\n"%float(subdiv_props[2]))
		file.write("\t\t<subdivision_smoothing>%s</subdivision_smoothing>\n"%(IndigoBoolean(subdiv_props[3])))
	file.write("\t\t<embedded_2>\n")
	
	UVLayerNames=mesh.getUVLayerNames()
	uv_index=0
	for layer_name in UVLayerNames:
		file.write("\t\t\t<expose_uv_set>\n")
		file.write("\t\t\t\t<index>%d</index>\n"%uv_index)
		file.write("\t\t\t\t<name>%s</name>\n"%layer_name)
		file.write("\t\t\t</expose_uv_set>\n")
		uv_index+=1
	
	# Write used materials
	if len(mesh.materials) > 0:
		for mat in mesh.materials:
			if mat != None:
				trueMaterialName=mat.name
				try:
					d = mat.properties['indigo']
				except KeyError:
					d = {}
					convertMaterial(mat)
					d = mat.properties['indigo'].convert_to_pyobject()
				mType = readValue(d, 'type', 0)
				if (mType == mInclude):
					print "Mesh use external material"
					fname=readValue(d,'include','')
					if (os.path.isfile(fname)):
						import re
						mTag = re.compile('\s*<material>')
						mName = re.compile("\s*<name>([\w\s]*)</name>")
						print mName
						f=open(fname,'r')
						line=f.readline()
						insideMat = 0
						while line != "":
							if mTag.match(line):
								insideMat = 1
							m=mName.match(line)
							if (insideMat and m):
								trueMaterialName=m.group(1)
							line=f.readline()
						f.close()
				file.write("\t\t\t<used_material_name>%s</used_material_name>\n" % trueMaterialName)
				#print("<used_material_name>%s</used_material_name>\n" % trueMaterialName)
			else:
				file.write("\t\t\t<used_material_name>Default</used_material_name>\n")
	else:
		file.write("\t\t\t<used_material_name>Default</used_material_name>\n")
	
	pos = objmatrix.translationPart()
	scl = objmatrix.scalePart()
	# the blender euler rotation is wrong in some cases and don't match indigo rotation so here a new one:
	rot = Blender.Mathutils.Matrix(objmatrix[0]/scl[0],objmatrix[1]/scl[1],objmatrix[2]/scl[2])
	
	mat = None
	if (abs(scl[0]-scl[1])>1e-4) or (abs(scl[0]-scl[2])>1e-4):
		mat = Blender.Mathutils.Matrix()
		for v in range(3):
			for z in range(3):
				mat[z][v] = objmatrix[z][v]
		rot = Blender.Mathutils.Matrix()
		scalefact = 1.0
	else:
		scalefact = (scl[0] + scl[1] + scl[2]) / 3
	
	if mat:
		transpose_m=mat.copy()
		transpose_m.invert().transpose()
	
	# Write vertices
	for v in mesh.verts:
		if mat and mesh.users == 1:
			pos = Mathutils.Vector(v.co) * mat
			nor = Mathutils.Vector(v.no) * transpose_m
		else:
			pos = v.co
			nor = v.no
		file.write("\t\t\t<v p=\"%f %f %f\" n=\"%f %f %f\"/>\n" % (pos[0], pos[1], pos[2], nor[0], nor[1], nor[2]))


 	
 	# Sort faces by material used on the face
	material_faces = {}
	for face in mesh.faces:
		if face.mat not in material_faces:
			material_faces[face.mat] = []
		material_faces[face.mat].append(face)
	
	if mesh.faceUV:
		for (matindex, face_list) in material_faces.iteritems(): # For each group of faces sharing the same material
			for face in face_list:
				UVLayerNames=mesh.getUVLayerNames()
				uv_index=0
				UVLayers={}
				for layer in UVLayerNames:
					mesh.activeUVLayer=layer
					UVLayers[uv_index]=face.uv
					uv_index+=1
				face_index=0
				for z in face.uv:
					file.write("\t\t\t<uvs")
					activeUVLayer=mesh.activeUVLayer
					uv_index = 0
					for layer in UVLayerNames:
						file.write(" uv%d=\"%f %f\"" % (uv_index, UVLayers[uv_index][face_index][0], UVLayers[uv_index][face_index][1]))
						uv_index+=1
					mesh.activeUVLayer=activeUVLayer
					face_index+=1
					file.write("/>\n")
	
	# Write triangles
	uv_index = 0
	for (matindex, face_list) in material_faces.iteritems():
		for face in face_list:
			if mesh.faceUV:
				file.write("\t\t\t<t v=\"%d %d %d\" uv=\"%d %d %d\" m=\"%d\"/>\n" %(face.verts[0].index, face.verts[1].index, face.verts[2].index, uv_index, uv_index+1, uv_index+2, matindex))
				if len(face.verts) > 3:
					file.write("\t\t\t<t v=\"%d %d %d\" uv=\"%d %d %d\" m=\"%d\"/>\n" %(face.verts[0].index, face.verts[2].index, face.verts[3].index, uv_index, uv_index+2, uv_index+3, matindex))
				uv_index += len(face.verts)
			else:
				file.write("\t\t\t<t v=\"%d %d %d\" m=\"%d\"/>\n" %
						(face.verts[0].index, face.verts[1].index, face.verts[2].index, face.mat))
				if len(face.verts) > 3:
					file.write("\t\t\t<t v=\"%d %d %d\" m=\"%d\"/>\n" %
							(face.verts[0].index, face.verts[2].index, face.verts[3].index, face.mat))
	
	file.write("\t\t</embedded_2>\n")
	file.write("\t</mesh>\n\n")


def exportMeshBinary(mesh, meshname, mesh_save_dir, current_mesh_index, objmatrix, file, subdiv = 0, subdiv_values={}):
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	
	verbose = True
	if verbose: print("exportMeshBinary()")
	if verbose: start_time = Blender.sys.time()
	igmesh = indigo_mesh.IndigoMesh()
	
	filename = utils.makeSafeFilename(meshname) + "_" + str(current_mesh_index) + ".igmesh"
	path = os.path.join(mesh_save_dir, filename)
	
	# Write used materials
	if len(mesh.materials) > 0:
		for mat in mesh.materials:
			use_mat_name = ""
			if mat != None:
				use_mat_name = mat.name
				try:
					d = mat.properties['indigo']
				except KeyError:
					d = {}
					convertMaterial(mat)
					d = mat.properties['indigo'].convert_to_pyobject()
				mType = readValue(d, 'type', 0)
				if (mType == mInclude):
					print "Mesh use external material"
					fname=readValue(d,'include','')
					if (os.path.isfile(fname)):
						import re
						mTag = re.compile('\s*<material>')
						mName = re.compile("\s*<name>([\w\s]*)</name>")
						print mName
						f=open(fname,'r')
						line=f.readline()
						insideMat = 0
						while line != "":
							if mTag.match(line):
								insideMat = 1
							m=mName.match(line)
							if (insideMat and m):
								use_mat_name=m.group(1)
							line=f.readline()
						f.close()
			else:
				use_mat_name = "Default"
			
			igmesh.addUsedMaterial(use_mat_name)
	else:
		igmesh.addUsedMaterial("Default")
	
	
	pos = objmatrix.translationPart()
	scl = objmatrix.scalePart()
	# the blender euler rotation is wrong in some cases and don't match indigo rotation so here a new one:
	rot = Blender.Mathutils.Matrix(objmatrix[0]/scl[0],objmatrix[1]/scl[1],objmatrix[2]/scl[2])
	
	mat = None
	if (abs(scl[0]-scl[1])>1e-4) or (abs(scl[0]-scl[2])>1e-4):
		mat = Blender.Mathutils.Matrix()
		for v in range(3):
			for z in range(3):
				mat[z][v] = objmatrix[z][v]
		rot = Blender.Mathutils.Matrix()
		scalefact = 1.0
	else:
		scalefact = (scl[0] + scl[1] + scl[2]) / 3
	
	if mat:
		transpose_m = mat.copy()
		transpose_m.invert().transpose()
	
	# Write vertices
	for v in mesh.verts:
		if mat and mesh.users == 1:
			pos = Mathutils.Vector(v.co) * mat
			nor = Mathutils.Vector(v.no) * transpose_m
		else:
			pos = v.co
			nor = v.no
		igmesh.addVertPosition(pos[0], pos[1], pos[2])
		igmesh.addVertNormal(nor[0], nor[1], nor[2])

	
	# print("getUVLayerNames(): " + str(mesh.getUVLayerNames()))
	
	# Build up a list of arrays of UV coords.
	# Each layer gets its own array.
	# These layers will then be interleaved and saved in the igmesh.
	
	if mesh.faceUV:
		layer_uv_arrays = [] # A list of UV arrays, one array per layer.
		
		for name in mesh.getUVLayerNames(): # For Each UV Layer
			# Set current active layer.  TODO: test this works.
			mesh.activeUVLayer = name
			
			layer_uv_arrays.append(array.array('f')) # Construct an array to hold the UVs, add to list
			uv_array = layer_uv_arrays[len(layer_uv_arrays) - 1] # Get last elem of list.
			face_index = 0
			for face in mesh.faces: # For each face in the mesh
				for vert_index in range(len(face.verts)): # For each vert in the face
					uv_array.append(face.uv[vert_index][0]) # Add U
					uv_array.append(face.uv[vert_index][1]) # Add V
		
		# Add UVs to IGMESH
		num_uv_layers = len(mesh.getUVLayerNames())
		
		if num_uv_layers > 0:
			num_uv_pairs = len(layer_uv_arrays[0]) / 2
			for i in range(num_uv_pairs): # For each UV pair
				for layer_i in range(num_uv_layers): # For each UV layer
					igmesh.addUVPair(layer_uv_arrays[layer_i][i*2], layer_uv_arrays[layer_i][i*2 + 1])
					
		
		igmesh.setNumUVMappings(num_uv_layers)
		
		# Write UV exports
		uv_index = 0
		for name in mesh.getUVLayerNames():
			igmesh.addUVSetExposition(name, uv_index)
			uv_index += 1
	
	else: # Else if no UVs
		igmesh.setNumUVMappings(0)
		
	
	# Write triangles
	uv_index = 0
	face_index = 0
	for face in mesh.faces:
		igmesh.addTriangle(
			face.verts[0].index, face.verts[1].index, face.verts[2].index,
			uv_index, uv_index+1, uv_index+2,
			face.mat
			)
		
		if len(face.verts) > 3:
			igmesh.addTriangle(
				face.verts[0].index, face.verts[2].index, face.verts[3].index,
				uv_index, uv_index+2, uv_index+3,
				face.mat
				)
		
		face_index += 1
		uv_index += len(face.verts)
	
	if verbose: print("\tBuilding IGMESH object Done.  (Took " + str(Blender.sys.time() - start_time) + " s)")
	
	# Write IGMESH to disk.
	if verbose: print("Saving igmesh to '" + path + "...")
	if verbose: start_time = Blender.sys.time()
	
	igmesh.writeToDisk(path)
	
	if verbose: print("\tDone.  (Took " + str(Blender.sys.time() - start_time) + " s)")
	
	
	# Write the Mesh XML element, this will refer to the IGMESH stored on disk.
	
	try:
		d=mesh.properties['subdivision'].convert_to_pyobject()
	except:
		d = {}
	
	file.write("\t<mesh>\n");
	file.write("\t\t<name>" + meshname + "</name>\n");
	file.write("\t\t<normal_smoothing>" + IndigoBoolean(mesh.faces[0].smooth) + "</normal_smoothing>\n");
	
	if meshname in d:
		subdiv_props=d[meshname]
		file.write("\t\t<max_num_subdivisions>%d</max_num_subdivisions>\n"%subdiv_props[0])
		file.write("\t\t<subdivide_pixel_threshold>%f</subdivide_pixel_threshold>\n"%subdiv_props[1])
		file.write("\t\t<subdivide_curvature_threshold>%f</subdivide_curvature_threshold>\n"%float(subdiv_props[2]))
		file.write("\t\t<subdivision_smoothing>%s</subdivision_smoothing>\n"%(IndigoBoolean(subdiv_props[3])))
	
	file.write("\t\t<external>\n");
	file.write("\t\t\t<path>" + filename + "</path>\n");
	file.write("\t\t</external>\n");
	file.write("\t</mesh>\n");


def exportObject(obj, exportedobjlist, file, dup=0, mat=0):
	def sign(x):
		s = 0
		if x>1e-6: s = 1
		if x<-1e-6: s = -1
		return s
	def readValue(dict, name, default):
		try:
			r = dict[name]
		except KeyError:
			r = default
		return r
	#End sign
	
	ws = WorldScale.val*(10**WorldScaleExp.val)
	
	def exportDupObject(matrix, meshname, string, file):
		pos = matrix.translationPart()
		rot = matrix.rotationPart()
		
		file.write("\t<model>\n")
		file.write("\t\t<pos>%f %f %f</pos>\n" %(pos[0]*ws, pos[1]*ws, pos[2]*ws))
		file.write("\t\t<scale>%s</scale>\n" %ws)
		file.write("\t\t<rotation>\n")
		file.write("\t\t\t<matrix>\n")
		file.write("\t\t\t\t")
		for v in range(3):
				for z in range(3):
					file.write("%f " % (rot[z][v]))
		file.write("\n")
		file.write("\t\t\t</matrix>\n")
		file.write("\t\t</rotation>\n")
		file.write("\t\t<mesh_name>%s</mesh_name>\n" %(meshname))
		file.write(string)
		file.write("\t</model>\n\n")
#End exportDupObj
	
	matrix = obj.getMatrix()
	objectname = obj.getName()
	objecttype = obj.getType()
	objectdup = obj.enableDupGroup# or obj.enableDupVerts
	
	if not obj.enableDupVerts and ((objecttype == "Mesh") or (objecttype == "Curve") or (objecttype == "Text")):
		meshname=obj.getData(name_only=1)
		if(objecttype == "Curve"):
			meshname +='_cu'
		imeshname = meshname.replace("<","-").replace(">","-")
		
		pObj=obj.getParent()
		if pObj != None:
			if pObj.enableDupGroup or pObj.enableDupVerts or pObj.enableDupFaces:
				if pObj.DupObjects != []:
					print "%d Instances of %s found"%(len(pObj.DupObjects), objectname)
					
					try:
						matlist=obj.getData().materials
					except:
						matlist=[]
					
					string = ''
					if len(matlist) > 0:
						emitter_idx=0
						for mat_i in matlist:
							if mat_i != None:
								try:
									d = mat_i.properties['indigo'].convert_to_pyobject()
								except KeyError:
									d = {}
							if matIsEmitter(mat_i):
								string+="\t\t<emission_scale>\n"
								string+="\t\t\t<material_name>%s</material_name>\n"%matlist[emitter_idx].getName()
								string+="\t\t\t<measure>luminous_flux</measure>"
								string+="\t\t\t<value>%f</value>\n"%readValue(d,'emitter.scale_value',0)
								string+="\t\t</emission_scale>\n"
								
								if readValue(d,'emitter.ies',0) and readValue(d,'emitter.ies_path','') != '':
									string+="\t\t<ies_profile>\n"
									string+="\t\t\t<material_name>%s</material_name>\n"%matlists[emitter_idx].getName()
									string+="\t\t\t<path>%s</path>\n"%readValue(d,'emitter.ies_path','')
									string+="\t\t</ies_profile>\n"
							emitter_idx+=1
					
					for o, m in pObj.DupObjects:
						exportDupObject(m, imeshname, string, file)
					print "%d Instances of %s exported"%(len(pObj.DupObjects), objectname)

		
		mesh=getMeshFromObject(obj)
		if not mesh:
			return
		if len(mesh.faces) <= 0:
			return
		if (objecttype == "Curve") or (objecttype == "Text"):
			for f in mesh.faces:
				f.smooth = 1
		if (objecttype == "Text"):
			cu=Blender.Curve.Get(obj.getData().getName())
			mesh.materials = cu.getMaterials()
		if (objecttype == "Curve"):
			mesh.materials = obj.getData().getMaterials()
		emitter = 0
		exitportal = 0
		mat_idx=0
		if len(mesh.materials) > 0:
			for mat_i in mesh.materials:
				if matIsExitPortal(mat_i):
					exitportal = 1
				mat_idx=mat_idx+1
		if not mesh:
			print 'WARNING: ignoring object "%s", because linked mesh "%s" has no faces'%(objectname, meshname)
			return
		if len(mesh.faces) <= 0:
			print 'WARNING: ignoring object "%s", because linked mesh "%s" has no faces'%(objectname, meshname)
			return ""
		file.write("\t<!-- %s -->\n" %(objectname.replace("<","-").replace(">","-")))
		pos = matrix.translationPart()
		scl = matrix.scalePart()
		rot = matrix.rotationPart()
		# process indigo primitives
		if objectname.find(".sphere") != -1:
			bb = obj.getBoundBox()
			mat0=mesh.materials[0]
			radius = abs((bb[0]-bb[1])*scl)/2
			file.write("\t<sphere>\n")
			file.write("\t\t<center>%f %f %f</center>\n"%(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			file.write("\t\t<radius>%f</radius>\n"%radius)
			file.write("\t\t<material_name>%s</material_name>\n"%mat0.name)
			file.write("\t</sphere>\n\n")
		elif objectname.find('.plane') != -1:
			try:
				mat0_name=mesh.materials[0].name
			except:
				mat0_name="Default"
			dist =ws*pos[0]*rot[2][0]+pos[1]*rot[2][1]+pos[2]*rot[2][2]
			file.write("\t<plane>\n")
			file.write("\t\t<normal>%f %f %f</normal>\n"%(rot[2][0], rot[2][1], rot[2][2]))
			file.write("\t\t<dist>%f</dist>\n"%dist)
			file.write("\t\t<material_name>%s</material_name>\n"%mat0_name)
			file.write("\t</plane>\n\n")
		else: #not sphere and not plane
			if (abs(scl[0]-scl[1])>1e-4) or (abs(scl[0]-scl[2])>1e-4):
				scalefact = 1.0
			else:
				scalefact = 1.0
			if exitportal:
				file.write("\t<exit_portal>\n")
			else:
				file.write("\t<model>\n")

			from Blender import Ipo
			ipo = obj.getIpo()
			if ipo != None:
				icux = ipo[Ipo.OB_LOCX]
				icuy = ipo[Ipo.OB_LOCY]
				icuz = ipo[Ipo.OB_LOCZ]

				icurx = ipo[Ipo.OB_ROTX]
				icury = ipo[Ipo.OB_ROTY]
				icurz = ipo[Ipo.OB_ROTZ]

			startF = Blender.Get('curframe')
			if (not MoBlur.val):
				file.write("\t\t<pos>%f %f %f</pos>\n" %(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			elif (ipo == None):
				file.write("\t\t<pos>%f %f %f</pos>\n" %(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			elif (icux == None and icuy == None and icuz == None):
				file.write("\t\t<pos>%f %f %f</pos>\n" %(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			file.write("\t\t<scale>%s</scale>\n" %(scalefact*ws))
			file.write("\t\t<rotation>\n")
			file.write("\t\t\t<matrix>\n")
			file.write("\t\t\t\t")

			#if MoBlur.val and ipo != None and (icurx != None and icury != None and icurz != None):
			#	#take rotation matrix from ipo
			#	ax=icurx[startF]*10*math.pi/180
			#	ay=icury[startF]*10*math.pi/180
			#	az=icurz[startF]*10*math.pi/180
			#	rx=Mathutils.Matrix([1,0,0],[0,math.cos(ax),-1*math.sin(ax)],[0,math.sin(ax),math.cos(ax)])
			#	ry=Mathutils.Matrix([math.cos(ay),0,math.sin(ay)],[0,1,0],[-1*math.sin(ay),0,math.cos(ay)])
			#	rz=Mathutils.Matrix([math.cos(az),-1*math.sin(az),0],[math.sin(az),math.cos(az),0],[0,0,1])
			#	rot=(rx*ry*rz).invert()
			for v in range(3):
				for z in range(3):
					file.write("%f " % (rot[z][v]))
			file.write("\n")
			file.write("\t\t\t</matrix>\n")
			file.write("\t\t</rotation>\n")

			if MoBlur.val and ipo != None and (icux != None and icuy != None and icuz != None):

				fps=Scene.GetCurrent().getRenderingContext().fps
				exposure=float(1/float(Shutter.val))
				frames = exposure*fps
				frame_step=1.0/fps
				time_step=1.0/frames
				t=0.0
				#export first frame
				file.write("\t\t<keyframe>\n")
				file.write("\t\t\t<time>0</time>\n")
				file.write("\t\t\t<pos>%f %f %f</pos>\n"%(icux[startF]*ws,icuy[startF]*ws,icuz[startF]*ws))
				file.write("\t\t\t<rotation_quaternion>\n")
				file.write("\t\t\t\t<axis>0 1 0</axis>\n")
				file.write("\t\t\t\t<angle>0.0</angle>\n")
				file.write("\t\t\t</rotation_quaternion>\n")
				file.write("\t\t</keyframe>\n")

				v=Mathutils.Vector(0,0,0)
				v_orig=Mathutils.Vector(0,0,0)
				if frames > 1:
					for i in range(1,int(frames)):
						t=t+time_step
						file.write("\t\t<keyframe>\n")
						file.write("\t\t\t<time>%f</time>\n"%t)
						file.write("\t\t\t<pos>%f %f %f</pos>\n"%(ws*icux[startF+(frames/int(frames))*i],ws*icuy[startF+(frames/int(frames))*i],ws*icuz[startF+(frames/int(frames))*i]))
						file.write("\t\t\t<rotation_quaternion>\n")
						#v=Mathutils.Vector(icurx[1+frames]-icurx[1],icury[1+frames]-icury[frames],icurz[1+frames]-icurz[1])
						#v=Mathutils.Vector(icurx[1+t],icury[1+t],icurz[1+t])
						if(icurx != None and icury != None and icurz != None):
							v=Mathutils.Vector(icurx[startF+(frames/int(frames))*i],icury[startF+(frames/int(frames))*i],icurz[startF+(frames/int(frames))*i])
							v_orig=v*10
						else:
							v=Mathutils.Vector(0,0,0)
						if v != Mathutils.Vector(0,0,0):
							v.normalize()
						else:
							v=Mathutils.Vector(0,1,0)
						file.write("\t\t\t\t<axis>%f %f %f</axis>\n"%(v.x,v.y,v.z))
						if v != Mathutils.Vector(0,0,0):
							a = math.sqrt(v_orig.x**2+v_orig.y**2+v_orig.z**2)
							a = a * math.pi/180
						else:
							a=0.0
						file.write("\t\t\t\t<angle>%f</angle>\n"%a)
						file.write("\t\t\t</rotation_quaternion>\n")
						file.write("\t\t</keyframe>\n")

				#export last frame	
				#from math import sqrt as _sqrt
				file.write("\t\t<keyframe>\n")
				file.write("\t\t\t<time>1</time>\n")
				file.write("\t\t\t<pos>%f %f %f</pos>\n"%(ws*icux[startF+frames],ws*icuy[startF+frames],ws*icuz[startF+frames]))
				file.write("\t\t\t<rotation_quaternion>\n")
				if(icurx != None and icury != None and icurz != None):
					v=Mathutils.Vector(icurx[startF+frames]-icurx[startF],icury[startF+frames]-icury[startF],icurz[startF+frames]-icurz[startF])
					v_orig=v*10
					print v_orig
				else:
					v=Mathutils.Vector(0,0,0)
				if v != Mathutils.Vector(0,0,0):
					v.normalize()
				else:
					v=Mathutils.Vector(0,1,0)
				file.write("\t\t\t\t<axis>%f %f %f</axis>\n"%(v.x,v.y,v.z))
				print v
				if v != Mathutils.Vector(0,0,0):
					a = math.sqrt(v_orig.x**2+v_orig.y**2+v_orig.z**2)
					a = a * math.pi/180
				else:
					a=0.0
				file.write("\t\t\t\t<angle>%f</angle>\n"%a)
				file.write("\t\t\t</rotation_quaternion>\n")
				file.write("\t\t</keyframe>\n")

			file.write("\t\t<mesh_name>%s</mesh_name>\n" %(imeshname))
			#Emission Scale
			if len(mesh.materials) > 0:
				emitter_idx=0
				for mat_i in mesh.materials:
					if mat_i != None:
						try:
							d = mat_i.properties['indigo'].convert_to_pyobject()
						except KeyError:
							d = {}
					if matIsEmitter(mat_i):
						if readValue(d,'emitter.scale',0):
							file.write("\t\t<emission_scale>\n")
							file.write("\t\t\t<material_name>%s</material_name>\n"%mesh.materials[emitter_idx].getName())
							file.write("\t\t\t<measure>")
							value=readValue(d,'emitter.scale_measure',0)
							if value == 0: file.write("luminous_flux")
							elif value == 1: file.write("luminous_intensity")
							elif value ==	2: file.write("luminance")
							elif value ==	3: file.write("luminous_emittance")
							file.write("</measure>\n")
							file.write("\t\t\t<value>%f</value>\n"%readValue(d,'emitter.scale_value',0))
							file.write("\t\t</emission_scale>\n")
						if readValue(d,'emitter.ies',0) and readValue(d,'emitter.ies_path','') != '':
							file.write("\t\t<ies_profile>\n")
							file.write("\t\t\t<material_name>%s</material_name>\n"%mesh.materials[emitter_idx].getName())
							file.write("\t\t\t<path>%s</path>\n"%readValue(d,'emitter.ies_path',''))
							file.write("\t\t</ies_profile>\n")
					emitter_idx+=1
			if exitportal:
				file.write("\t</exit_portal>\n")
			else:
				file.write("\t</model>\n\n")
	elif (objectdup):
		print "object DupGroup"
		if (obj.enableDupGroup):
			group = obj.DupGroup
			groupname = group.name
		for o, m in obj.DupObjects:
			exportObject(o, exportedobjlist, file, 1, mat)

def getMeshFromObject(ob, container_mesh=None, apply_modifiers=True, vgroups=True, scn=None):
	'''
	ob - the object that you want to get the mesh from
	container_mesh - a Blender.Mesh type mesh that is reused to avoid a new datablock per call to getMeshFromObject
	apply_modifiers - if enabled, subsurf bones etc. will be applied to the returned mesh. disable to get a copy of the mesh.
	vgroup - For mesh objects only, apply the vgroup to the the copied mesh. (slower)
	scn - Scene type. avoids getting the current scene each time getMeshFromObject is called.
	
	Returns Mesh or None
	'''
	
	if not scn:
		scn = Blender.Scene.GetCurrent()
	if not container_mesh:
		mesh=Blender.Mesh.New(ob.name)
	else:
		mesh= container_mesh
		mesh.verts= None
	
	ob_type = ob.type
	dataname = ob.getData(1)
	tempob= None
	if apply_modifiers or ob_type != 'Mesh':
		try:
			mesh.getFromObject(ob,0,1)
		except:
			return None
	else:
		'''
		Dont apply modifiers, copy the mesh.
		So we can transform the data. its easiest just to get a copy of the mesh.
		'''
		tempob= scn.objects.new(ob.getData(mesh=1))
		mesh.getFromObject(tempob, render=1)
		scn.objects.unlink(tempob)
	tmesh=ob.getData(mesh=1)
	try:
		d=tmesh.properties['subdivision'].convert_to_pyobject()
	except:
		d = {}
	mesh.properties['subdivision']=d
	
	return mesh

### EXPORT TO FILES ####
# filename - file name where will be exported to
# objlist - objects to export
# meshlist - meshes to export
# matlist - materials to export
# includelist - file names that will be included
def exportIndigoScene(filename, objlist, meshlist, matlist, includelist):
	Window.WaitCursor(1)
	
	# NICK NEWCODE
	mesh_save_dir = os.path.dirname(filename)
	current_mesh_index = 0
	
	ws = WorldScale.val*(10**WorldScaleExp.val)
	file = open(filename, 'w')
	file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
	file.write('<scene>\n')
	
	# write render_settings
	file.write("\t<renderer_settings>\n")
	file.write("\t\t<width>%d</width>\n" % (SizeX.val*ScaleSize.val/100))
	file.write("\t\t<height>%d</height>\n" % (SizeY.val*ScaleSize.val/100))
	if not Override.val:
		file.write("\t\t<metropolis>%s</metropolis>\n" %(IndigoBoolean(MLT.val)))
#		if MLT.val:
#			file.write("\t\t<large_mutation_prob>%f</large_mutation_prob>\n"%LMP.val)
#		file.write("\t\t<max_change>%f</max_change>\n"%MaxChange.val)
		file.write("\t\t<max_num_consec_rejections>%d</max_num_consec_rejections>\n" % MaxNumConsRej.val)
		file.write("\t\t<bidirectional>%s</bidirectional>\n" %(IndigoBoolean(Bidirectional.val)))
#		file.write("\t\t<max_depth>%s</max_depth>\n" % MaxDepth)
		file.write("\t\t<logging>%s</logging>\n" %(IndigoBoolean(Logging.val)))
		file.write("\t\t<info_overlay>%s</info_overlay>\n" %(IndigoBoolean(IOverlay.val)))
		file.write("\t\t<save_untonemapped_exr>%s</save_untonemapped_exr>\n" %(IndigoBoolean(SaveUTMExr.val)))
		file.write("\t\t<save_tonemapped_exr>%s</save_tonemapped_exr>\n" %(IndigoBoolean(SaveTMExr.val)))
		file.write("\t\t<save_igi>%s</save_igi>\n" %(IndigoBoolean(SaveIgi.val)))
		file.write("\t\t<display_period>%s</display_period>\n" % float(DisplayPeriod.val))
		file.write("\t\t<halt_time>%s</halt_time>\n" % (max(HaltTime.val, -1)))
		file.write("\t\t<halt_samples_per_pixel>%s</halt_samples_per_pixel>\n" % HaltSamples.val)
#		file.write("\t\t<frame_upload_period>%s</frame_upload_period>\n" %(FrameUp.val))
		file.write("\t\t<image_save_period>%s</image_save_period>\n" %(ImageSave.val))
		file.write("\t\t<super_sample_factor>%s</super_sample_factor>\n" %(SuperSample.val))
		file.write("\t\t<hybrid>%s</hybrid>\n" %(IndigoBoolean(Hybrid.val)))
		file.write("\t\t<bih_tri_threshold>%s</bih_tri_threshold>\n" %(BIH.val))
		file.write("\t\t<ray_origin_nudge_distance>%de%d</ray_origin_nudge_distance>\n"%(RayNudgeMant.val,RayNudgeExp.val))
#		file.write("\t\t<cache_trees>%s</cache_trees>\n" %(IndigoBoolean(CacheTree.val)))
		
		#file.write("\t\t<num_threads>%d</num_threads>\n"%Threads.val)
		
		file.write("\t\t<auto_choose_num_threads>%s</auto_choose_num_threads>\n"%(IndigoBoolean(AutoChoose.val)))
		file.write("\t\t<splat_filter>\n")
		if SplatFilter.val == 0:
			file.write("\t\t\t<box/>\n")
		if SplatFilter.val == 1:
			file.write("\t\t\t<gaussian></gaussian>\n")
		if SplatFilter.val == 2:
			file.write("\t\t\t<mn_cubic>\n")
			file.write("\t\t\t\t<blur>%f</blur>\n"%SplatF_MN_Blur.val)
			file.write("\t\t\t\t<ring>%f</ring>\n"%SplatF_MN_Ring.val)
			file.write("\t\t\t</mn_cubic>\n")
		file.write("\t\t</splat_filter>\n")
		file.write("\t\t<downsize_filter>\n")
		if DownsizeFilter.val == 0:
			file.write("\t\t\t<box/>\n")
		if DownsizeFilter.val == 1:
			file.write("\t\t\t<gaussian></gaussian>\n")
		if DownsizeFilter.val == 2:
			file.write("\t\t\t<mn_cubic>\n")
			file.write("\t\t\t\t<blur>%f</blur>\n"%DownsizeF_MN_Blur.val)
			file.write("\t\t\t\t<ring>%f</ring>\n"%DownsizeF_MN_Ring.val)
			file.write("\t\t\t</mn_cubic>\n")
		file.write("\t\t</downsize_filter>\n")
		file.write("\t\t<aperture_diffraction>%s</aperture_diffraction>\n"%(IndigoBoolean(ApertureDiffr.val)));
	file.write("\t\t<watermark>%s</watermark>\n"%(IndigoBoolean(Watermark.val)));
	if Scene.GetCurrent().getRenderingContext().borderRender:
		region=Scene.GetCurrent().getRenderingContext().border
		file.write("\t\t<render_region>\n")
		file.write("\t\t\t<x1>%d</x1>\n"%((SizeX.val*ScaleSize.val/100)*region[0]))
		file.write("\t\t\t<y1>%d</y1>\n"%((SizeY.val*ScaleSize.val/100)-(SizeY.val*ScaleSize.val/100)*region[3]))
		file.write("\t\t\t<x2>%d</x2>\n"%((SizeX.val*ScaleSize.val/100)*region[2]))
		file.write("\t\t\t<y2>%d</y2>\n"%((SizeY.val*ScaleSize.val/100)-(SizeY.val*ScaleSize.val/100)*region[1]))
		file.write("\t\t</render_region>\n")
	if AlphaMask.val:
		file.write("\t\t<render_foreground_alpha>true</render_foreground_alpha>\n")
	file.write("\t</renderer_settings>\n\n")
	
	# write tone mapping
	if ToneMapType.val == 0:
		if (Autoexp.val == 0):
			exposure = ToneMapScale.val * (10**ToneMapScaleExp.val)
		else:
			if Shutter.val == 0:
				print "WARNING: Shutter is not set (using 125)"
				Shutter.val = 125
			exposure = ToneMapScale.val * (10**ToneMapScaleExp.val) * float(FilmIso.val) / float(Shutter.val)
		file.write("\t<tonemapping>\n")
		file.write("\t\t<linear>\n")
		file.write("\t\t\t<scale>%g</scale>\n" %exposure)
		file.write("\t\t</linear>\n")
		file.write("\t</tonemapping>\n\n")
	if ToneMapType.val == 1:
		file.write("\t<tonemapping>\n")
		file.write("\t\t<reinhard>\n")
		file.write("\t\t\t<pre_scale>%g</pre_scale>\n" %ToneMapPreScale.val)
		file.write("\t\t\t<post_scale>%g</post_scale>\n" %ToneMapPostScale.val)
		file.write("\t\t\t<burn>%g</burn>\n" %ToneMapBurn.val)
		file.write("\t\t</reinhard>\n")
		file.write("\t</tonemapping>\n\n")
	if ToneMapType.val == 2:
		file.write("\t<tonemapping>\n")
		file.write("\t\t<camera>\n");
		file.write("\t\t<ev_adjust>%g</ev_adjust>\n"%ToneMapEV.val)
		file.write("\t\t<film_iso>%g</film_iso>\n"%float(FilmIso.val))
		if ToneMapResponseCurve.val:
			try:
				file.write("\t\t<response_function_path>data/camera_response_functions/%s.txt</response_function_path>\n"%tonemapcurves[ToneMapResponseCurve.val])
			except:
				file.write("\t\t<response_function_path>data/camera_response_functions/dscs315.txt</response_function_path>\n")
		else:
			file.write("\t\t<response_function_path>data/camera_response_functions/dscs315.txt</response_function_path>\n")
		file.write("\t\t</camera>\n")
		file.write("\t</tonemapping>\n\n")
	
	# write Color background
	if EnvType.val == 0:
		#worldcolor = Blender.World.Get('World').getHor()
		file.write("\t<background>\n")
		file.write("\t\t<spectrum>\n")
		file.write("\t\t\t<rgb>\n")
		file.write("\t\t\t\t<rgb>%g %g %g</rgb>\n" %(BGColor.val[0]*BGGain.val, BGColor.val[1]*BGGain.val, BGColor.val[2]*BGGain.val))
		file.write("\t\t\t\t<gamma>2.2</gamma>\n")
		file.write("\t\t\t</rgb>\n")
		file.write("\t\t</spectrum>\n")
		file.write("\t</background>\n\n")
	
	##### Process sun ######
	if EnvType.val == 1:
		flag_sun = 0
		for obj in  objlist:
			if obj.getType() == "Lamp":
				if obj.data.getType() == 0 and not flag_sun: # lamp object
					invmatrix = Mathutils.Matrix(obj.getInverseMatrix())
					file.write("\t<skylight>\n")
					file.write("\t\t<sundir>%f %f %f</sundir>\n" % (invmatrix[0][2], invmatrix[1][2], invmatrix[2][2]))
					file.write("\t\t<turbidity>%f</turbidity>\n" %Turbidity.val)
					if ExtraAtm.val:
						file.write("\t\t<extra_atmospheric>%s</extra_atmospheric>\n" %IndigoBoolean(ExtraAtm.val))
					if SunLayerToggle.val:
						file.write("\t\t<sun_layer>%d</sun_layer>\n"%SunLayer.val)
					if SkyLayerToggle.val:
						file.write("\t\t<sky_layer>%d</sky_layer>\n"%SkyLayer.val)
					file.write("\t</skylight>\n\n")
					flag_sun=1
				if obj.data.getType() == 1 and not flag_sun: # sun object
					invmatrix = Mathutils.Matrix(obj.getInverseMatrix())
					file.write("\t<skylight>\n")
					file.write("\t\t<sundir>%f %f %f</sundir>\n" % (invmatrix[0][2], invmatrix[1][2], invmatrix[2][2]))
					file.write("\t\t<turbidity>%f</turbidity>\n" %Turbidity.val)
					if ExtraAtm.val:
						file.write("\t\t<extra_atmospheric>%s</extra_atmospheric>\n" %IndigoBoolean(ExtraAtm.val))
					if SunLayerToggle.val:
						file.write("\t\t<sun_layer>%d</sun_layer>\n"%SunLayer.val)
					if SkyLayerToggle.val:
						file.write("\t\t<sky_layer>%d</sky_layer>\n"%SkyLayer.val)
					file.write("\t</skylight>\n\n")
					flag_sun=1
		if not flag_sun:
			#Draw.PupMenu("Error%t|No Sun found");
			file.write("\t<background>\n")
			file.write("\t\t<spectrum>\n")
			file.write("\t\t\t<uniform>\n")
			file.write("\t\t\t\t<value>1.0</value>\n")
			file.write("\t\t\t</uniform>\n")
			file.write("\t\t</spectrum>\n")
			file.write("\t</background>\n\n")
			BGColor.val=(1.0,1.0,1.0)
			EnvType.val=0
	
	##### Process Env Map #####
	if EnvType.val == 2:
		if EnvFile.val != "none" and EnvFile.val != "":
			file.write("\t<env_map>\n")
			file.write("\t\t<%s>\n" %(EnvMapTypeV[EnvMapType.val]))
			file.write("\t\t\t<path>%s</path>\n" %(relpath(file.name, EnvFile.val)))
			#file.write("\t\t\t<path>%s</path>\n" %os.path.relpath(EnvFile.val,file.name))
			file.write("\t\t\t<gain>%g</gain>\n" %(EnvGain.val*1e4)) #scaled for using with meshlight
			if EnvMapType.val == 0:
				file.write("\t\t\t<width>%d</width>\n" %(EnvWidth.val))
			file.write("\t\t</%s>\n" %(EnvMapTypeV[EnvMapType.val]))
			file.write("\t</env_map>\n\n")

	
	##### Process camera ######
	currentscene = Scene.GetCurrent()
	camObj = currentscene.getCurrentCamera()
	if camObj:
		matrix = camObj.getMatrix()
		pos = matrix[3]
		forwards = matrix[2]
		up = matrix[1]
		file.write("\t<camera>\n")
		if not CamMoBlur.val:
			file.write("\t\t<pos>%f %f %f</pos>\n" % (pos[0]*ws, pos[1]*ws, pos[2]*ws))
		file.write("\t\t<up>%f %f %f</up>\n" % (up[0], up[1], up[2]))
		file.write("\t\t<forwards>%f %f %f</forwards>\n" % (-forwards[0], -forwards[1], -forwards[2]))
		aspect = float(SizeX.val) / float(SizeY.val)
		film = 0.001*FilmWidth.val
		
		f_stop = float(FStop.val) * 0.1
		FOV=camObj.data.angle
		if aspect < 1:
			FOV = FOV*aspect
		lens_sensor_dist = film/(2*math.tan(FOV*math.pi/360))
		
		if FStop.val == 0:
			print "WARNING: FStop is not set (using 8.0)"
			FStop.val = 80
		aperture_radius = lens_sensor_dist / (2*f_stop)
		file.write("\t\t<aperture_radius>%f</aperture_radius>\n" % aperture_radius)
		exposure=float(1/float(Shutter.val))
		file.write("\t\t<exposure_duration>%f</exposure_duration>\n"%exposure)
		
		from Blender import Ipo
		focus_dist=FocusDistance.val
		try:
			caIpo = Ipo.Get('CaIpo')
		except:
			caIpo = None
		
		if caIpo != None and CamFocus.val == 1:
			icu = caIpo[Ipo.CA_FDIST ]
			if icu != None and len(icu.bezierPoints) > 0:
				focus_dist=icu[Blender.Get('curframe')]
		
		file.write("\t\t<focus_distance>%f</focus_distance>\n" % focus_dist)
		file.write("\t\t<aspect_ratio>%g</aspect_ratio>\n" % aspect)
		file.write("\t\t<sensor_width>%f</sensor_width>\n" % film)
		file.write("\t\t<lens_sensor_dist>%f</lens_sensor_dist>\n" % lens_sensor_dist)
		file.write("\t\t<white_balance>%s</white_balance>\n" % whiteBalanceV[WhiteBalance.val])
		file.write("\t\t<aperture_shape>\n")
		if ApertureType.val == 1:
			file.write("\t\t\t<circular></circular>\n")
		elif ApertureType.val == 2:
			file.write("\t\t\t<generated>\n")
			file.write("\t\t\t\t<num_blades>%d</num_blades>\n"%BladesNum.val)
			file.write("\t\t\t\t<blade_offset>%f</blade_offset>\n"%float(BladeOffset.val))
			file.write("\t\t\t\t<blade_curvature_radius>%f</blade_curvature_radius>\n"%float(BladeCurvature.val))
			file.write("\t\t\t\t<start_angle>%f</start_angle>\n"%float(StartAngle.val))
			file.write("\t\t\t</generated>\n")
		elif ApertureType.val == 3:
			file.write("\t\t\t<image>\n")
			file.write("\t\t\t<path>%s</path>\n"%(relpath(Blender.sys.dirname(IndigoPath.val), ApertureImg.val)))
			file.write("\t\t\t</image>\n")
		file.write("\t\t</aperture_shape>\n")
		if ApertureType < 2 and not ObstacleMap.val == '':
			file.write("\t\t<obstacle_map>\n")
			file.write("\t\t\t<path>%s</path>\n"%(relpath(Blender.sys.dirname(IndigoPath.val), ObstacleMap.val)))
			file.write("\t\t</obstacle_map>\n")
		if LensShiftUp.val != 0:
			file.write("\t\t<lens_shift_up_distance>%f</lens_shift_up_distance>\n"%float(LensShiftUp.val))
		if LensShiftRight.val != 0:
			file.write("\t\t<lens_shift_right_distance>%f</lens_shift_right_distance>\n"%float(LensShiftRight.val))
		if AutoFocus.val==1:
			file.write("\t\t<autofocus/>\n")
		
		if CamMoBlur.val:
			ipo = camObj.getIpo()
			
			icux = ipo[Ipo.OB_LOCX]
			icuy = ipo[Ipo.OB_LOCY]
			icuz = ipo[Ipo.OB_LOCZ]

			
			icurx = ipo[Ipo.OB_ROTX]
			icury = ipo[Ipo.OB_ROTY]
			icurz = ipo[Ipo.OB_ROTZ]
			
			if ipo != None:
				fps=currentscene.getRenderingContext().fps
				exposure=float(1/float(Shutter.val))
				frames = exposure*fps
				frame_step=1.0/fps
				time_step=1.0/frames
				t=0.0
				print("frames: %f"%frames)
				print("frame_step: %f"%frame_step)
				#export first frame
				file.write("\t\t<keyframe>\n")
				file.write("\t\t\t<time>0</time>\n")
				if icux != None and icuy != None and icuz != None:
					file.write("\t\t\t<pos>%f %f %f</pos>\n"%(ws*icux[1],ws*icuy[1],ws*icuz[1]))
				else:
					matrix = camObj.getMatrix()
					pos = matrix[3]
					file.write("\t\t\t<pos>%f %f %f</pos>\n" % (pos[0]*ws, pos[1]*ws, pos[2]*ws))
				file.write("\t\t\t<rotation_quaternion>\n")
				file.write("\t\t\t\t<axis>0 1 0</axis>\n")
				file.write("\t\t\t\t<angle>0.0</angle>\n")
#				if(icurx != None and icury != None and icurz != None):
#					v=Mathutils.Vector(icurx[1],icury[1],icurz[1])
#					v_orig=v*10
#				else:
#					v=Mathutils.Vector(0,0,0)
#				if v != Mathutils.Vector(0,0,0):
#					v.normalize()
#				else:
#					v=Mathutils.Vector(0,1,0)
#				file.write("\t\t\t\t<axis>%f %f %f</axis>\n"%(v.x,v.y,v.z))
#				if v != Mathutils.Vector(0,0,0):
#					a = math.sqrt(v_orig.x**2+v_orig.y**2+v_orig.z**2)
#					a = a * math.pi/180
#				else:
#					a=0.0
#				file.write("\t\t\t\t<angle>%f</angle>\n"%a)
				file.write("\t\t\t</rotation_quaternion>\n")
				file.write("\t\t</keyframe>\n")
				
				v=Mathutils.Vector(0,0,0)
				v_orig=Mathutils.Vector(0,0,0)
				
				startF = Blender.Get('curframe')
				
				if frames > 1:
					for i in range(1,int(frames)):
						t=t+time_step
						posx=icux[startF+(frames/int(frames))*i]
						posy=icuy[startF+(frames/int(frames))*i]
						posz=icuz[startF+(frames/int(frames))*i]
						file.write("\t\t<keyframe>\n")
						file.write("\t\t\t<time>%f</time>\n"%t)
						file.write("\t\t\t<pos>%f %f %f</pos>\n"%(ws*posx,ws*posy,ws*posz))
						file.write("\t\t\t<rotation_quaternion>\n")
						#file.write("\t\t\t\t<axis>0 1 0</axis>\n")
						#file.write("\t\t\t\t<angle>0.0</angle>\n")
						#file.write("\t\t\t</rotation_quaternion>\n")
						if(icurx != None and icury != None and icurz != None):
							v=Mathutils.Vector(icurx[startF+(frames/int(frames))*i]-icurx[startF],icury[startF+(frames/int(frames))*i]-icury[startF],icurz[startF+(frames/int(frames))*i]-icurz[startF])
							v_orig=v*10
						else:
							v=Mathutils.Vector(0,0,0)
						if v != Mathutils.Vector(0,0,0):
							v.normalize()
						else:
							v=Mathutils.Vector(0,1,0)
						file.write("\t\t\t\t<axis>%f %f %f</axis>\n"%(v.x,v.y,v.z))
						if v != Mathutils.Vector(0,0,0):
							a = math.sqrt(v_orig.x**2+v_orig.y**2+v_orig.z**2)
							a = a * math.pi/180
						else:
							a=0.0
						file.write("\t\t\t\t<angle>%f</angle>\n"%a)
						file.write("\t\t\t</rotation_quaternion>\n")
						file.write("\t\t</keyframe>\n")

				#export last frame	
				file.write("\t\t<keyframe>\n")
				file.write("\t\t\t<time>1</time>\n")
				#file.write("\t\t\t<pos>%f %f %f</pos>\n"%(ws*icux[1+frames],ws*icuy[1+frames],ws*icuz[1+frames]))
				if icux != None and icuy != None and icuz != None:
					file.write("\t\t\t<pos>%f %f %f</pos>\n"%(ws*icux[startF+frames],ws*icuy[startF+frames],ws*icuz[startF+frames]))
				else:
					matrix = camObj.getMatrix()
					pos = matrix[3]
					file.write("\t\t\t<pos>%f %f %f</pos>\n" % (pos[0]*ws, pos[1]*ws, pos[2]*ws))
				file.write("\t\t\t<rotation_quaternion>\n")
				#file.write("\t\t\t\t<axis>0 1 0</axis>\n")
				#file.write("\t\t\t\t<angle>0.0</angle>\n")
				if(icurx != None and icury != None and icurz != None):
					v=Mathutils.Vector(icurx[startF+frames]-icurx[startF],icury[startF+frames]-icurx[startF],icurz[startF+frames]-icurx[startF])
					v_orig=v*10
				else:
					v=Mathutils.Vector(0,0,0)
				if v != Mathutils.Vector(0,0,0):
					v.normalize()
				else:
					v=Mathutils.Vector(0,1,0)
				file.write("\t\t\t\t<axis>%f %f %f</axis>\n"%(v.x,v.y,v.z))
				if v != Mathutils.Vector(0,0,0):
					a = math.sqrt(v_orig.x**2+v_orig.y**2+v_orig.z**2)
					a = a * math.pi/180
				else:
					a=0.0
				file.write("\t\t\t\t<angle>%f</angle>\n"%a)
				file.write("\t\t\t</rotation_quaternion>\n")
				file.write("\t\t</keyframe>\n")
		file.write("\t</camera>\n\n")
    
    # Seperate materials file?
	if (SepMatFile.val):
		matsfn = Blender.sys.makename(filename, '-mats.xml')
		matsfile=open(matsfn,'w')
		matsfile.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
		matsfile.write('<scenedata>\n')
	else:
	    matsfile = file
	
	matsfile.write("\t<material>\n")
	matsfile.write("\t\t<name>Default</name>\n")
	matsfile.write("\t\t<diffuse>\n")
	matsfile.write("\t\t\t<albedo>\n\t\t\t\t<constant><rgb>\n\t\t\t\t\t<rgb>0.95 0.95 0.95</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb></constant>\n\t\t\t</albedo>\n")
	matsfile.write("\t\t</diffuse>\n")
	matsfile.write("\t</material>\n\n")
	matsfile.write("\t<material>\n")
	matsfile.write("\t\t<name>null</name>\n")
	matsfile.write("\t\t<null_material/>\n")
	matsfile.write("\t</material>\n\n")
	exportedmatlist = ["Default", "null"]
	includelist = []
	
	for obj in objlist:
		try:
			if obj.getType() == "Text":
				cu=Blender.Curve.Get(obj.getData().getName())
				matlist=cu.getMaterials()
			else:
				matlist=obj.getData().getMaterials()
		except:
			matlist=[]
		
		for mat in matlist:
			matname = mat.getName()
			print("  * " + matname)
			
			try:
				exportedmatlist.index(matname)
			except:
				exportMaterial(mat, matsfile, exportedmatlist, includelist)
				exportedmatlist.append(matname)
				
		if len(includelist) > 0:
			for i in includelist:
				matsfile.write("\t<include>\n")
				matsfile.write("\t\t<pathname>%s</pathname>\n"%i)
				matsfile.write("\t</include>\n\n")
    
    # Seperate materials file?
	if (SepMatFile.val):
		matsfile.write('</scenedata>\n')
		matsfile.close
		
		print("Writing relative matfile %s to filename %s"%(matsfile.name, filename))
		relmatsfn = relpath(filename, matsfile.name)
		file.write("\t<include>\n")
		file.write("\t\t<pathname>%s</pathname>\n"%relmatsfn)
		file.write("\t</include>\n\n")

	exportedmeshlist=[]
	for obj in objlist:
		objecttype = obj.getType()
		if (objecttype == "Mesh")  or (objecttype == "Curve") or (objecttype == "Text"):
			meshname = obj.getData(name_only=1)
			if(objecttype == "Curve"):
				meshname +='_cu'
			mesh=getMeshFromObject(obj)
			if not mesh:
				continue
			if len(mesh.faces) <= 0:
				continue
			if (objecttype == "Curve"):
				mesh.materials = obj.getData().getMaterials()
			if (objecttype == "Text"):
				curveName=obj.getData().getName()
				mesh.materials = Blender.Curve.Get(curveName).getMaterials()
			try:
				exportedmeshlist.index(meshname)
			except:
				uvlayer='default'
				uvflag=0
				if len(mesh.getUVLayerNames()) != 0:
					uvlayer=mesh.getUVLayerNames()
					uvflag=1
				for mat in mesh.materials:
					try:
						d = mat.properties['indigo']
					except KeyError:
						d = {}
						convertMaterial(mat)
					d = mat.properties['indigo'].convert_to_pyobject()
					if not uvflag and ('albedo.path' in d and d['albedo.path'] != '' or 'bamp.path' in d and d['bump.path'] != '' or 'displace.path' in d and d['displace.path'] != ''):
						Window.WaitCursor(0)
						message="Error%t|"
						message+="Material '%s' in Mesh '%s' has no UVset!"%(mat.getName(),meshname)
						Draw.PupMenu(message,20)
						return 1
					#print d['albedo.uv_set']
				exportMeshBinary(mesh, meshname, mesh_save_dir, current_mesh_index, obj.getMatrix(), file)
				current_mesh_index += 1
				#exportMesh2(mesh, meshname, obj.getMatrix(), file)def readValue(dict, name, default):
				exportedmeshlist.append(meshname)
	
	exportedobjlist=[]
	for obj in objlist:
		try:
			exportedobjlist.index(obj.getName())
		except:
			exportObject(obj, exportedobjlist, file)
			exportedobjlist.append(obj.getName())

	
	file.write('</scene>\n')
	file.close()
	Window.WaitCursor(0)
	return 0


def exportMatPreview(mat):
	fileName = os.path.join(utils.getTempDir(), 'preview_scene.igs')
	
	f = open(fileName, "w")
	f.write("<?xml version='1.0' encoding='utf-8'?>\n<scene>\n")
	
	exportMaterial(mat, f, ["Default", "null"], [], "preview_material", 'default')
	
	f.write("\n\t<include>\n\t\t<pathname>" + indigo.getPreviewScenePath() + "</pathname>\n\t</include>\n</scene>")
	f.close()
	
	indigo.launchGui(fileName,"")



def export(filename, anim):
	global IndigoFilePath, SaveIgi
	IndigoFilePath=filename
	print("\nINDIGO EXPORT\n")
	time1 = Blender.sys.time()
	filepath = Blender.sys.dirname(filename)
	
	objlist = []
	matlist = []
	meshlist = []
	Draw.Draw()#Force an immediate redraw
	
	if anim:
		startF = Blender.Get('staframe')
		endF = Blender.Get('endframe')
		print('start %d end %d'%(startF,endF))
		for i in range(startF, endF+1):
			print('exporting frame %d\n'%i)
			Blender.Set('curframe', i)
			Blender.Redraw()
			activelayers = Window.ViewLayers()
			for obj in Blender.Scene.GetCurrent().getChildren():
				visible = 0
				for layer in obj.layers:
					if layer in activelayers:
						visible = 1
				if visible:
					objlist.append(obj)
			string="%03d"% i
			framefilename = makename(filename, "-"+string+".igs")
			include = []
			exportIndigoScene(framefilename, objlist, meshlist, matlist, include)
			del objlist[:]#delete all list items
			del meshlist[:]
			del matlist[:]
		
		Blender.Set('curframe', endF)
		Blender.Redraw()
	else:
		#get all the objects in this scene in visible layers
		activelayers = Window.ViewLayers()
		for obj in Blender.Scene.GetCurrent().objects:
			visible = 0
			for layer in obj.layers:
				if layer in activelayers:
					visible = 1
			if visible:
				objlist.append(obj)
		include = []
		if exportIndigoScene(filename, objlist, meshlist, matlist, include):
			return
		if ExportMeshs.val:
			exportMeshs_flag=1
		else:
			meshsxml = ''
			matlist = []
			for m in Material.Get():
				matlist.append(m.getName())
	time2 = Blender.sys.time()
	print("Processing time: %f\n" %(time2-time1))
	options=""
	if not(anim) and (ExecuteIndigo.val == 1):
		if not AutoChoose.val:
			options+=" -t %d"%Threads.val
		if (IndigoOutPath.val != "")  and os.path.exists(os.path.dirname(IndigoOutPath.val)):
			options+=" -o \""+IndigoOutPath.val+"\""
			if SaveIgi.val:
				import string
				options+=" -igio \""+os.path.join(os.path.dirname(IndigoOutPath.val),string.split(os.path.basename(IndigoOutPath.val),'.')[0]+".igi")+"\""
		indigo.launchGui(filename,options, ProcessPrio.val)
	if (anim) and (ExecuteIndigo.val == 1):
		print("Exporting Animation")
		if not AutoChoose.val:
			indigo.launchAnimation(filename, Threads.val, IndigoOutPath.val)
		else:
			indigo.launchAnimation(filename, 0, IndigoOutPath.val)
		#if (IndigoOutPath.val != "")  and os.path.exists(os.path.dirname(IndigoOutPath.val)):
		#	options+=" -o \""+IndigoOutPath.val+"\""

def exportStill(filename):
	print("Export Still to %s"%filename)
	export(filename, 0)

def exportAnim(filename):
	print "Export Animation"
	export(filename, 1)

print ""
print "----------------------"
print "BLENDIGO v" + blendigoVersion
print "----------------------"

linux = 0
if sys.platform[:3] == 'win':
	print "  Windows-platform: "+sys.platform+"   os.sep: "+os.sep
else:
	linux = 1
	print "  Linux-platform: "+sys.platform+"  os.sep: "+os.sep



######################################################
# Settings GUI
######################################################

ExecuteIndigo = Draw.Create(1)
DefaultExport = Draw.Create(1)


# Assign event numbers to buttons
evtNoEvt	= 0
evtExport   = 1
evtExportAnim   = 2
evtIsNumber = 3
evtFocusA = 23
evtFocusS = 4
evtFocusC = 5
evtFocusD = 20
openCamera = 6
openEnv = 7
openRSet = 8
openSSet = 9
openTmap = 10
evtloadimg = 11
evtchangesize = 12
evtHStop = 13
openMat = 14
evtloadobstacle = 15
evtloadaperture = 16
openMesh = 17
evtSubdivMesh = 18
evtUpdateSubdiv = 19
#evtSetIndigoPath = 21
evtSetIndigoOutPath = 22

# Set initial values of buttons

sceneSizeX = Scene.GetCurrent().getRenderingContext().imageSizeX()
sceneSizeY = Scene.GetCurrent().getRenderingContext().imageSizeY()

SizeX = Draw.Create(sceneSizeX)
SizeY = Draw.Create(sceneSizeY)

strScaleSize = "Render scale:%t | 200 % %x200 | 150 % %x150 | 100 % %x100 | 75 % %x75 | 50 % %x50 | 25 % %x25 | 20 % %x20 | 10 % %x10 "
ScaleSize = Draw.Create(100)

ColExponent = Draw.Create(2.2)

AlphaMask=Draw.Create(0)

SplatFilter=Draw.Create(2)
SplatF_MN_Blur=Draw.Create(0.3)
SplatF_MN_Ring=Draw.Create(0.3)
DownsizeFilter=Draw.Create(2)
DownsizeF_MN_Blur=Draw.Create(0.3)
DownsizeF_MN_Ring=Draw.Create(0.3)

MLT = Draw.Create(0)

#LMP = Draw.Create(0.4)

#MaxChange = Draw.Create(0.02)

MaxNumConsRej = Draw.Create(1000)

#MaxDepth = Draw.Create(1000)

Bidirectional = Draw.Create(1)

Threads = Draw.Create(1)

Logging = Draw.Create(1)
IOverlay=Draw.Create(0)

SaveUTMExr = Draw.Create(0)

SaveTMExr = Draw.Create(0)

SaveIgi = Draw.Create(0)

#UseGUI = Draw.Create(1)

Watermark = Draw.Create(1)

NetworkWM = Draw.Create(0)
NetworkSL = Draw.Create(0)

if sys.platform[:3] == 'win':
	ProcessPrio = Draw.Create(1)
else:
	ProcessPrio = Draw.Create(10)

DisplayPeriod = Draw.Create(20)

HaltTime = Draw.Create(-1)

HaltSamples = Draw.Create(-1)

#FrameUp = Draw.Create(20)

ImageSave = Draw.Create(120)

FilmIso = Draw.Create(100)
Shutter = Draw.Create(125)
FilmWidth = Draw.Create(35)
FStop = Draw.Create(80)
FocusDistance = Draw.Create(2.0)

LensShiftUp=Draw.Create(0.0)
LensShiftRight=Draw.Create(0.0)

AutoFocus = Draw.Create(1)
SelFocus = Draw.Create(0)
CurFocus = Draw.Create(0)
CamFocus = Draw.Create(0)

CamMoBlur = Draw.Create(0)
MoBlur = Draw.Create(0)

ApertureDiffr = Draw.Create(1)
ApertureType = Draw.Create(1)
BladesNum = Draw.Create(6)
BladeOffset = Draw.Create(0.4)
BladeCurvature = Draw.Create(3)
StartAngle = Draw.Create(0.2)
ObstacleMap = Draw.Create('')
ApertureImg = Draw.Create('')

strWhiteBalance = "White Balance:%t | E %x0 | D50 %x1 | D55 %x2 | D65 %x3 | D75 %x4 "
strWhiteBalance += "| A %x5 | B %x6 | C %x7 | 9300 %x8 | F2 %x9 | F7 %x10 | F11 %x11"
WhiteBalance = Draw.Create(0)
whiteBalanceV = {}
whiteBalanceV[0] = "E"
whiteBalanceV[1] = "D50"
whiteBalanceV[2] = "D55"
whiteBalanceV[3] = "D65"
whiteBalanceV[4] = "D75"
whiteBalanceV[5] = "A"
whiteBalanceV[6] = "B"
whiteBalanceV[7] = "C"
whiteBalanceV[8] = "9300"
whiteBalanceV[9] = "F2"
whiteBalanceV[10] = "F7"
whiteBalanceV[11] = "F11"



## Environment Type
strEnvType = "Env Type:%t | Background Colour %x0 | Physical Sky+sun %x1 | Environment Map %x2 | None (lit by mesh emitters) %x3"
EnvType = Draw.Create(1)

SunLayerToggle = Draw.Create(1)
SunLayer = Draw.Create(0)
SkyLayerToggle = Draw.Create(1)
SkyLayer = Draw.Create(1)
Turbidity = Draw.Create(2.2)
ExtraAtm = Draw.Create(0)

## Separate materials
SepMatFile = Draw.Create(0)

## Separate meshs
#SepMeshFile = Draw.Create(0)

##ExpotMeshs
ExportMeshs = Draw.Create(1)

ClayRender = Draw.Create(0)

## Environment map
EnvFile = Draw.Create("none")
strEnvMapType = "Map type:%t | Spherical %x0 | LatLong %x1"
EnvMapType = Draw.Create(1)
EnvMapTypeV = {}
EnvMapTypeV[0] = "spherical"
EnvMapTypeV[1] = "latlong"
EnvGain = Draw.Create(1.0)
EnvWidth = Draw.Create(640)

## Exposure
ToneMapScale = Draw.Create(1.0)
ToneMapScaleExp = Draw.Create(0)
ToneMapEV=Draw.Create(0)

Autoexp = Draw.Create(0)
HStop = Draw.Create(0)

## Tonemapping
strToneMapType = "ToneMap type:%t | Linear %x0 | Reinhard %x1 | Camera %x2"



ToneMapType = Draw.Create(0)
ToneMapPreScale = Draw.Create(3.0)
ToneMapPostScale = Draw.Create(1.0)
ToneMapBurn = Draw.Create(6.0)
ToneMapResponseCurve=Draw.Create(0)

## Overall color gain
DiffuseGain = Draw.Create(1.00)
SpecularGain = Draw.Create(1.00)

## Background color
BGColor = Draw.Create(0.0, 0.0, 0.0)
BGGain = Draw.Create(1.0)

# text color fix
textcol = [0, 0, 0]

# World scale
WorldScale = Draw.Create(1.0)
WorldScaleExp = Draw.Create(0)

# Supersample
SuperSample = Draw.Create(2)

# Pathtrace/MLT Hybrid
Hybrid = Draw.Create(0)
BIH = Draw.Create(500000)

RayNudgeMant = Draw.Create(1)
RayNudgeExp = Draw.Create(-5)

Override = Draw.Create(0)
# Cache KD-trees to disk
#CacheTree = Draw.Create(1)

AutoChoose = Draw.Create(1)

#IndigoPath = Draw.Create('')
IndigoOutPath = Draw.Create('')



# load/save other settings to blender scene
def readWriteSettings(write=0):
	global DefaultExport, ExportMeshs, ClayRender, ScaleSize, ColExponent, FilmWidth
	global FStop, FocusDistance, AutoFocus, SelFocus, CurFocus, CamFocus, LensShiftUp, LensShiftRight, CamMoBlur, MoBlur
	global WhiteBalance, FilmIso, MLT, MaxNumConsRej
	global SplatFilter,SplatF_MN_Blur, SplatF_MN_Ring, DownsizeFilter,DownsizeF_MN_Blur, DownsizeF_MN_Ring, Bidirectional, Threads, Logging, IOverlay
	global SaveUTMExr, SaveTMExr, SaveIgi, DisplayPeriod, ToneMapType
	global ToneMapScale, ToneMapScaleExp, ToneMapEV, ToneMapResponseCurve, Shutter
	global Autoexp, ToneMapPreScale, ToneMapPostScale, ToneMapBurn, HStop, SuperSample, Glare, GlareW, GlareR, GlareB, BIH, RayNudgeMan, RayNudgeExp, Hybrid, Override, AutoChoose
	global EnvType, EnvFile, EnvMapType, EnvGain, EnvWidth, SunLayerToggle, SunLayer, SkyLayerToggle, SkyLayer, Turbidity, ExtraAtm
	global SepMatFile, ExecuteIndigo, HaltTime, HaltSamples, ImageSave, subdivMeshes
	# SepMeshFile,
	
	#global IndigoPath
	global IndigoOutPath
	
	try:
		d = Blender.Scene.GetCurrent().properties['indigo'].convert_to_pyobject()
	except KeyError:
		d = {}
	readWriteValue(d, 'DefaultExport', DefaultExport, write, 1)
	readWriteValue(d, 'ExportMeshs', ExportMeshs, write, 1)
	readWriteValue(d, 'ClayRender', ClayRender, write, 0)
	readWriteValue(d, 'scale size', ScaleSize, write, 100)
	readWriteValue(d, 'colexponent', ColExponent, write, 2.2)
	readWriteValue(d, 'film width', FilmWidth, write, 35)
	readWriteValue(d, 'fstop', FStop, write, 80)
	readWriteValue(d, 'hstop', HStop, write, 0)
	readWriteValue(d, 'lens.shift.up', LensShiftUp, write, 0)
	readWriteValue(d, 'lens.shift.right', LensShiftRight, write, 0)
	readWriteValue(d, 'focusdistance', FocusDistance, write, 2.0)
	readWriteValue(d, 'autofocus', AutoFocus, write, 1)
	readWriteValue(d, 'selfocus', SelFocus, write, 0)
	readWriteValue(d, 'curfocus', CurFocus, write, 0)
	readWriteValue(d, 'camfocus', CamFocus, write, 0)
	readWriteValue(d, 'cam_mo_blur', CamMoBlur, write, 0)
	readWriteValue(d, 'mo_blur', MoBlur, write, 0)
	readWriteValue(d, 'aperture_diffraction', ApertureDiffr, write, 0)
	readWriteValue(d, 'aperturen_type', ApertureType, write, 2)
	readWriteValue(d, 'aperture_blades_num', BladesNum, write, 8)
	readWriteValue(d, 'aperture_blade_offset', BladeOffset, write, 0.4)
	readWriteValue(d, 'aperture_blade_curvature', BladeCurvature, write, 3)
	readWriteValue(d, 'aperture_start_angle',StartAngle, write, 0.2)
	readWriteValue(d, 'opstacle_map', ObstacleMap, write, '')
	readWriteValue(d, 'aperture_img', ApertureImg, write, '')
	readWriteValue(d, 'white balance', WhiteBalance, write, 0)
	readWriteValue(d, 'filmiso', FilmIso, write, 100)
	readWriteValue(d, 'mlt', MLT, write, 1)
	readWriteValue(d, 'splat_filter', SplatFilter,write, 2)
	readWriteValue(d, 'splatf_mn_blur', SplatF_MN_Blur,write, 0.3)
	readWriteValue(d, 'splatf_mn_ring', SplatF_MN_Ring,write, 0.3)
	readWriteValue(d, 'downsize_filter', DownsizeFilter,write, 2)
	readWriteValue(d, 'downsizef_mn_blur', DownsizeF_MN_Blur,write, 0.3)
	readWriteValue(d, 'downsizef_mn_ring', DownsizeF_MN_Ring,write, 0.3)
	readWriteValue(d, 'maxnumconsrej', MaxNumConsRej, write, 1000)
	readWriteValue(d, 'bidirectional', Bidirectional, write, 1)
	readWriteValue(d, 'threads', Threads, write, 1)
	readWriteValue(d, 'logging', Logging, write, 1)
	readWriteValue(d, 'ioverlay', IOverlay, write, 0)
	readWriteValue(d, 'saveutmexr', SaveUTMExr, write, 0)
	readWriteValue(d, 'savetmexr', SaveTMExr, write, 0)
	readWriteValue(d, 'saveigi', SaveIgi, write, 0)
#	readWriteValue(d, 'usegui', UseGUI,write,1)
	readWriteValue(d, 'watermark',Watermark,write,0)
	readWriteValue(d, 'network.wm',NetworkWM,write,0)
	if sys.platform[:3] == 'win':
		readWriteValue(d, 'process_prio',ProcessPrio, write, 1)
	else:
		readWriteValue(d, 'process_prio',ProcessPrio, write, 10)
#	readWriteValue(d, 'indigo_path',IndigoPath, write, '')
	readWriteValue(d, 'indigo_out_path',IndigoOutPath, write, '')
	readWriteValue(d, 'network.sl',NetworkSL,write,0)
	readWriteValue(d, 'display period', DisplayPeriod, write, 20)
	readWriteValue(d, 'tonemaptype', ToneMapType, write, 1)
	readWriteValue(d, 'tonemapscale', ToneMapScale, write, 1.0)
	readWriteValue(d, 'tonemapscaleexp', ToneMapScaleExp, write, 0)
	readWriteValue(d, 'tonemapev', ToneMapEV, write, 0)
	readWriteValue(d, 'tonemapres', ToneMapResponseCurve, write, 0)
	readWriteValue(d, 'shutter', Shutter, write, 125)
	readWriteValue(d, 'autoexp', Autoexp, write, 0)
	readWriteValue(d, 'tonemapprescale', ToneMapPreScale, write, 4.0)
	readWriteValue(d, 'tonemappostscale', ToneMapPostScale, write, 1.2)
	readWriteValue(d, 'tonemapburn', ToneMapBurn, write, 6.0)
	readWriteValue(d, 'envtype', EnvType, write, 1)
	readWriteValue(d, 'envfile', EnvFile, write, '')
	readWriteValue(d, 'envmaptype', EnvMapType, write, 1)
	readWriteValue(d, 'envgain', EnvGain, write, 1.0)
	readWriteValue(d, 'envwidth', EnvWidth, write, 640)
	readWriteValue(d, 'sun_layer_toggle', SunLayerToggle, write, 1)
	readWriteValue(d, 'sun_layer', SunLayer, write, 0)
	readWriteValue(d, 'sky_layer_toggle', SkyLayerToggle, write, 1)
	readWriteValue(d, 'sky_layer', SkyLayer, write, 1)
	readWriteValue(d, 'turbidity', Turbidity, write, 2.2)
	readWriteValue(d, 'extraAtm', ExtraAtm,write,0)
	readWriteValue(d, 'sepmatfile', SepMatFile, write, 0)
	#readWriteValue(d, 'sepmeshfile', SepMeshFile, write, 0)
	readWriteValue(d, 'ExecuteIndigo', ExecuteIndigo, write, 1)
	readWriteValue(d, 'HaltTime', HaltTime, write, -1.0)
	readWriteValue(d, 'HaltSamples', HaltSamples, write, -1.0)
	readWriteValue(d, 'imagesave', ImageSave, write, 120)
	readWriteValue(d, 'super_sample_factor', SuperSample, write, 2)
	readWriteValue(d, 'bih', BIH, write, 500000)
	readWriteValue(d, 'rayNudgeMant',RayNudgeMant, write, 1)
	readWriteValue(d, 'rayNudgeExp',RayNudgeExp, write, -5)
	readWriteValue(d, 'hybrid', Hybrid, write, 0)
	readWriteValue(d, 'autoChoose',AutoChoose, write, 1)
	readWriteValue(d, 'override',Override, write, 0)
	if write == 0:
		if 'subdivs' in d:
			subdivMeshes=d['subdivs']
	else:
		d['subdivs']=subdivMeshes
	try:
		if write == 0:
			BGColor.val = (d['backgroundcolor.r'], d['backgroundcolor.g'], d['backgroundcolor.b'])
		else:
			d['backgroundcolor.r'] = BGColor.val[0]
			d['backgroundcolor.g'] = BGColor.val[1]
			d['backgroundcolor.b'] = BGColor.val[2]
	except KeyError:
		print ""
	readWriteValue(d, 'backgroundgain', BGGain, write, 1.0)
	readWriteValue(d, 'worldscale', WorldScale, write, 1.0)
	readWriteValue(d,'worldscaleexp',WorldScaleExp,write,0)
	if write != 0:
		Blender.Scene.GetCurrent().properties['indigo'] = d


readWriteSettings(0)

def calcFL():
	global hyperFocal, NearFocusLimit, FarFocusLimit
	currentscene = Scene.GetCurrent()
	camObj = currentscene.objects.camera
	
	if camObj:
		focal = camObj.data.lens
		coc = 0.029
		fstopIndex = float(FStop.val) / 10
		f_number = 2 ** (fstopIndex / 2)
		focus = FocusDistance.val * 1000
		hyperFocal = (focal * focal) / (f_number * coc) + focal
		NearFocusLimit = ((hyperFocal - focal) * focus) / (hyperFocal + focus - (2*focal))
		NearFocusLimit = str(float(int(NearFocusLimit / 10)) / 100)
		if ((hyperFocal - focus) <= 10):
			FarFocusLimit = "Infinity";  #set infinity at 10000m
		else:
			FarFocusLimit = ((hyperFocal-focal) * focus) / (hyperFocal - focus)
			FarFocusLimit = str(float(int(FarFocusLimit/ 10)) / 100)
		hyperFocal = str(float(int(hyperFocal) / 10) / 100)
	else:
		hyperFocal = "There's no active camera!"
		NearFocusLimit = "There's no active camera!"
		FarFocusLimit = "There's no active camera!"

######  Draw Camera  ###############################################
def drawCamera():
####################################################################
	global evtchangesize, Shutter, Autoexp, AutoFocus, SelFocus, CurFocus, CamFocus, HStop, evtHStop, CamMoBlur, MoBlur
	global FStop, FocusDistance, SepMatFile, FilmWidth, SizeX, SizeY, ScaleSize, strScaleSize, LensShiftUp, LensShiftRight
	global strWhiteBalance, WhiteBalance, whiteBalanceV, ExecuteIndigo, FilmIso
	global ApertureDiffr, ApertureType, BladesNum, BladeOffset, BladeCurvature, StartAngle, ObstacleMap, ApertureImg, ScrollY, ScrollAreaY
	global hyperFocal, NearFocusLimit, FarFocusLimit
	
	drawScreenButtons()
	
	ScrollAreaY = 500
	
	AreaDims = Window.GetAreaSize()
	y = AreaDims[1]+ScrollY
	
	gui = guiLayout()
	
	gui.title("Camera setting")
	
	gui.subtitle("Render dimensions")
	gui.newline("Width")
	SizeX = Draw.Number("", evtchangesize, gui.x, gui.y, 80, 18, SizeX.val, 1, 10000, "width of the render max 10,000")
	gui.units("pixels", 80)
	
	gui.newline("Height")
	SizeY = Draw.Number("", evtchangesize, gui.x, gui.y, 80, 18, SizeY.val, 1, 10000, "height of the render max 10,000")
	gui.units("pixels", 80)
	
	gui.newline("Scale")
	ScaleSize = Draw.Menu(strScaleSize, evtNoEvt, gui.x, gui.y, 80, 18, ScaleSize.val, "scale image size of ...")
	
	gui.subtitle("Film setting")
	
	gui.newline("Film width")
	FilmWidth = Draw.Number("", evtNoEvt, gui.x, gui.y, 60, 18, FilmWidth.val, 8, 254, "width of the \"film\" in mm")
	gui.units("mm", 60)
	
	gui.newline("Film ISO")
	FilmIso = Draw.Menu("25%x25|50%x50|100%x100|200%x200|400%x400|800%x800|1600%x1600", evtNoEvt, gui.x, gui.y, 75, 18, FilmIso.val, " \"film\" speed")
	gui.units("iso units", 75)
	
	gui.newline("Exposure")
	gui.text("1/")
	Shutter = Draw.Menu("1%x1|2%x2|4%x4|8%x8|15%x15|30%x30|60%x60|125%x125|250%x250|500%x500|1000%x1000|2000%x2000", evtNoEvt, gui.x, gui.y, 75, 18, Shutter.val, "exposure time in seconds")
	gui.units("or", 75)
	
	Autoexp = Draw.Toggle("Autoexposure", evtNoEvt, gui.x, gui.y, 100, 18, Autoexp.val, "enable auto exposure")
	
	# To be re-enabled....
	gui.newline("F-Stop")
#	HStop = Draw.Toggle("halfstops", evtNoEvt,gui.x, gui.y, 60, 18,HStop.val, "enable the halfstop range in menu")
#	if HStop.val == 0:
	FStop = Draw.Menu("0.5%x5|0.7%x7|1.0%x10|1.4%x14|2%x20|2.8%x28|4%x40|5.6%x56|8%x80|11%x110|16%x160|22%x220|32%x320|64%x640|90%x900|128%x1280", evtNoEvt, gui.x, gui.y, 75, 18, FStop.val, "depth of field, (blurring of fore and background)")
	# else:
	# 	FStop = Draw.Menu("F-Stop%t|1.2%x12|1.7%x17|2.4%x24|3.3%x33|4.8%x48|6.6%x66|9.6%x96|13.2%x132|19.2%x192|26.4%x264|44%x440", evtNoEvt, 80, y-145, 75, 18, FStop.val, "depth of field, (blurring of fore and background)")
	
	gui.newline("Lens Shift")
	
	gui.newline("    Up ")
	LensShiftUp   = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, LensShiftUp.val, -1.0, 1.0, "Up shift")
	gui.units("mm", 100)
	
	gui.newline("    Right ")
	LensShiftRight= Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, LensShiftRight.val, -1.0, 1.0, "Right shift")
	gui.units("mm", 100)
	
	gui.subtitle("       Aperture diffraction")
	
	ApertureDiffr=Draw.Toggle(" X ",evtNoEvt, 16, gui.y + 6, 20, 18, ApertureDiffr.val, "enable aperture diffraction")
	
	if(ApertureDiffr.val):
		gui.newline("   Aperture type")
		ApertureType = Draw.Menu("Circular%x1|Generated%x2|Image%x3",evtNoEvt,gui.x, gui.y, 120, 18, ApertureType.val, "Type pf aperture")
		
		if ApertureType == 2:
			gui.newline("   Blades")
			BladesNum = Draw.Number("",evtNoEvt, gui.x, gui.y, 100, 18, BladesNum.val, 3, 20, "number of blades")
			
			gui.newline("   Offset")
			BladeOffset = Draw.Number("",evtNoEvt, gui.x, gui.y, 80, 18, BladeOffset.val, 0, 0.5, "blades offset")
			gui.units("radians", 80)
			
			gui.newline("   Curvature")
			BladeCurvature = Draw.Number("",evtNoEvt, gui.x, gui.y, 80, 18, BladeCurvature.val, BladeOffset.val, 10, "blade curvature radius")
			gui.units("radians", 80)
			
			gui.newline("   Start Angle")
			StartAngle = Draw.Number("",evtNoEvt, gui.x, gui.y, 80, 18, StartAngle.val, 0, 2, "blades start angle")
			gui.units("radians", 80)
		
		if ApertureType == 3:
			gui.newline("   Aperture Image")
			ApertureImg = Draw.String("filename: ", evtNoEvt, gui.x, gui.y, 300, 18, ApertureImg.val, 255, "the file name a square PNG with power-of-two dimension at least 512x512")
			Draw.Button("...", evtloadaperture, gui.x + 310, gui.y, 20 ,18,"open file selector")
		
		if ApertureType == 1 or ApertureType == 2:
			gui.newline("   Obstacle map")
			ObstacleMap = Draw.String("filename: ", evtNoEvt, gui.x, gui.y, 300, 18, ObstacleMap.val, 255, "the file name a square PNG with power-of-two dimension at least 512x512")
			Draw.Button("...", evtloadobstacle, gui.x + 310, gui.y, 20 ,18,"open file selector")
	
	gui.subtitle("Camera setting")
	
	gui.newline("White balance")
	WhiteBalance = Draw.Menu(strWhiteBalance, evtNoEvt, gui.x, gui.y, 75, 18, WhiteBalance.val, "set the white balance for different lighting conditions")
	
	gui.newline("Focus distance")
	FocusDistance = Draw.Number("", evtNoEvt, gui.x, gui.y, 75, 18, FocusDistance.val, 0.0, 100, "the distance to objects that are clearly in focus")
	gui.units("m or", 75)
	
	AutoFocus = Draw.Toggle("Autofocus", evtFocusA, gui.x, gui.y , 70, 18, AutoFocus.val, "enable autofocus")
	gui.units("or", 70)
	
	CamFocus = Draw.Toggle("Blender camera", evtFocusD, gui.x, gui.y, 110, 18, CamFocus.val, "get the Blender camera depth-of-field distance")
	gui.units("or", 110)
	
	Draw.Button("Selected", evtFocusS, gui.x, gui.y, 60, 18, "get the distance from the selected object")
	gui.units("or", 60)
	
	Draw.Button("3Dcursor", evtFocusC, gui.x, gui.y, 60, 18, "get the distance from the 3d cursor")
	
	if (FilmWidth.val == 35.0) and (False):
		calcFL()
		gui.newline("   Hyperfocal")
		gui.text(hyperFocal)
		gui.units("m")
		
		gui.newline("   Near focus")
		gui.text(NearFocusLimit)
		gui.units("m")
		
		gui.newline("   Far focus")
		gui.text(FarFocusLimit)
		gui.units("m")
	
	
	gui.subtitle("Motion blur")
	gui.newline("  On camera")
	CamMoBlur = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 35, 18, CamMoBlur.val, "Enable Camera Motion Blur")
	
	gui.newline("  On objects")
	MoBlur = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 35, 18, MoBlur.val, "Enable Motion Blur")
	
	drawExportButtons(gui)

##############  Draw Environment  #######################################
def drawEnv():
#########################################################################
	global evtNoEvt
	global strEnvType, EnvType, EnvFile, strEnvMapType, EnvMapType, EnvGain, EnvWidth, SunLayerToggle, SunLayer, SkyLayerToggle, SkyLayer, Turbidity, ExtraAtm, BGColor, BGGain, WorldScale, WorldScaleExp, ScrollAreaY
	
	drawScreenButtons()
	
	gui = guiLayout()
	
	gui.title("Environment setting")
	gui.newline()
	
	y = 0
	
	gui.newline("Environment type")
	EnvType = Draw.Menu(strEnvType, evtNoEvt, gui.x, gui.y, 205, 18, EnvType.val, "set the environment type")
	
	if EnvType.val == 2:
		gui.subtitle("Environment map")

		gui.newline("EXR map filename")
		EnvFile = Draw.String("", evtNoEvt, gui.x, gui.y, 250, 18, EnvFile.val, 255, "the file name of an .float/.exr probe")
		Draw.Button("...", evtloadimg, gui.x + 255, gui.y, 20 ,18,"open file selector")

		gui.newline("Gain")
		EnvGain = Draw.Number("Gain: ", evtNoEvt,gui.x, gui.y, 105, 18, EnvGain.val, 0.001, 10000.00, "gain adjustment, range 0.001-1000")
		gui.units("dimensionless", 105)

		gui.newline("Mapping")
		EnvMapType = Draw.Menu(strEnvMapType, evtNoEvt, gui.x, gui.y, 100, 18, EnvMapType.val, "set the map type of the probe")

		if EnvMapType.val == 0:
			gui.newline("  Width")
			EnvWidth = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, EnvWidth.val, 1, 10000, "input width (= height) of map")
			gui.units("pixels", 100)
			
	if EnvType.val == 1:
		gui.subtitle("       Sun layer")
		SunLayerToggle = Draw.Toggle(" x ",evtNoEvt, 16, gui.y + 5, 20,18, SunLayerToggle.val, "Enable Sun Layer")
		
		if SunLayerToggle.val:
			gui.newline("Layer #")
			SunLayer = 	Draw.Number("",evtNoEvt, gui.x, gui.y, 50, 18, SunLayer.val, 0, 9, "layer that sun will be drawn to")

		gui.subtitle("       Sky layer")

		SkyLayerToggle = Draw.Toggle(" x ",evtNoEvt, 16, gui.y + 5, 20,18, SkyLayerToggle.val, "Enable Sky Layer")
		if SkyLayerToggle.val:
			gui.newline("Layer #")
			SkyLayer = 	Draw.Number("",evtNoEvt, gui.x, gui.y, 50, 18, SkyLayer.val, 0, 9, "layer that sky light will be drawn to")

			gui.newline("Sky turbidity")
			Turbidity = Draw.Number("", evtNoEvt,gui.x, gui.y, 100, 18, Turbidity.val, 1.5, 7.5, "set sky turbidity, range 2 clear - 5 hazy")
			gui.units("dimensionless", 100)

			# gui.newline("Extra Atmospheric")
			# ExtraAtm = Draw.Toggle(" X ",evtNoEvt,gui.x, gui.y,30,18,ExtraAtm.val,"Extra Atmospheric Scattering")

	if EnvType.val == 0:
		BGColor = Draw.ColorPicker(evtNoEvt, 10, y-160, 30, 60, BGColor.val, "colour and picker (LMB)")
		BGGain = Draw.Number("Gain:", evtNoEvt, 40, y-140, 100, 18, BGGain.val, 1e-4, 1e9, "set background color gain")
	BGL.glRasterPos2i(10,y-180); Draw.Text("Worldscale:","normal")
	WorldScale = Draw.Number("", evtNoEvt, 10, y-200, 70, 18, WorldScale.val, 1,9, "scale factor of scene at export else 1 Bl unit == 1 metre in Indigo")
	WorldScaleExp = Draw.Number("*10^", evtNoEvt, 80, y-200, 70, 18, WorldScaleExp.val, -4,6, "Exponent")
	
	drawExportButtons()

###############  Draw Rendersettings   ###################################
def drawSettings(y):
#####################################################
	global Bidirectional # RRLP StrataWidth
	global AlphaMask, SplatFilter, SplatF_MN_Blur, SplatF_MN_Ring, DownsizeFilter,DownsizeF_MN_Blur, DownsizeF_MN_Ring, MLT,  MaxNumConsRej, SuperSample, BIH, RayNudgeMant, RayNudgeExp, Hybrid, ScrollAreaY, ClayRender, rendererAdvanced
	
	drawScreenButtons()
	
	gui = guiLayout()
	
	gui.title("Renderer setting")
	
	gui.subtitle("Render type")
	
	gui.newline("Alpha render")
	AlphaMask=Draw.Toggle(" X ",evtNoEvt, gui.x, gui.y, 35, 18, AlphaMask.val, "enable Alpha Mask rendering")

	gui.newline("Clay render")
	ClayRender = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 35,18,ClayRender.val, "only grey diffuse material")
	
	gui.subtitle("Render method")

	gui.newline("Metropolis mode")
	MLT = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 35, 18, MLT.val, "enable MLT (useful for indirect light) else pathtracer (for direct light)")
	gui.units("Complex light transport system for slow scenes", 35)

	gui.newline("Bidirectional mode")
	Bidirectional = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 35, 18, Bidirectional.val, "enable bidirectional tracing (useful for indirect light and caustics)")
	gui.units("We recommend you leave this enabled", 35)
	
	gui.subtitle("      Advanced settings")
	rendererAdvanced = Draw.Toggle(" X ", evtNoEvt, 16, gui.y + 6, 20, 18, rendererAdvanced.val, "")
	
	if rendererAdvanced.val:
		gui.newline("Splat filter")
		SplatFilter=Draw.Menu("Box %x0 | Gaussian %x1 | Mitchell-Netravali %x2",evtNoEvt,gui.x, gui.y,130,18,SplatFilter.val, "Splat Filter Type")
	
	
		if SplatFilter.val == 2:
			gui.newline("  Blur")
			SplatF_MN_Blur=Draw.Number("",evtNoEvt, gui.x, gui.y,100, 18,SplatF_MN_Blur.val, 0.0, 1.0, "Blur value")
			gui.units("pixels", 100)

			gui.newline("  Ring")
			SplatF_MN_Ring=Draw.Number("",evtNoEvt, gui.x, gui.y,100,18,SplatF_MN_Ring.val, 0.0, 1.0, "Ring value")
			gui.units("pixels", 100)

		gui.newline("Downsize filter")
	
		DownsizeFilter=Draw.Menu("Box %x0 | Gaussian %x1 | Mitchell-Netravali %x2",evtNoEvt,gui.x, gui.y,150,18,DownsizeFilter.val, "Downsize Filter Type")
	
		if DownsizeFilter.val == 2:
			gui.newline("  Blur")
			DownsizeF_MN_Blur=Draw.Number("Blur: ",evtNoEvt, gui.x, gui.y,100,18,DownsizeF_MN_Blur.val, 0.0, 1.0, "Blur value")
			gui.units("pixels", 100)

			gui.newline("  Ring")
			DownsizeF_MN_Ring=Draw.Number("Ring: ",evtNoEvt, gui.x, gui.y,100,18,DownsizeF_MN_Ring.val, 0.0, 1.0, "Ring value")
			gui.units("pixels", 100)
	
		gui.newline("MNCR")
		MaxNumConsRej = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, MaxNumConsRej.val, 0, 100000, "low values give more biased calculations and fewer fireflies")
		gui.units("rejections", 100)

		gui.newline("Supersampling")
		SuperSample = Draw.Number("", evtNoEvt, gui.x, gui.y,100,18, SuperSample.val,1,4, "more helps anti alias edges but eats memory")
		gui.units("times", 100)
	
	drawExportButtons(gui)

##################  Draw RSettings  #########################
def drawSystem(y):
#############################################################
	global Logging, IOverlay
	global SaveUTMExr, SaveTMExr, SaveIgi, Watermark, NetworkWM, NetworkSL, DisplayPeriod, ColExponent, ProcessPrio
#	global IndigoPath
	global IndigoOutPath
	global HaltTime, HaltSamples, ImageSave, SepMatFile, Threads, Override,  AutoChoose, ScrollAreaY
	global DefaultExport, ExecuteIndigo, ExportMeshs, SepMeshFile
	
	drawScreenButtons()
	
	gui = guiLayout()
	
	gui.title("Renderer setting")

	# Override = Draw.Toggle("Override", evtNoEvt, 10, y-100, 80, 18, Override.val, "Override settings with ini file values or defaults")
	
	# ColExponent = Draw.Number("Gamma: ", evtNoEvt, gui.x, gui.y, 205, 18, ColExponent.val, 0.01, 5.0, "overall gamma correction")

	gui.subtitle("Performance")

	gui.newline("Number of threads")
	Threads = Draw.Number("", evtNoEvt, gui.x, gui.y,50,18, Threads.val,1,8, "the number of processor threads/cores to be used")
	gui.units(" or ",50)
	AutoChoose = Draw.Toggle("Automatic", evtNoEvt, gui.x, gui.y,80,18,AutoChoose.val, "Enable auto choice of threads number")
	
	gui.newline("Display period")
	DisplayPeriod = Draw.Number("", evtNoEvt, gui.x, gui.y,100, 18, DisplayPeriod.val, 10, 10000, "the time between GUI display updates [sec]")
	gui.units("seconds", 100)
	
	gui.newline("Save period")
	ImageSave = Draw.Number("", evtNoEvt, gui.x, gui.y,100, 18, ImageSave.val, 20, 10000, "the time between saving .png images [sec]")
	gui.units("seconds", 100)

#	FrameUp = Draw.Number("Frame Upl:", evtNoEvt, 10, y-160, 205, 18, FrameUp.val, 20, 10000, "the time between network slave uploads of image to master [sec]")

	gui.subtitle("Halting")
	
	gui.newline("Halt time")
	HaltTime = Draw.Number("", evtNoEvt, gui.x, gui.y,100, 18, HaltTime.val, -1, 86400, "the rendering duration [sec] setting -1 will continue until manual exit")
	gui.units("seconds", 100)
	
	gui.newline("Halt SPP")
	HaltSamples = Draw.Number("", evtNoEvt, gui.x, gui.y,100, 18, HaltSamples.val, -1, 10000, "the rendering quality setting -1 will continue until manual exit")
	gui.units("samples per pixel", 100)
	
	gui.subtitle("Image saving options")

	gui.newline("Add Indigo logo")
	Watermark = Draw.Toggle(" X ",evtNoEvt, gui.x + 20, gui.y, 35, 18, Watermark.val,"Add indigo logo")

	gui.newline("Write untonemapped EXR")
	SaveUTMExr = Draw.Toggle(" X ", evtNoEvt, gui.x + 20, gui.y, 35, 18, SaveUTMExr.val, "save image also as an untonemapped .exr file")

	gui.newline("Write tonemapped EXR")
	SaveTMExr = Draw.Toggle(" X ", evtNoEvt, gui.x + 20, gui.y, 35, 18, SaveTMExr.val, "save image also as a tonemapped .exr file")

	gui.newline("Write IGI")
	SaveIgi = Draw.Toggle(" X ", evtNoEvt, gui.x + 20, gui.y, 35, 18, SaveIgi.val, "save image also as an .igi file WARNING: can be large and slow")

	gui.subtitle("Other...")
	
	gui.newline("Logging")
	Logging = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 35, 18, Logging.val, "enable writing a log.txt")
	
	gui.newline("Information overlay")
	IOverlay = Draw.Toggle(" X ", evtNoEvt, gui.x, gui.y, 35, 18, IOverlay.val, "enable info text writing onto the image")

	gui.newline("Image output path")
	IndigoOutPath = Draw.String("",evtNoEvt, gui.x, gui.y, 250, 18, IndigoOutPath.val, 399, "Path to Indigo output")
	Draw.Button("...",evtSetIndigoOutPath, gui.x + 255, gui.y, 20, 18, "open file selector")
	
	gui.newline("Autogenerate igs path")
	DefaultExport = Draw.Toggle(" X ",evtNoEvt,gui.x, gui.y,35,18, DefaultExport.val, "use \"default.igs\" as the file name")

	gui.newline("Autostart indigo ")
	ExecuteIndigo = Draw.Toggle(" X ", evtNoEvt,gui.x, gui.y,35,18, ExecuteIndigo.val, "enable auto start of Indigo after export - to path set in Indigowrapper.conf")
	
	# if not ExportMeshs.val: SepMeshFile.val = 1  # without mesh export force use of external mesh file
	# SepMeshFile = Draw.Toggle("sep. meshes",evtNoEvt,260,y-ScrollAreaY+5,80,18, SepMeshFile.val, "enable saving meshes to a separate file")
	# SepMatFile = Draw.Toggle("sep. mats", evtNoEvt, 340,y-ScrollAreaY-10,80,18, SepMatFile.val, "enable saving materials to a separate file")

	
	gui.newline("Process priority")
	if sys.platform[:3] == 'win':
		ProcessPrio=Draw.Menu("NORMAL %x0 | BELOW_NORMAL %x1",evtNoEvt, gui.x, gui.y, 200, 18, ProcessPrio.val, "set process priority for Indigo")
	else:
		ProcessPrio=Draw.Number("",evtNoEvt, gui.x, gui.y, 50, 18, ProcessPrio.val, 1, 19, "the NICE increment")
	
	drawExportButtons()


def meshGUI():
	global meshSubdivide, MaxNumSubdiv, SubdivPixelTreshold, SubdivCurvatureTreshold, SubdivSmooth, subdivMeshes
	def MeshSubdivReset():
		global MaxNumSubdiv, SubdivPixelTreshold, SubdivCurvatureTreshold, SubdivSmooth, subdivMeshes
		MaxNumSubdiv.val = 2
		SubdivPixelTreshold.val=0.0
		SubdivCurvatureTreshold.val=0.1
		SubdivSmooth.val = 1
	
	drawScreenButtons()
	
	gui = guiLayout()
	gui.title("Mesh subdvision")

	y = gui.y + 50
	
	scn=Scene.GetCurrent()
	if len(scn.objects.selected) == 1:
		obj = Blender.Object.GetSelected()[0]
		
		meshname=obj.getData(name_only=1)
		objecttype = obj.getType()
		if(objecttype == "Curve"):
			meshname +='_cu'
		imeshname = meshname.replace("<","-").replace(">","-")
		
		mesh=obj.getData()
		try:
			d=mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		#Subdivision
		if imeshname in d:
			meshSubdivide.val = 1
			MaxNumSubdiv.val=d[imeshname][0]
			SubdivPixelTreshold.val=d[imeshname][1]
			SubdivCurvatureTreshold.val=d[imeshname][2]
			SubdivSmooth.val=d[imeshname][3]
		else:
			meshSubdivide.val = 0
			MeshSubdivReset()
		meshSubdivide = Draw.Toggle("Subdivide", evtSubdivMesh, 10,y-100, 80, 18, meshSubdivide.val, "enable Indigo Subdivide")
		if meshSubdivide.val:
			MaxNumSubdiv = Draw.Number("Max Num Subdivisions: ", evtUpdateSubdiv, 100, y-100, 205, 18, MaxNumSubdiv.val, 1, 20, "Set the Max number for subdivision level")
			SubdivPixelTreshold = Draw.Number("Pixel Treshold", evtUpdateSubdiv, 100, y-125, 205, 18, SubdivPixelTreshold.val, 0.0, 100.0, "triangles will not be subdivided if they subtend a smaller pixel width than this threshold")
			SubdivCurvatureTreshold = Draw.Number("Curvature Treshold", evtUpdateSubdiv, 100, y-145, 205, 18, SubdivCurvatureTreshold.val, -1, 100.0, "triangles will not be subdivided if their curvature is lower than this value")
			SubdivSmooth = Draw.Toggle("Subdivision Smoothing", evtUpdateSubdiv, 100, y-165, 205, 18, SubdivSmooth.val, "enable Subdivision Smoothing")
	drawExportButtons()

	
	
	

#################  Draw Tonemapping  #########################
def drawTonemap(y):
##############################################################
	global ToneMapType, ToneMapPreScale, ToneMapPostScale, ToneMapBurn, ToneMapScale, ToneMapScaleExp, ToneMapEV, ScrollAreaY, ToneMapResponseCurve
	
	drawScreenButtons()
	
	ScrollAreaY = 300
	
	gui = guiLayout()

	gui.title("Tonemapping")
	gui.newline()
	
	gui.newline("Tonemapping method")
	ToneMapType = Draw.Menu(strToneMapType, evtNoEvt, gui.x, gui.y, 105, 18, ToneMapType.val, "set the type of the tonemapping")
	
	if ToneMapType.val == 0:
		gui.subtitle("Linear tonemapping")
		gui.newline("Scale")
		ToneMapScale = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, ToneMapScale.val, 1, 9, "Linear scale: refer to the Indigo Manual ;)")
		gui.units(" x 10 ^", 100)
		ToneMapScaleExp = Draw.Number("*", evtNoEvt, gui.x, gui.y, 100, 18, ToneMapScaleExp.val, -20, 20, "Linear scale exponent")
		gui.units("dimensionless", 100)

	if ToneMapType.val == 1:
		gui.subtitle("Reinhard tonemapping")

		gui.newline("Burn")
		ToneMapBurn = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, ToneMapBurn.val, 0.1, 10.0, "burn to white, 0.1= a lot, 10= none")
		gui.units("dimensionless", 100)
		
		gui.newline("Pre-scale")
		ToneMapPreScale = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, ToneMapPreScale.val, 0,50.0, "scales the mid-grey-point of the rendered pixels, practical range 1-10")
		gui.units("dimensionless", 100)
		
		gui.newline("Post scale")
		ToneMapPostScale = Draw.Number("", evtNoEvt, gui.x, gui.y, 100, 18, ToneMapPostScale.val, 0,50.0, "scales the white-point of the rendered pixels , practical range 1-10 ")
		gui.units("dimensionless", 100)

	if ToneMapType.val == 2:
		gui.subtitle("Camera tonemapping")

		gui.newline("EV Adjust")
		ToneMapEV=Draw.Number("",evtNoEvt,gui.x, gui.y,105,18,ToneMapEV.val,-20,20,"Exposure Value Adjustment")
		gui.units("exposure stops", 105)

		gui.newline("Film")
		ToneMapResponseCurve=Draw.Menu(strToneMapResponseCurve,evtNoEvt,gui.x, gui.y,185,18,ToneMapResponseCurve.val, "Choose Camera Response Curve")
		
	drawExportButtons()

##############
def drawGUI():
##############
	global ScrollY, ScrollAreaY, Screen
	texts=Text.Get()
	shader="Indigo Shader:%t|NONE%x-1"
	for s in texts:
		shadername=s.getName()
		if(shadername[:4] == "isl_"):
			if len(ShaderList) == 0:
				ShaderList.append(s.getName())
			else:
				i=0
				flag=0
				for l in ShaderList:
					if ShaderList[i] == s.getName():
						flag=1
						break
					i+=1
				if not flag:
					ShaderList.append(s.getName())
	
	global tonemapcurves, IndigoPath, strToneMapResponseCurve
	tonemapcurves = {}
	
	responsecurvepath = os.path.join(indigo.getResourcesPath(),"data\camera_response_functions")
	if os.path.exists(responsecurvepath):
		strToneMapResponseCurve = "Response Curve:%t "
		curvefiles = os.listdir(responsecurvepath)
		i = 0
		for cfile in curvefiles:
			strToneMapResponseCurve += "| " + cfile[:-4] + " %x" +str(i)
			tonemapcurves[i] = cfile[:-4]
			i += 1
	else:
		strToneMapResponseCurve = "Response Curve:%t | No curves installed %x1"
	
	if Screen==0:
		drawCamera()
	if Screen==1:
		drawEnv()
	if Screen==2:
		drawSettings(230)
	if Screen==3:
		drawSystem(230)
	if Screen==4:
		drawTonemap(230)
	if Screen == 6:
		meshGUI()
	if Screen==5:
		materialGUI()

def drawScreenButtons():
	global openCamera, openEnv, openRSet, openSSet, openTmap, meshGui, matGui, openMesh, ScrollY, ScrollAreaY, selectedSection
	AreaDims = Window.GetAreaSize()
	
	y = AreaDims[1]
	BGL.glColor3f(0.2,0.2,0.2)
	AreaDims = Window.GetAreaSize()
	BGL.glRectf(0,0,AreaDims[0],AreaDims[1])
	
	y+= ScrollY
	
	global indigoLogo
	indigoLogo.reload()
	indigoLogo.glLoad()
	drawImage(indigoLogo, 0, AreaDims[1] - 512, 1.0, 1.0)
	
	# Draw the header bar
	BGL.glColor3f(0.1,0.1,0.1)
	BGL.glRectf(0,y,460,y-70)
	BGL.glColor3f(1,1,1);BGL.glRasterPos2i(10,y-25) ; Draw.Text("Blendigo  v" + blendigoVersion + "  ::  Indigo exporter module", "small")
	BGL.glRasterPos2i(370,y-25) ; Draw.Text("Press Q to close", "tiny")
	BGL.glColor3f(0.6, 0.6, 0.6) 
	BGL.glRectf(0,y-70,460,y-72)
	BGL.glColor3f(0.9, 0.9, 0.9);
	
	x = 10
	y -= 20
	w = 55
	p = 6
	Draw.Toggle("Camera", openCamera, x, y-40, w, 18, (selectedSection == 'camera'), "set camera parameters")
	x += (w+p)
	
	Draw.Toggle("Environment", openEnv, x, y-40, w, 18, (selectedSection == 'environment'), "set environment parameters")
	x += (w+p)
	
	Draw.Toggle("Renderer", openRSet, x, y-40, w, 18, (selectedSection == 'renderer'), "set rendering parameters")
	x += (w+p)
	
	Draw.Toggle("Tonemap", openTmap, x, y-40, w, 18, (selectedSection == 'tonemap'), "set tonemap parameters")
	x += (w+p)
	
	Draw.Toggle("System", openSSet, x, y-40, w, 18, (selectedSection == 'system'), "set system options")
	x += (w+p)
	
	meshGui=Draw.Toggle("Mesh", openMesh, x, y-40, w, 18, (selectedSection == 'mesh'), "set Mesh parameters")
	x += (w+p)
	
	matGui=Draw.Toggle("Materials", openMat, x, y-40, w, 18, (selectedSection == 'material'), "show material editor")
	x += (w+p)
	
	if(AreaDims[1] < ScrollAreaY):
		if(ScrollY == 0):
			BGL.glColor3f(0.7,0.0,0.0)
			BGL.glRectf(0,0,AreaDims[0],3)
			BGL.glColor3f(0.9,0.9,0.9)
		else:
			BGL.glColor3f(0.7,0.0,0.0)
			BGL.glRectf(0,AreaDims[1]-4,AreaDims[0],AreaDims[1])
			BGL.glColor3f(0.9,0.9,0.9)

def drawExportButtons(gui=0):
	global envExport, envExportAmin, envNoEnv, ExecuteIndigo, DefaultExport, ExportMeshs, ClayRender, SepMatFile, ScrollY, ScrollAreaY
	
	if gui==0:
		gui = guiLayout()
		gui.y -= ScrollAreaY
	
	gui = guiLayout()
	
	# SepMeshFile,
	AreaDims = Window.GetAreaSize()
	y = gui.y + ScrollAreaY - 70
	
	BGL.glColor3f(0.6, 0.6, 0.6)
	BGL.glRectf(245, gui.y, 440, gui.y-36)

	Draw.Button("RENDER", evtExport, 370, y-ScrollAreaY+40, 60, 24, "open file selector to export and save Indigo .igs")
	Draw.Button("Render Anim", evtExportAnim, 275, y-ScrollAreaY+40, 90, 24, "open file selector to export and save for each frame WARNING: eats disk space")
	ExportMeshs = Draw.Toggle("xm",evtNoEvt,250,y-ScrollAreaY+40,20,24, ExportMeshs.val, "Export meshes? Disable to speed-up export time.")

	if(AreaDims[1] < ScrollAreaY):
		if(ScrollY == 0):
			BGL.glColor3f(0.7,0.0,0.0)
			BGL.glRectf(0,0,AreaDims[0],3)
			BGL.glColor3f(0.9,0.9,0.9)
		else:
			BGL.glColor3f(0.7,0.0,0.0)
			BGL.glRectf(0,AreaDims[1]-4,AreaDims[0],AreaDims[1])
			BGL.glColor3f(0.9,0.9,0.9)

def event(evt, val):  # function that handles keyboard and mouse events
	global ScrollY, ScrollAreaY, Screen, activeObject, activeObjectMat0
	ScrollStep=25
	
	obj = Blender.Scene.GetCurrent().objects.active
	if(obj != None):
		try:
			try:
				if obj.getType() == "Text":
					cu=Blender.Curve.Get(obj.getData().getName())
					mat0=cu.materials[0]
				else:
					mat0 = (obj.getData(0,1).materials)[obj.activeMaterial - 1]
			except IndexError:
				mat0 = None
		except AttributeError:
			mat0 = None
	else: mat0 = None
	if obj != None and activeObject != obj:
		activeObject = obj
#		activeObjectMat0 = mat0
		Draw.Redraw(1)
	if(evt == Draw.WHEELDOWNMOUSE) or (evt == Draw.DOWNARROWKEY):
		AreaDims = Window.GetAreaSize()
		if True: #(AreaDims[1]+ScrollY < ScrollAreaY):
			ScrollY+=ScrollStep
			# drawGUI()
			Draw.Redraw(1)
	if(evt == Draw.WHEELUPMOUSE) or (evt == Draw.UPARROWKEY):
		AreaDims = Window.GetAreaSize()
		#print AreaDims[1], ScrollAreaY, ScrollY
		if (ScrollY>0):
			ScrollY-=ScrollStep
			# drawGUI()
			Draw.Redraw(1)
	if evt == Draw.QKEY:
		# stop = Draw.PupMenu("OK?%t|cancel export %x1")
		# if stop == 1:
		Draw.Exit()
	if Screen == 0 and CamFocus.val == 1:
		setFocus("D")
		Draw.Redraw()
	
	materialEvent(evt, val)



def setEnvMap(efilename):
	EnvFile.val = efilename

def setObstacleMap(filename):
	ObstacleMap.val = filename

def setApertureImage(filename):
	ApertureImg.val = filename

#def setIndigoPath(filename):
#	IndigoPath.val = filename
#	readWriteSettings(1)

def setIndigoOutPath(filename):
	IndigoOutPath.val = filename
	readWriteSettings(1)

def setIndigoAndExportDefault(filename):
	setIndigoPath(filename)
	defaultFile = os.path.join(Blender.sys.dirname(IndigoPath.val), "default.igs")
	exportStill(defaultFile)

def setIndigoAndExportFile(filename):
	setIndigoPath(filename)
	Blender.Window.FileSelector(exportStill, "Export", newFName('igs'))


def buttonEvt(evt):  # function that handles button events
	global Screen, EnFile, indigopath, lastScreen, subdivMeshes, selectedSection
	
	if evt == evtExport:
		readWriteSettings(1)
		if DefaultExport.val == 1:
			defaultFile = os.path.join(utils.getTempDir(), "default.igs")
			exportStill(defaultFile)
		else:
			Blender.Window.FileSelector(exportStill, "Export", newFName('igs'))
	
	if evt == evtExportAnim:
		SepMatFile.val=0
		ExportMeshs.val=1
		Blender.Window.FileSelector(exportAnim, "export animation", newFName('igs'))
	
	if evt == evtNoEvt:
		Draw.Redraw(1)
		readWriteSettings(1)
	
	if evt == evtchangesize:
		Scene.GetCurrent().getRenderingContext().imageSizeX(SizeX.val);
		Scene.GetCurrent().getRenderingContext().imageSizeY(SizeY.val);
		Draw.Redraw(1)
	
	if evt == openCamera:
		selectedSection = 'camera'
		Screen = 0
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openEnv:
		selectedSection = 'environment'
		Screen = 1
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openRSet:
		selectedSection = 'renderer'
		Screen = 2
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openSSet:
		selectedSection = 'system'
		Screen = 3
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openTmap:
		selectedSection = 'tonemap'
		Screen = 4
		drawGUI()
		Draw.Redraw(1)
	
	if evt == openMesh:
		selectedSection = 'mesh'

		try:
			obj = Blender.Object.GetSelected()[0]
			mesh=obj.getData()
			d=mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		
		matGui.val=0
		if meshGui.val:
			if Screen <> 5:
				lastScreen=Screen
			Screen = 6
			drawGUI()
			Draw.Redraw(1)
		else:
			Screen=lastScreen
			drawGUI()
			Draw.Redraw(1)
	
	if evt == evtSubdivMesh:
		obj = Blender.Object.GetSelected()[0]
		mesh=obj.getData()
		try:
			d=mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		if meshSubdivide.val:
			meshname=obj.getData(name_only=1)
			objecttype = obj.getType()
			if(objecttype == "Curve"):
				meshname +='_cu'
			imeshname = meshname.replace("<","-").replace(">","-")
			d[imeshname]=[MaxNumSubdiv.val, SubdivPixelTreshold.val, SubdivCurvatureTreshold.val, SubdivSmooth.val]
			mesh.properties['subdivision']=d
		else:
			meshname=obj.getData(name_only=1)
			objecttype = obj.getType()
			if(objecttype == "Curve"):
				meshname +='_cu'
			imeshname = meshname.replace("<","-").replace(">","-")
			if imeshname in d:
				del d[imeshname]
				mesh.properties['subdivision']=d
		readWriteSettings(1)
	
	if evt == evtUpdateSubdiv:
		obj = Blender.Object.GetSelected()[0]
		meshname=obj.getData(name_only=1)
		objecttype = obj.getType()
		if(objecttype == "Curve"):
			meshname +='_cu'
		imeshname = meshname.replace("<","-").replace(">","-")
		mesh=obj.getData()
		try:
			d=mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		if imeshname in d:
			d[imeshname]=[MaxNumSubdiv.val, SubdivPixelTreshold.val, SubdivCurvatureTreshold.val, SubdivSmooth.val]
			mesh=obj.getData()
			mesh.properties['subdivision']=d
		readWriteSettings(1)
	
	if evt == evtloadimg:
		Blender.Window.FileSelector(setEnvMap, "load environment map")
		readWriteSettings(1)
	
	if evt == evtloadobstacle:
		Blender.Window.FileSelector(setObstacleMap,"load obstacle map")
	
	if evt == evtloadaperture:
		Blender.Window.FileSelector(setApertureImage,"load aperture image")

#	if evt == evtSetIndigoPath:
#		Blender.Window.FileSelector(setIndigoPath, "Select Indigo",emptyFName())
	
	if evt == evtSetIndigoOutPath:
		Blender.Window.FileSelector(setIndigoOutPath, "Select Out Indigo", newFName('png'))
	
	if evt == evtFocusA:
		#setFocus("A")
		# if AutoFocus.val == 0:
		# 	AutoFocus.val = 1
		# else:
		# 	AutoFocus.val = 0
#		SelFocus.val=0;
#		CurFocus.val=0
		CamFocus.val=0
		readWriteSettings(1)
	
	if evt == evtFocusS:
#		if SelFocus.val == 0:
#			SelFocus.val = 1
		if setFocus("S"):
#			AutoFocus.val=0
#			CurFocus.val=0
			CamFocus.val=0
		else:
			SelFocus.val=0
		readWriteSettings(1)
	
	if evt == evtFocusC:
#		if CurFocus.val == 0:
#			CurFocus.val = 1
#		AutoFocus.val=0
#		SelFocus.val=0
		setFocus("C")
		CamFocus.val=0
		readWriteSettings(1)
	
	if evt == evtFocusD:
#		if CamFocus.val == 0:
#			CamFocus.val = 1
		setFocus("D")
		AutoFocus.val=0
#		SelFocus.val=0
#		CurFocus.val=0
		readWriteSettings(1)
	
	if evt == openMat:
		selectedSection = 'material'
		if matGui.val:
			if Screen <> 6:
				lastScreen = Screen
			Screen = 5
			drawGUI()
			Draw.Redraw(1)
		else:
			Screen = lastScreen
			drawGUI()
			Draw.Redraw(1)
	
	if evt == evtMatEmittTypeRGB:
		if matEmittTypeRGB == 1:
			matEmittTypeBB.val = 0
		else:
			matEmittTypeBB.val = 1
	
	if evt == evtMatEmittTypeBB:
		if matEmittTypeBB == 1:
			matEmittTypeRGB.val = 0
		else:
			matEmittTypeRGB.val = 1

	
	if evt == evtMatEmittIES:
		if matEmittIES == 1:
			matEmittScale.val=0
	
	if evt == evtMatEmittScale:
		if matEmittScale == 1:
			matEmittIES.val=0
	
	materialButtonEvt(evt)

def setFocus(target='A', val=0):
	global FocusDistance
	currentscene = Scene.GetCurrent()
	camObj = currentscene.getCurrentCamera()
	loc1 = camObj.getLocation()
	ws=WorldScale.val*(10**WorldScaleExp.val)
	if target == "S":
		try:
			selObj = Object.GetSelected()[0]
			loc2 = selObj.getLocation()
		except:
			Draw.PupMenu("Error%t|Select an Object")
			return 0
		FocusDistance.val = ws *((((loc1[0]-loc2[0])**2)+((loc1[1]-loc2[1])**2)+((loc1[2]-loc2[2])**2))**0.5)
		return 1
	if target == "C":
		loc2 = Window.GetCursorPos()
		FocusDistance.val = ws *((((loc1[0]-loc2[0])**2)+((loc1[1]-loc2[1])**2)+((loc1[2]-loc2[2])**2))**0.5)
		return 1
	if target == "D":
		loc2 = camObj.data.dofDist
		FocusDistance.val = ws * loc2
		return 1


# load settings
readWriteSettings(0)


Screen = 0
lastScreen = 0
Draw.Register(drawGUI, event, buttonEvt)


#if IndigoPath.val == '':
#	strg=str(getResourcesPath())
#	if strg != '':
#		setIndigoPath(strg)
#	else:
#		result=Draw.PupMenu("Indigo Path not set%t|Set|Abort")
#		if result == 1:
#			Blender.Window.FileSelector(setIndigoPath,"Select Indigo")



